<?php
require('application.php');
require('application.bootstrap.php');

$page = new Page('dashboard.php', 'master.php', array('theme' => $evp->settings->theme, 'version' => $evp->options->project_version));
$page->title(array(pick($evp->settings->site_name, $evp->options->project_name), 'Dashboard'));
$xs->restrictTo(array('groups' => array('Administrators', 'Managers', 'Clients', 'Users')), 'signin.php', array('return' => pass_create()), true);

if($xs->isMemberOf($xs->userData->id, 'Users') || $xs->isMemberOf($xs->userData->id, 'Clients')) redirect('admin/list.files.php');

//
//$evp->primeFileCache();

// Handle client settings
if($isClient) {
 // Get all files in the allowed folders
 $client_files = array();
 if(!empty($xs->userData->assigned_folder_id)) {
		foreach($xs->userData->assigned_folder_id as $fid) {
			$client_files_in = getArray("SELECT `file_id` FROM {$evp->options->prefix}folders_relationships WHERE `folder_id` = '{$fid}'");
			$client_files = array_merge($client_files, $client_files_in);
		}
 }
 $client_files_str = flattenByKey($client_files, 'file_id');
 $client_files_str = array_wrap($client_files_str, "'");
 $client_files_str = implode($client_files_str, ',');

 $client_string_id = "&& `id` IN({$client_files_str})";
 $client_string_file_id = "&& `file_id` IN({$client_files_str})";
 $client_string_file_ref = "&& `file_ref` IN({$client_files_str})";
} else {
 $client_string_id = '';
 $client_string_file_id = '';
 $client_string_file_ref = '1=1';
}

// Get the file list for the graphs
$folders_list = $evp->getFoldersAsTree(true);
$page->set('folders_list', $folders_list);

// Let's get the most popular file
if(!$isClient) {
	$popular_ref = getRow("SELECT `file_ref` AS id, COUNT(`uuid`) AS plays FROM {$evp->options->prefix}track_total_completes GROUP BY `file_ref` ORDER BY plays DESC LIMIT 1");
	if(!empty($popular_ref)) {
		$p = explode('::', $popular_ref->id, 2);
		if(empty($p[1])) $profile = '';
		else $profile = $p[1];

		$popular = $evp->getFile($popular_ref->id);
		if($popular) {
			$page->set(array(
				'popular' => $popular,
				'popular_profile' => $profile
			));
		}
	}
}

// What's the latest file?
if(!$isClient) {
	$latest_file_ref = grab(getRow("SELECT f.id FROM {$evp->options->prefix}files f WHERE f.parent_id IS NULL && f.id NOT IN(SELECT file_id FROM {$evp->options->prefix}folders_relationships) ORDER BY f.time DESC"), 'id');
	if($latest_file_ref) {
		$latest_file = $evp->getFile($latest_file_ref);
		$latest_file_stats = $evp->statistics->getStats($latest_file_ref);
		$latest_average_drop_off = $evp->statistics->averageDropOff($latest_file_ref);
	} else {
		$latest_average_drop_off = 0.7;
		$latest_file_stats = new Object();
		$latest_file = new Object();
		$latest_file_ref = '';
	}
}

$page->set(array(
	'latest_average_drop_off' => $latest_average_drop_off,
	'latest_file_stats' => $latest_file_stats,
	'latest_file' => $latest_file,
	'latest_file_ref' => $latest_file_ref
));

// Let's do the account stats
$total_starts = grab(getRow("SELECT COUNT(uuid) AS total FROM {$evp->options->prefix}track_total_starts WHERE {$client_string_file_ref}"), 'total');
$total_completions = grab(getRow("SELECT COUNT(uuid) AS total FROM {$evp->options->prefix}track_total_completes WHERE {$client_string_file_ref}"), 'total');
$total_completion_percentage = 0;
if($total_completions > 0) {
	$total_completion_percentage = (ceil(($total_completions / $total_starts) * 100));
}

$total_income = grab(getRow("SELECT SUM(`price`) AS total FROM {$evp->options->prefix}conversions WHERE `converted` = '1' {$client_string_file_id}"), 'total');

$account_stats = new Object(array(
	'total_files' => grab(getRow("SELECT COUNT(id) AS total FROM {$evp->options->prefix}files WHERE `parent_id` IS NULL"), 'total'),
	'total_folders' => sizeof($evp->getFolders(false)),
	'total_sites' => grab(getRow("SELECT COUNT(DISTINCT `referrer`) AS sites FROM {$evp->options->prefix}track_total_completes WHERE {$client_string_file_ref}"), 'sites'),
	'completion_percentage' => $total_completion_percentage,
	'total_completions' => $total_completions,
	'total_starts' => $total_starts,
	'total_income' => $total_income
));

if($isClient) {
 $account_stats->total_files = sizeof($client_files);
 $account_stats->total_folders = sizeof($xs->userData->assigned_folder_id);
}

$page->set('account_stats', $account_stats);
$page->set('isClient', $isClient);

$page->css('styles/style.page.files.statistics.css');
$page->js('https://www.gstatic.com/charts/loader.js');

$page->js('scripts/libraries/jquery-1.5.1.min.js');
$page->js('jQuery.noConflict();');
$page->js(url('player/assets/jquery.notifications.js'));
$page->css(url('player/assets/colorbox-1.3.18/style-3/colorbox.css?v=8-5')); // For the desktop app download

$page->addEvent('onTemplateMerge', 'onTemplateMerge');
echo $page->parse();
?>
