/*
 * Example: <a data-trigger="files-delete" data-files-delete-options="{'xhr': 'http://mysite.com/xhr.php', 'file_id': 'some_id', 'profile': '', 'showConfirm': true, 'spin': '! tr'}">Delete</a>
 */

Delegator.register('click', {
	'files-delete': {
		'require': ['xhr', 'xhr_username', 'xhr_password', 'file_id'],
		'handler': function(event, link, api) {
			var file_id = api.get('file_id');
			var file_name = api.get('file_name');
			var profile = api.get('profile');
			var url = api.get('xhr');
			var api_username = api.get('xhr_username');
			var api_password = api.get('xhr_password');
			var showConfirm = api.get('showConfirm');
			var confirmTitle = api.get('confirmTitle') || 'Files';
			var confirmMessage = api.get('confirmMessage') || 'Do you want to delete this file?';
			var confirmFailed = api.get('confirmFailed') || 'This file could not be deleted.';
			var spinnerMessage = api.get('spinnerMessage') || 'Deleting...';
			
			if(file_name) {
				confirmTitle = confirmTitle.substitute({'name': file_name});
				confirmMessage = confirmMessage.substitute({'name': file_name});
			}
			
			var del = function() {
				if(api.get('spin')) {
					var spinner_element = link.getElement(api.get('spin'));
					spinner_element.spin({'msg': spinnerMessage});
				}
				
				var xhr = new Request.JSON({
					'url': url,
					'onSuccess': function() {
						if(spinner_element) {
							spinner_element.unspin();
							spinner_element.kill();
						}
					},
					'onFailure': function() {
						if(spinner_element) spinner_element.unspin();
						if(showConfirm) {
							StickyWin.alert(confirmTitle, confirmFailed);
						}
					}
				});
				xhr.post({'method': 'files-delete', 'file_id': file_id, 'profile': profile, 'username': api_username, 'password': api_password});
			}
			
			if(showConfirm) {
				StickyWin.confirm(confirmTitle, confirmMessage, del);
			} else {
				del();
			}
		}
	}
});