/*
 * Example: <input type="checkbox" data-trigger="checkSwitch" data-checkswitch-options="{'targets': '#list_files input[type=checkbox]'}" />
 * The selector is ran globally - this can possibly be improved for speed
 */

Delegator.register('click', {
	'checkSwitch': {
		'require': ['targets'],
		'handler': function(event, link, api) {
			var targets = $$(api.get('targets'));
			var f = targets[0];
			if (targets.length) targets.set('checked', !f.get('checked')).fireEvent('change');
			else api.warn('There were no inputs found to check.');
		}
	}
});

/*
 * Example:
 * 
 */
Behavior.addGlobalFilter('Grid', {
	setup: function(element, api) {
		var columns = api.get('columns') || 'auto';
		var padding = api.get('padding') || '';
		var grid = new Grid(element, {'columns': columns, 'padding': padding});
		$(element).store('Grid', grid);
		return grid;
	}
});

/*
 * Example:
 * 
 */
Behavior.addGlobalFilter('ColourPicker', {
	setup: function(element, api) {
		var m = new SimpleRainbow(element);
		$(element).store('ColourPicker', m);
		return m;
	}
});

/*
 * Example:
 * 
 */
Behavior.addGlobalFilter('InputMask', {
	setup: function(element, api) {
		/*
		var mask = new Meio.Mask(JSON.decode($(element).get('data-meiomask-options')));
		mask.link($(element));
		return mask;
		*/
		$(element).meiomask($(element).get('data-meiomask'), JSON.decode($(element).get('data-meiomask-options')));
	}
});

/*
 * Example:
 * 
 */
Behavior.addGlobalFilter('RadioGroup', {
	setup: function(element, api) {
		var width = api.get('width') || '100px';
		var tie = api.get('tie') || true;
		var group = new RadioGroup(element, {'width': width});
		group.addEvent('onCheck', function() {
			var a = ['onCheck'];
			for(i = 0; i < arguments.length; i++) a.push(arguments[i]);
			element.fireEvent.apply(element, a);
		});
		$(element).store('RadioGroup', group);
		if($('thisFile') && tie !== 'false') {
			var el_name = element.getElement('div').getData('name');
			var tie = new EvpTieToElement(element, el_name, 'thisFile');
			element.getElements('div').addEvent('click', function() {
				tie.set(this.getData('value'), false);
			});
			tie.set($('thisFile').retrieve('data')[el_name], false);
			if(typeof $('thisFile').retrieve('data')[el_name] !== 'undefined') {
				var total_elements = element.getParent('div').getElements('input[type="radio"]');
				var idx = 0;
				total_elements.each(function(item, index) {
					var yes = (item.get('value') == $('thisFile').retrieve('data')[el_name]);
					if(yes) idx = index;
				});
			} else {
				var idx = element.getParent('div').getElement('input[type="radio"]');
			}
			group.select(idx);
			$(element).store('Tie', tie);
		}
		return group;
	}
});

/*
 * Example:
 * 
 */
Behavior.addGlobalFilter('SelectBox', {
	setup: function(element, api) {
		var useTie = !((api.get('tie') || true) == 'false');
		var selectWrap = new Element('div', {'class': 'selectWrapper'}).wraps($(element));
		var wrapperHeight = api.get('wrapperHeight') || 194;
		var sel = new mooSelecta({
			'wrapperHeight': wrapperHeight,
			'selector': $$(element).map(function(el) {
				return (el.getProperty('size') ? false : el)
			}),
			'wrapperWidthAdjustment': 0,
			'triggerPadding': 24,
			'triggerBeforeImage': 'dummy',
			'triggerBeforeImageWidth': 2,
			'triggerBeforeImageHeight': 36
  });
		$(element).store('SelectBox', sel);

		if($('thisFile') && !!useTie) {
			var tie = new EvpTieToElement(element, element.get('name'), 'thisFile');
			$(element).addEvent('change', function() {
				tie.set(this.get('value'));
			});
			var key = $(element).get('name');
			tie.set($('thisFile').retrieve('data')[key]);
			$(element).store('Tie', tie);
		}
		return sel;
	}
});

/*
 * Example:
 * 
 */
Behavior.addGlobalFilter('iPhoneCheckbox', {
	setup: function(element, api) {
		var checkedLabel = api.get('checkedLabel') || 'YES';
		var uncheckedLabel = api.get('uncheckedLabel') || 'NO';
		var cb = new IPhoneCheckboxes({
			'elements': $(element),
			'checkedLabel': checkedLabel,
			'uncheckedLabel': uncheckedLabel
		});
		$(element).store('iPhoneCheckbox', cb);
		return cb;
	}
});

/*
 * Example:
 * 
 */
Behavior.addGlobalFilter('DeviantMenu', {
	setup: function(element, api) {
		var menu = new deviantMenu(element);
		$(element).store('deviantMenu', menu);
		return menu;
	}
});

/*
 * Example:
 * 
 */
Behavior.addGlobalFilter('DatePicker', {
	setup: function(element, api) {
		
		var dp = new DatePicker(element, {
			'format': api.get('format') || '%Y-%m-%d',
			'pickerClass': 'datepicker datepicker_evp',
			'minDate': api.get('minDate'),
			'maxDate': api.get('maxDate'),
			'allowEmpty': api.get('allowEmpty') || false,
			'timePicker': api.get('timePicker') || false,
			'useFadeInOut': false
		});

		$(element).store('DatePicker', dp);
		return dp;
	}
});

/*
 * Example:
 * 
 */
Behavior.addGlobalFilter('Form', {
	defaults: {
		// Form options
		test: 'yeah',
		// FormValidator options
		useTitles: true,
		scrollToErrorsOnSubmit: true,
		scrollToErrorsOnBlur: false,
		scrollToErrorsOnChange: false,
		ignoreHidden: true,
		ignoreDisabled: true,
		useTitles: false,
		evaluateOnSubmit: true,
		evaluateFieldsOnBlur: true,
		evaluateFieldsOnChange: true,
		serial: true,
		stopOnFailure: true
	},
	setup: function(element, api) {
		// Create the new Form
		var form = new SexyForm(element, Object.cleanValues(api.getAs({
			'test': String
		})));
		
		$(element).store('SexyForm', form);
		
		// Instantiate the form validator
		var validator = element.retrieve('validator');
		if (!validator) {
			validator = new Form.Validator.Inline(element, 
				Object.cleanValues(
					api.getAs({
						useTitles: Boolean,
						scrollToErrorsOnSubmit: Boolean,
						scrollToErrorsOnBlur: Boolean,
						scrollToErrorsOnChange: Boolean,
						ignoreHidden: Boolean,
						ignoreDisabled: Boolean,
						useTitles: Boolean,
						evaluateOnSubmit: Boolean,
						evaluateFieldsOnBlur: Boolean,
						evaluateFieldsOnChange: Boolean,
						serial: Boolean,
						stopOnFailure: Boolean
					})
				)
			);
		}
	}
});

/*
 * Example: <a href="#" data-behavior="Button" data-button-width="200" data-button-icon="alert" data-button-iconpos="top" data-button-type="save">Testing</a>
 * Hover styles are automatically added and would be .moobutton.moobutton_{type} {} and .moobutton.moobutton_{type}.moobutton_hover {} for instance
 */
Behavior.addGlobalFilter('Button', {
	setup: function(element, api) {
		var padding = api.get('padding') || '';
		var type = api.get('type') || 'default', st;
		var width = api.get('width') || $(element).getParent().getSize().x || '400px';
		var icon = api.get('icon');
		var iconpos = api.get('iconpos') || 'left';
		$(element).addClass('moobutton_'+type);
		if(iconpos == 'notext') width = 16;
		else if(typeof icon !== 'undefined') $(element).addClass('moobutton_icon_'+iconpos);
		st = {'width': width, 'height': 'auto', 'styles': {
			'padding': padding
		}, 'icon': (icon ? 'ui-icon ui-icon-pos-'+iconpos+' ui-icon-'+icon : false)};
		var mb = new MooButton(element, st);
		element.store('MooButton', mb);
		return mb;
	}
});

(function() {
	var createTip = function(link, point, opts) {
		var tip = new Tips.Pointy(link, {
			className: opts.className,
			pointyTipOptions: {
				destroyOnClose: false,
				width: opts.width || 250,
				point: point.toInt()
			}
		});
		return tip;
	};
	
	Behavior.addGlobalFilter('BetterTip', function(element){
		var point = element.getData('tip-direction', 12);
		var width = element.getData('tip-width');
		var tip = createTip(element, point, {'width': width, 'className': element.getData('tip-class')});
		//destroy the tips on cleanup
		this.markForCleanup(element, function(){
			tip.destroy();
		});
		return tip;
	});
})();