Element.implement({
	kill: function(callback) {
		if(Browser.Engine.trident && Browser.Engine.version <= 8) { // Remove the fade - causes stupid problems in IE
			$(this).destroy();
			if(callback) callback();
		} else {
			var fx = new Fx.Tween(this).addEvent('onComplete', function() {
				this.destroy();
				if(callback) callback();
			}.bind(this));
			fx.start('opacity', 0);
		}
	},
	disappear: function(callback) {
		var fx = new Fx.Tween(this).addEvent('onComplete', function() {
			this.setStyle('display', 'none');
			if(callback) callback();
		}.bind(this));
		fx.start('opacity', 0);
	},
	reappear: function(callback) {
		var fx = new Fx.Tween(this).addEvent('onComplete', function() {
			if(callback) callback();
		}.bind(this));
		this.setStyle('display', 'block');
		fx.start('opacity', 1);
	},
	removeStyle: function(ele) {
		regex = new RegExp('[~;\\s]' + ele + '.*?[;~]', "i");
		style = this.get('style')+';';
		this.set('style', style.replace(regex, ''));
	},
	dissolve: function(options) { // Remove dissolve and reveal functionality for IE
		if(Browser.Engine.trident && Browser.Engine.version <= 8) {
			this.setStyle('display', 'none');
		} else {
			this.get('reveal').setOptions(options).dissolve();
		}
		return this;
	},
	reveal: function(options) {
		if(Browser.Engine.trident && Browser.Engine.version <= 8) {
			this.setStyle('display', 'block');
		} else {
			this.get('reveal').setOptions(options).reveal();
		}
		return this;
	}
});

Element.implement({
	removeClasses: function (classNames) {
		this.className = this.className.replace(new RegExp("\\b(" + classNames.replace(/\s+/g, "|") + ")\\b", "g"), " ").clean();
		return this;
	}
});

String.implement({
	makeSeed: function() {
		var value = this.clean().toLowerCase().replace(/\s+?/g, '-').replace(/\&/g, 'and').replace(/[^a-zA-Z0-9-\s]/g, '');
		if(value.length > 250) value = value.substr(0, 250);
		return value;
	},
	wordwrap: function(int_width, str_break, cut) {
		var m = ((arguments.length >= 1) ? arguments[0] : 75);
		var b = ((arguments.length >= 2) ? arguments[1] : "\n");
		var c = ((arguments.length >= 3) ? arguments[2] : false);
		
		var i, j, l, s, r;
		
		var str = this+'';
		
		if (m < 1) {
		    return str;
		}
		
		for (i = -1, l = (r = str.split(/\r\n|\n|\r/)).length; ++i < l; r[i] += s) {
		    for (s = r[i], r[i] = ""; s.length > m; r[i] += s.slice(0, j) + ((s = s.slice(j)).length ? b : "")) {
		        j = c == 2 || (j = s.slice(0, m + 1).match(/\S*(\s)?$/))[1] ? m : j.input.length - j[0].length || c == 1 && m || j.input.length + (j = s.slice(m).match(/^\S*/)).input.length;
		    }
		}
		
		return r.join("\n");
	}
});

window.url = function(url) {
	window.location = url;
}

SexyForm = new Class({
	Implements: [Options],
	options: {
		'test': 'default'
	},
	'initialize': function(form, options) {
		this.setOptions(options);
		this.form = $(form);
		this.form.addClass('styled');
		
		this.dropdowns();
	},
	'dropdowns': function() {
		this.form.getElements('select').each(function(el, idx) {
			if(!el.retrieve('FormDropdown')) {
				var d = new Form.Dropdown(el, {});
				el.store('FormDropdown', d);
			}
		});
	}
});

Loading = new Class({
	Implements: [Options, Events],
	options: {
		'spin': false, // Element to spin
		'spin-msg': 'Loading...', // Message
		'element': false, // Element to animate
		'animation': 'slide', // slide, fade
		'onStart': function(){},
		'onFinish': function(){}
	},
	'initialize': function(options) {
		this.setOptions(options);
		
		if(this.options.animation == 'fade' && this.options.element) {
			this.fx = new Fx.Tween(this.options.element);
		} else if(this.options.animation == 'slide' && this.options.element) {
			this.fx = new Fx.Reveal(this.options.element);
		}
	},
	'start': function() {
		if(this.options.spin) {
			$(this.options.spin).spin({'msg': this.options['spin-msg']});
		}
		
		var klass = this;
		
		if(this.options.animation == 'fade' && this.options.element) {
			this.fx.start('opacity', 0).chain(function() {
				klass.fireEvent('onFinish');
			});
		} else if(this.options.animation == 'slide' && this.options.element) {
			this.fx.dissolve().chain(function() {
				klass.fireEvent('onFinish');
			});
		}
	},
	'stop': function() {
		if(this.options.spin) {
			$(this.options.spin).unspin();
		}
		this.fireEvent('onFinish');
	}
});

WidgetisedArea = new Class({
	Implements: [Options, Events],
	options: {
		'animate': true,
		'spin': false, // Element to spin
		'spin-msg': 'Loading...',
		'update': false, // Update an Element with the contents?
		'update-method': 'update', // update, replace, append
		'animation': 'fade', // fade, slide
		'hideInitial': false,
		'onBegin': function(){},
		'onFinish': function(){},
		'onSuccess': function(){},
		'onFailed': function(){}
	},
	'initialize': function(widget_href, widget_data, options) {
		this.setOptions(options);
		
		if(Browser.Engine.trident && Browser.Engine.version <= 8) {
			this.options.animate = false;
			this.options.spin = false;
		}
		
		this.animate = this.options.animate;
		if(this.options.spin) {
			this.options.spin = $(this.options.spin);
		}
		this.widget_url = new URI(widget_href);
		this.widget_url.setData(widget_data, true);
		
		// Make the request
		var opts = {
			'url': this.widget_url.toString(),
			'method': 'get',
			'link': 'cancel',
			'evalScripts': false,
			'evalResponse': false,
			'noCache': true,
			'onRequest': this.begin.bind(this),
			'onSuccess': this.success.bind(this),
			'onComplete': this.finish.bind(this)
		};
		
		var klass = this;
		
		if(this.options.update) {
			var p = function(a, javascript_code) {
				
				var response = $(klass.response);
				
				if(klass.options['update-method'] == 'update') {
					$(klass.options.update).empty().adopt(response);
				} else if(klass.options['update-method'] == 'append') {
					response.inject($(klass.options.update), 'bottom');
				} else if(klass.options['update-method'] == 'replace') {
					response.inject($(klass.options.update), 'after');
					$(klass.options.update).destroy();
				}
				
				if(javascript_code) Browser.exec(javascript_code);
				
				new Behavior().apply(response);
				new Delegator().attach(response);
			};
			
			var wrap = $(this.options.update);//new Element('div').wraps($(this.options.update));
			
			if(klass.options.animation == 'fade') {
				this.fx = new Fx.Tween(wrap);
			} else if(klass.options.animation == 'slide') {
				this.fx = new Fx.Reveal(wrap);
			}
			
			if(klass.animate == false && klass.options.hideInitial) {
				$(klass.options.update).setStyle('opacity', 0);
			}
			
			this.addEvent('onSuccess', function(a, b, c, d) {
				var temp = new Element('div').set('html', c);
				klass.response = temp;

				if(klass.animate !== false) {
					if(klass.options.animation == 'fade') {
						klass.fx.start('opacity', 0).chain(function() {
							p(a, d);
							klass.fx.start('opacity', 1);
						});
					} else if(klass.options.animation == 'slide') {
						klass.fx.dissolve().chain(function() {
							p(a, d);
							klass.fx.reveal();
						});
					}
				} else {
					$(klass.options.update).setStyle('opacity', 1);
					p(a, d);
				}
			});
		}
		
		this.request = new Request.HTML(opts);
	},
	'start': function(data, animate) {
		if(typeof animate !== 'undefined') this.animate = animate;
		this.request.get(data);
	},
	'begin': function() {
		if(this.options.spin) {
			this.options.spin.spin({'msg': this.options['spin-msg']});
		}
		this.fireEvent('onBegin', arguments);
	},
	'finish': function() {
		if(this.options.spin) {
			this.options.spin.unspin();
		}
		this.fireEvent('onFinish', arguments);
	},
	'success': function() {
		this.fireEvent('onSuccess', arguments);
	},
	'failed': function() {
		this.fireEvent('onFailed', arguments);
	}
});

Grid = new Class({
	Implements: [Options],
	options: {
		'columns': 3,
		'padding': ''
	},
	'initialize': function(grid, options) {
		this.setOptions(options);
		this.grid = $(grid);
		this.grid_original = this.grid.clone();
		this.grid.addClass('grid-wrap');
		if(this.options.columns == 'auto') {
			this.options.columns = this.grid.getChildren().length;
		}
		this.columns = [];
		this.build();
	},
	'build': function() {
		var total_width = this.grid.getSize().x;
		var width_offset = 0;
		var col_widths = [];
		var i = this.options.columns;
		for(i; i > 0; i--) {
			var w = Math.floor((total_width / this.options.columns) + width_offset);
			this.getItems(i).each(function(item) {
				var cw = item.getData('column-width');
				if(cw) {
					w = cw.toInt();
					width_offset = width_offset + w;
				}
			});
			col_widths[i] = w;
		}
		i = this.options.columns; // Reset to go over them all again
		for(i; i > 0; i--) {
			var col = new Element('div', {'class': 'grid-column grid-column-'+i, 'styles': {'width': col_widths[i]}});
			var target;
			if(this.options.padding !== '') {
				var wrap_width = col_widths[i].toInt() - (this.options.padding.toInt() * 2);
				var wrap = new Element('div', {'class': 'grid-column-wrap', 'styles': {'width': wrap_width, 'margin-left': this.options.padding.toInt(), 'margin-right': this.options.padding.toInt()}});
				col.adopt(wrap);
				target = wrap;
			} else {
				target = col;
			}
			this.columns[i - 1] = col;
			this.getItems(i).each(function(item) {
				target.adopt(item);
			}, this);
		}
		for(i = 0; this.columns[i]; i++) { // We're injecting this way to reverse them to match HTML order!
			this.columns[i].inject(this.grid);
		}
	},
	'getItems': function(i) {
		var num = Math.ceil(this.grid_original.getChildren().length / this.options.columns);
		var filtered = this.grid.getChildren().splice(num * (i - 1), num);
		return filtered;
	}
});