var IPhoneCheckboxes = new Class({
	Implements: [Options, Events],

	options: {
		checkedLabel: 'YES',
		uncheckedLabel: 'NO',
		background: '#fff',
		containerClass: 'iPhoneCheckContainer',
		labelOnClass: 'iPhoneCheckLabelOn',
		labelOffClass: 'iPhoneCheckLabelOff',
		handleClass: 'iPhoneCheckHandle',
		handleBGClass: 'iPhoneCheckHandleBG',
		handleSliderClass: 'iPhoneCheckHandleSlider',
		elements: 'input[type=checkbox]',
		onEnable: $empty,
		onDisable: $empty
	},

	//initialization
	initialize: function(options) {
		//set options
		this.setOptions(options);
		//elements
		this.elements = $$(this.options.elements);
		//observe checkboxes
		this.elements.each(function(el) {
			this.observe(el);
		},this);
	},

	//a method that does whatever you want
	observe: function(el) {
		//turn off opacity
		el.set('opacity',0);
		//create wrapper div
		var wrap = new Element('div',{
			'class': this.options.containerClass
		}).inject(el, 'after');
		el.store('iPhoneSwitchWrap', wrap);
		//inject this checkbox into it
		el.inject(wrap);
		//now create subsquent divs and labels
		var handle = new Element('div',{'class':this.options.handleClass}).inject(wrap);
		var handlebg = new Element('div',{'class':this.options.handleBGClass,'style':this.options.background}).inject(handle);
		var handleSlider = new Element('div',{'class':this.options.handleSliderClass}).inject(handle);
		var offLabel = new Element('label',{'class':this.options.labelOffClass,text:this.options.uncheckedLabel}).inject(wrap);
		var onLabel = new Element('label',{'class':this.options.labelOnClass,text:this.options.checkedLabel}).inject(wrap);
		var rightSide = wrap.getSize().x;
		if(rightSide > 0) rightSide = rightSide - 43;
		else rightSide = 46;
		//fx instances
		el.offFx = new Fx.Tween(offLabel,{'property':'opacity','duration':200});
		el.onFx = new Fx.Tween(onLabel,{'property':'opacity','duration':200});
		el.t = new Fx.Tween(handle,{
			duration: 100,
			'property': 'left'
		});
		var klass = this;
		//mouseup / event listening
		wrap.addEvent('click',function() {
			var is_onstate = !el.get('checked'); //originally 0
			var new_left = (is_onstate ? rightSide : 0);
			var bg_left = (is_onstate ? 34 : 0);
			handlebg.hide();
			el.t.start(new_left).chain(function() {
				handlebg.setStyle('left',bg_left).show();
			});
			//label animations
			if(is_onstate) {
				el.offFx.start(0);
				el.onFx.start(1);
				klass.fireEvent('onEnable', el);
				klass.fireEvent('onManualEnable', el);
			}	else {
				el.offFx.start(1);
				el.onFx.start(0);
				klass.fireEvent('onDisable', el);
				klass.fireEvent('onManualDisable', el);
			}
			//set checked
			el.set('checked', is_onstate).fireEvent('change');
		});
		//initial load
		if(el.get('checked')){
			offLabel.setStyle('opacity',0);
			onLabel.setStyle('opacity',1);
			handle.setStyle('left',rightSide);
			handlebg.setStyle('left',34);
			klass.fireEvent('onEnable', el);
			el.fireEvent('change');
		} else {
			el.set('checked', false);
			onLabel.setStyle('opacity',0);
			handlebg.setStyle('left',0);
			klass.fireEvent('onDisable', el);
			el.fireEvent('change');
		}
	}
});