//	Copyright (c) 2009 Marc Fowler (http://dfr.me)
//	
//	Permission is hereby granted, free of charge, to any person
//	obtaining a copy of this software and associated documentation
//	files (the "Software"), to deal in the Software without
//	restriction, including without limitation the rights to use,
//	copy, modify, merge, publish, distribute, sublicense, and/or sell
//	copies of the Software, and to permit persons to whom the
//	Software is furnished to do so, subject to the following
//	conditions:
//	
//	The above copyright notice and this permission notice shall be
//	included in all copies or substantial portions of the Software.
//	
//	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
//	EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
//	OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
//	NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
//	HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
//	WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
//	FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
//	OTHER DEALINGS IN THE SOFTWARE.

Element.implement({
	hasEvent: function(eventType,fn) {
		//get the element's events
		var myEvents = this.retrieve('events');
		//can we shoot this down?
		return myEvents && myEvents[eventType] && (fn == undefined || myEvents[eventType].keys.contains(fn));
	}
});

var MooButton = new Class({
	Implements: [Options],
	options: {
		'height': 38,
		'width': 240,
		'html': false,
		'icon': false, // Can be ui-icon-delete for instance
		'styles': {
			'text-align': 'center',
			'cursor': 'pointer',
			'color': '#fff'
		},
		'span_styles': {
			//
		},
		'classes': {
			'normal': 'moobutton',
			'hover': 'moobutton_hover',
			'mousedown': 'moobutton_mousedown',
			'span': 'moobutton_text'
		}
	},
	initialize: function(el, options) {
		this.setOptions(options);
		el = $(el);
		if(!el) return false;
		if($type(this.options.height) == 'string' && this.options.height !== 'auto') this.options.height = this.options.height.toInt();
		if(this.options.width !== '100%') if($type(this.options.width) == 'string') this.options.width = this.options.width.toInt();
		var klass = this;
		if(this.options.icon && this.options.icon.test('notext')) {
			//
		} else {
			var content = new Element('span', {'class': klass.options.classes.span, 'text': el.get('text') || el.get('value')});
		}
		
		var styles = klass.options.styles;
		styles.height = klass.options.height;
		styles.width = klass.options.width;
		if(klass.options.height != 'auto') styles['line-height'] = klass.options.height;
		
		this.button = new Element('div', {
			'id': el.get('id'),
			'class': klass.options.classes.normal + ' ' + el.getProperty('class'),
			'styles': styles,
			'events': {
				'mousedown': function() {
					this.addClass(klass.options.classes.mousedown);
				},
				'mouseup': function() {
					this.removeClass(klass.options.classes.mousedown);
				},
				'mouseenter': function() {
					this.addClass(klass.options.classes.hover);
				},
				'mouseleave': function() {
					this.removeClass(klass.options.classes.hover).removeClass(klass.options.classes.mousedown);
				},
				'selectstart': $lambda(false)
			}
		});
		if(this.options.html == true) {
			var content = new Element('span', {'class': klass.options.classes.span, 'html': el.get('html') || el.get('text') || el.get('value')});
		}
		if(this.options.icon !== false) {
			var icon = new Element('span', {'class': this.options.icon});
			this.button.adopt(icon);
		}
		if(content) this.button.adopt(content.setStyles($merge({}, klass.options.span_styles)));
		this.button.replaces(el);
		this.button.cloneEvents(el);
		if(!this.button.hasEvent('click') && this.button.getParent('form') && (el.get('type') == 'submit'/*|| el.get('tag') == 'a'*/)) {
			this.button.addEvent('click', function() {
				this.getParent('form').submit();
			});
		}
		this.button.unselectable = 'on';
		
		if(el.get('href')) {
			this.button.addEvent('click', function() {
				if(el.getProperty('target') == '_blank') {
					var n = new Date().getTime();
					window.open(el.get('href'), 'evp'+n);
				} else {
					window.location = el.get('href');
				}
			});
		}
		
		var useTips = false;
		if(el.get('title')) {
			var useTips = true;
			this.button.set('title', el.get('title'));
		}
		if(el.get('rel')) {
			var useTips = true;
			this.button.set('rel', el.get('rel'));
		}
		
		if(useTips) {
			new Tips.Pointy(this.button, {
				'pointyTipOptions': {
					'point': parseInt(el.getData('tip-point') || 3),
					'width': parseInt(el.getData('tip-width') || 150)
				}
			});
		}
		
	}
});