var ThumbnailCycle = new Class({
	Implements: [Options],
	options: {
		'length': 20,
		'width': 100,
		'height': 100,
		'speed': 1000,
		'className': 'thumbnail-cycle',
		'noprefix': false, // should append thumbnail- to base
		'extension': 'jpg' // could be png
	},
	'initialize': function(el, options) {
		this.setOptions(options);
		this.el = $(el);
		
		this.el.store('ThumbnailCycle', this);
		
		var klass = this;
		
		this.setup = false;
		this.shouldCycle = false;
		
		// Let's syle the wrapper
		$(el).setStyles({
			'position': 'relative',
			'width': parseInt(this.options.width),
			'height': parseInt(this.options.height)
		}).addClass(this.options.className);
		
		// Get ready
		this.imgs = [];
		this.timer = false;
		this.currentImg = 0;
		
		var prefix = (this.options.noprefix ? '' : '-thumbnail-');
		
		// Let's make the very first one
		var base = this.options.base.split('.');
		base.pop();
		base = base.join('.');
		
		if(this.options.noprefix) base = base.substr(0, base.length - 1);

		var src = base+prefix+'0.'+this.options.extension;//+'?v=1';
		var img = new Element('img', {
			'src': src,
			'width': this.options.width,
			'height': this.options.height,
			'styles': {
				'display': 'block',
				'position': 'absolute',
				'top': 0,
				'left': 0
			}
		});
		this.imgs.push(img);
		$(this.el).adopt(img);
		
		// Add the event
		$(el).addEvents({
			'mouseenter': function() {
				klass.start();
			},
			'mouseleave': function() {
				klass.stop();
			}
		});
		
	},
	'build': function(callback) {
		if(this.setup) {
			if(typeof callback == 'function' && this.shouldCycle === true) callback();
			return;
		}
		
		var all_imgs = [];
		var base = this.options.base.split('.');
		base.pop();
		base = base.join('.');
		
		var x = 1; // Start from 1, because 0 is already done!
		
		var prefix = (this.options.noprefix ? '' : '-thumbnail-');
		
		if(this.options.noprefix) base = base.substr(0, base.length - 1);
		
		while(x < this.options.length) {
			var src = base+prefix+x+'.'+this.options.extension;//+'?v=1';
			all_imgs.push(src);
			
			var img = new Element('img', {
				'src': src,
				'width': this.options.width,
				'height': this.options.height,
				'styles': {
					'display': 'none',
					'position': 'absolute',
					'top': 0,
					'left': 0
				}
			});
			
			$(this.el).adopt(img);
			
			this.imgs.push(img);
			
			x++;
		}
		
		new Asset.images(all_imgs, {
			'onComplete': function() {
				if(typeof callback == 'function' && this.shouldCycle === true) callback();
			}.bind(this)
		});
		
		this.setup = true;
	},
	'start': function() {
		var klass = this;
		klass.shouldCycle = true;
		this.build(function() {
			clearInterval(this.timer);
			this.timer = setInterval(function() {
				klass.currentImg++;
				$$(klass.imgs).setStyles({'display': 'none'});
				if(!klass.imgs[klass.currentImg]) klass.reset();
				else $(klass.imgs[klass.currentImg]).setStyles({'display': 'block'});
			}, this.options.speed);
		}.bind(this));
	},
	'stop': function() {
		this.shouldCycle = false;
		clearInterval(this.timer);
		this.reset();
	},
	'reset': function() {
		this.currentImg = 0;
		$$(this.imgs).setStyles({'display': 'none'});
		$(this.imgs[0]).setStyles({'display': 'block'});
	}
});