<?php
require('application.php');
require('application.bootstrap.php');

$page = new Page('upload.php', 'master.php', array('theme' => $evp->settings->theme, 'version' => $evp->options->project_version));
$page->title(array(pick($evp->settings->site_name, $evp->options->project_name), 'Upload'));
$xs->restrictTo(array('groups' => array('Administrators', 'Managers', 'Clients')), 'signin.php', array('return' => pass_create()), true);

// Update the cache, and then grab a list of every file which isn't a YouTube vid
//$evp->primeFileCache();
$existing_files = flattenByKey(getArray("SELECT `file_name` FROM {$evp->options->prefix}files WHERE `source` != '3'"), 'file_name');
if(empty($existing_files)) $existing_files = array();
$page->set('existing_files', $existing_files);

$supportedVideoFiles = array();
$supportedAudioFiles = array();

if($evp->local) {
	if($evp->local->canUpload()) $page->set('localCanWrite', true);
	$maximumSize = $evp->local->prepareUpload();
	$page->set('localMaximumSize', $maximumSize);
	$supportedVideoFiles = $evp->local->options->supported_video_files;
	$supportedAudioFiles = $evp->local->options->supported_audio_files;
}

if($evp->s3) {
	$bucket = $evp->settings->s3_bucket;
	$accessKeyId = $evp->settings->s3_accesskey;
	$secret = $evp->settings->s3_secretkey;

	$acl = ($evp->cloudfront ? 'public-read' : 'private');

	// prepare policy
	$policy = base64_encode(json_encode(array(
		'expiration' => date('Y-m-d\TH:i:s.000\Z', strtotime('+1 day')),
		'conditions' => array(
			array('bucket' => $bucket),
			array('acl' => $acl),//'private'),//'public-read'),
			array('starts-with', '$key', ''),
			array('eq', '$x-amz-meta-via', 'web'),
			array('starts-with', '$Content-Disposition', ''),
			array('starts-with', '$Content-Type', ''),
			array('success_action_status' => '201'),
			array('starts-with', '$name', ''),
			array('starts-with', '$Filename', ''),
		)
	)));

	// sign policy
	$signature = base64_encode(hash_hmac('sha1', $policy, $secret, true));

	// Expose
	$page->set('signature', $signature);
	$page->set('bucket', $bucket);
	$page->set('policy', $policy);
	$page->set('acl', $acl);

	$supportedVideoFiles = array_merge($supportedVideoFiles, $evp->s3->options->supported_video_files);
	$supportedAudioFiles = array_merge($supportedAudioFiles, $evp->s3->options->supported_audio_files);

	$page->set('s3MaximumSize', '2000mb');
}

$page->set('supportedVideoFiles', array_unique($supportedVideoFiles));
$page->set('supportedAudioFiles', array_unique($supportedAudioFiles));

// This is for the system which locates the license key
// This is the main user, so the first administrator we find
$primary_username = grab(getRow("SELECT `username` FROM {$evp->options->prefix}users WHERE `id` = '1'"), 'username');
$page->set('primary_username', $primary_username);

// Any special things this page needs?
$page->js('scripts/libraries/jquery-1.5.1.min.js');
$page->js('jQuery.noConflict();');
$page->js('scripts/libraries/plupload/plupload.js');
$page->js('scripts/libraries/plupload/plupload.flash.js');
$page->css('styles/style.page.upload.css');
$page->css(url('player/assets/colorbox-1.3.18/style-3/colorbox.css?v=8-5')); // For the desktop app download

$page->addEvent('onTemplateMerge', 'onTemplateMerge');
echo $page->parse();
?>
