<?php
require('application.php');
require('application.bootstrap.php');

header('Content-Type: text/html; charset=utf-8');

error_reporting(0); // Disable all error reporting

$allowed = array();

function list_files($folder = false) {
	$evp = Registry::get('EVP');
	
	if($folder) $folder = escape($folder, true);
	$raw_files = $evp->getFiles($folder);
	
	if($folder) $folder = $evp->getFolder($folder);
	
	$files = array();
	if(!empty($raw_files)) {
		foreach($raw_files as $f) {
			$files[] = array_remove_keys($f, array('data', 'options', 'events', 'widgets', 'page_widgets'));
		}
	}
	
	Ajax::success(array('files' => $files, 'folder' => $folder));
}
$allowed[] = 'list_files';

function list_folders($parent = false) {
	$evp = Registry::get('EVP');
	
	if($parent) $parent = escape($parent, true);
	$folders = $evp->getFolders($parent);
	
	Ajax::success(array('folders' => $folders, 'parent_id' => $parent));
}
$allowed[] = 'list_folders';

function file_setfolder($file_id, $folder_id) {
	$evp = Registry::get('EVP');
	
	if(is_array($file_id)) $ids = $file_id;
	else $ids = array($file_id);
	
	$modifiedFolders = array();
	
	foreach($ids as $id) {
		$id = escape($id, true);
		$folder_id = escape($folder_id, true);
		
		$modified = getRow("SELECT `folder_id` FROM {$evp->options->prefix}folders_relationships WHERE `file_id` = '{$id}'");
		if(!empty($modified)) $modifiedFolders[] = $modified->folder_id;
		query("DELETE FROM {$evp->options->prefix}folders_relationships WHERE `file_id` = '{$id}'");
		
		if(!empty($folder_id)) {
			query("INSERT INTO {$evp->options->prefix}folders_relationships (`file_id`, `folder_id`) VALUES ('{$id}', '{$folder_id}')");
		}
	}
	
	$modifiedFolders = array_unique($modifiedFolders);
	
	foreach($modifiedFolders as $f) query("UPDATE {$evp->options->prefix}folders SET `file_count` = `file_count` - ".sizeof($ids)." WHERE `id` = '{$f}' LIMIT 1");
	
	query("UPDATE {$evp->options->prefix}folders SET `file_count` = `file_count` + ".sizeof($ids)." WHERE `id` = '{$folder_id}' LIMIT 1");
	
	Ajax::success();
}
$allowed[] = 'file_setfolder';

function file_enable_security($file_ref) {
	$evp = Registry::get('EVP');
	$evp_api = Registry::get('EVP_API');
	
	$file = $evp->getFile($file_ref);
	if(!$file) Ajax::failed('Could not identify this file.');
	
	$s3 = $evp->s3->remote;

	$acl = $s3->getAccessControlPolicy($evp->settings->s3_bucket, $file->file_name);
	$acl['acl'] = array(array('type' => 'CanonicalUser', 'id' => $acl['owner']['id'], 'name' => $acl['owner']['name'], 'permission' => 'FULL_CONTROL'));
	$yes = $s3->setAccessControlPolicy($evp->settings->s3_bucket, $file->file_name, $acl);
	
	if($yes) Ajax::success();
	Ajax::failed();
}
$allowed[] = 'file_enable_security';

function file_disable_security() {
	$evp = Registry::get('EVP');
	$evp_api = Registry::get('EVP_API');
	
	$file = $evp->getFile($file_ref);
	if(!$file) Ajax::failed('Could not identify this file.');
	
	$s3 = $evp->s3->remote;

	$acl = $s3->getAccessControlPolicy($evp->settings->s3_bucket, $file->file_name);
	$public = array('type' => 'Group', 'uri' => 'http://acs.amazonaws.com/groups/global/AllUsers', 'permission' => 'READ');
	$owner = array('type' => 'CanonicalUser', 'id' => $acl['owner']['id'], 'name' => $acl['owner']['name'], 'permission' => 'FULL_CONTROL');
	$acl['acl'] = array($public, $owner);
	$yes = $s3->setAccessControlPolicy($evp->settings->s3_bucket, $file->file_name, $acl);
	
	if($yes) Ajax::success();
	Ajax::failed();
}
$allowed[] = 'file_disable_security';

function files_batch_delete($files = array()) {
	$evp = Registry::get('EVP');
	$evp_api = Registry::get('EVP_API');
	$yes = true;
	foreach($files as $idx => $file_id) {
		if($yes) {
			$yes = $evp_api->files_delete($file_id);
		}
	}
	if($yes) Ajax::success();
	Ajax::failed();
}
$allowed[] = 'files_batch_delete';

function file_import($url) {
	if(empty($url)) Ajax::failed();
	$evp = Registry::get('EVP');
	$evp_api = Registry::get('EVP_API');
	
  if(strtolower(substr($url, 0, 8)) !== 'https://' && strtolower(substr($url, 0, 7)) !== 'http://') {
		$url = 'http://'.$url;
  }
	
  // Check if it's a YouTube embed URL
  if(stripos($url, 'youtube.com/embed/') !== false || stripos($url, 'youtu.be') !== false) {
			$url = trim($url, '/');
			$ytd = explode('/', $url);
			$ytid = array_pop($ytd);
			$url = 'http://youtube.com/watch?v='.$ytid;
  }
  
  // Check if it's a remote YouTube video
  if(stripos($url, 'youtube.com') !== false) { // Huzzah
		parse_str(parse_url($url, PHP_URL_QUERY), $yt_vars);
		if(!empty($yt_vars) && !empty($yt_vars['v'])) {
			$youtube = true;
			$youtube_id = $yt_vars['v'];
			// Let's try to grab the name quickly
			$req = new Request_Curl(array('timeout' => 25, 'url' => 'http://www.youtube.com/watch'));
			$yt_page = $req->send(array('v' => $youtube_id));
			if(!empty($yt_page)) {
				preg_match('/<title>([^>]*)<\/title>/si', $yt_page, $yt_title);
				if(!empty($yt_title) && !empty($yt_title[1]) && strtolower(string_whitespace($yt_title[1])) != 'youtube - broadcast yourself.') {
					$file_name = $yt_title[1];
				} else {
					Ajax::failed(encode(array('msg' => 'The title could not be extracted; the video likely does not exist or is unavailable.')));
				}
			} else {
				Ajax::failed(encode(array('msg' => 'YouTube could not be reached.')));
			}
			if(!isset($yt_title)) { // We did not modify the name in any way
				$file_name = 'YouTube Imported Video';
			}
			$file_name = str_replace('youtube---', '', makeSeed($file_name));
			$extension = 'flv';
			$data = array(
				'page_seed' => makeSeed($file_name)
			);
		}
  } else {
		$file_name = getFilename($url);
		$extension = getExtension($url);
		$data = array(
			'page_seed' => makeSeed($file_name)
		);
	}
	
	$file = new Object(array(
		'id' => base64_encode($file_name),
		'name' => $file_name,
		'file_name' => $file_name,
		'time' => gmdate('U'),
		'size' => '',
		'extension' => $extension,
		'type' => 'video',
		'location' => $url,
		'data' => $data
	));
	
	$evp->insertMySQLObject('files', $file);
	
	Ajax::success('File imported successfully.');
}
$allowed[] = 'file_import';

function file_delete($file_id, $profile = false) {
	if(empty($file_id)) Ajax::failed();
	$evp = Registry::get('EVP');
	$evp_api = Registry::get('EVP_API');
	if($profile === false) {
		$yes = $evp_api->files_delete($file_id);
	} else {
		$yes = false;
	}
	if($yes) Ajax::success('Deleted successfully.');
	Ajax::failed('This file could not be deleted');
}
$allowed[] = 'file_delete';

function file_rename($file_id, $new_name) {
	if(empty($file_id)) Ajax::failed();
	$evp = Registry::get('EVP');
	$evp_api = Registry::get('EVP_API');
	$yes = $evp_api->files_rename($file_id, $new_name);
	if($yes) Ajax::success('Renamed successfully.');
	Ajax::failed('This file could not be renamed');
}
$allowed[] = 'file_rename';

function files_profile_new($file_id, $profile) {
	if(empty($file_id)) Ajax::failed();
	$evp = Registry::get('EVP');
	$evp_api = Registry::get('EVP_API');
	
	$file_ref = $evp_api->files_profile_new($file_id, $profile);
	
	if(!empty($file_ref)) Ajax::success(array('msg' => 'Created your Profile successfully.', 'profile' => $profile, 'file_ref' => $file_ref));
	Ajax::failed(array('Your Profile could not be created.'));
}
$allowed[] = 'files_profile_new';

function files_profile_delete($file_id, $profile) {
	if(empty($file_id)) Ajax::failed();
	$evp = Registry::get('EVP');
	$evp_api = Registry::get('EVP_API');
	
	$file_ref = $evp_api->files_profile_delete($file_id, $profile);
	
	if(!empty($file_ref)) Ajax::success(array('msg' => 'Deleted your Profile successfully.', 'profile' => $profile, 'file_ref' => $file_ref));
	Ajax::failed(array('Your Profile could not be deleted.'));
}
$allowed[] = 'files_profile_delete';

function files_profile_rename($file_id, $profile, $new_name) {
	if(empty($file_id) || empty($new_name)) Ajax::failed();
	$evp = Registry::get('EVP');
	$evp_api = Registry::get('EVP_API');
	
	$file_ref = $evp_api->files_profile_rename($file_id, $profile, $new_name);
	
	if(!empty($file_ref)) Ajax::success(array('msg' => 'Renamed your Profile successfully.', 'profile' => $new_name, 'file_ref' => $file_ref));
	Ajax::failed(array('Your Profile could not be renamed.'));
}
$allowed[] = 'files_profile_rename';

function files_profile_duplicate($file_id, $source_profile, $target_name) {
	if(empty($file_id) || empty($target_name)) Ajax::failed();
	$evp = Registry::get('EVP');
	$evp_api = Registry::get('EVP_API');
	
	$file_ref = $evp_api->files_profile_duplicate($file_id, $source_profile, $target_name);
	
	if(!empty($file_ref)) Ajax::success(array('msg' => 'Duplicated your Profile successfully.', 'profile' => $target_name, 'file_ref' => $file_ref));
	Ajax::failed(array('Your Profile could not be duplicated.'));
}
$allowed[] = 'files_profile_duplicate';

function file_options($file_reference, $data, $allowEmptyEvents = false) {
	if(empty($file_reference)) Ajax::failed();
	$evp = Registry::get('EVP');
	$evp_api = Registry::get('EVP_API');
	
	$data = unescape($data, true); // Unescape the data here
	
	$success = $evp_api->files_options($file_reference, $data);
	
	if($allowEmptyEvents === 'true' && empty($data['events'])) { // If we're saving Events specifically, they might be empty! In that case, we should delete them
		$evp->deleteDataKey('events', $file_reference, 'file', true);
	}
	
	if($allowEmptyEvents === 'true' && empty($data['options']['chapters'])) { // If we're saving Chapters specifically, they might be empty! In that case, we should delete them
		query("DELETE FROM {$evp->options->prefix}files_data WHERE `file_id` = '{$file_reference}' && `key` = 'options[chapters]'");
	}
	
	// We should confirm that widgets and plugins, due to their different handling, aren't empty
	/*
	if(empty($data['widgets'])) {
		$evp->insertDataKey('widgets', '', $file_reference, 'file');
		$evp->insertDataKey('options[widgets_order]', '', $file_reference, 'file');
	} else {
		$evp->deleteDataKey('widgets', $file_reference, 'file');
	}
	
	if(empty($data['plugins'])) {
		$evp->insertDataKey('plugins', '', $file_reference, 'file');
	}
	*/
	
	$file = $evp->getFile($file_reference);
	$profile = 'default';
	
	if(strpos($file_reference, '::')) {
		$id = explode('::', $file_reference, 2);
		$id = $id[0];
		$profile = $id[1];
		if(empty($profile)) $profile = 'default';
	}
	
	$embed_code = $evp->videoEmbedCode($file, 'sd', $profile, '');
	$embed_url = $evp->videoEmbedURL($file, 'sd', $profile, 'player-genuine');
	$file_settings = $evp->collateFileSettings($file, $profile);
	
	if($success) {
		// Let's build the sales tracking code
		$upsells = array();
		if($file_settings->options['sales_track'] == true) {
			$i = 1;
			while(!empty($file_settings->options['sales_track_product_upsell_'.$i])) {
				$name = $file_settings->options['sales_track_product_upsell_'.$i];
				$price = $file_settings->options['sales_track_product_upsell_price_'.$i];
				$upsells[$name] = $evp->salesTrackingImage($file_reference, $name, $price, true, 'conversion-insert');
				$i++;
			}
		}
		
		// And respond
		Ajax::success(array(
			'success' => true,
			'message' => 'Saved your settings successfully',
			'file' => $file,
			'embed_code' => $embed_code,
			'embed_url' => $embed_url,
			'sales_track' => $file_settings->options['sales_track'],
			'sales_tracking_upsells' => ($file_settings->options['sales_track'] == true && !empty($upsells)),
			'sales_tracking_main' => $evp->salesTrackingImage($file_reference, $file_settings->options['sales_track_product_main'], $file_settings->options['sales_track_product_main_price'], false),
			'sales_tracking_upsells' => $upsells
		));
	}
	Ajax::failed('Your settings could not be saved.');
}
$allowed[] = 'file_options';

function file_option($file_reference, $key, $value) {
	if(empty($file_reference)) Ajax::failed();
	$evp = Registry::get('EVP');
	$evp_api = Registry::get('EVP_API');
	
	$success = $evp_api->files_option($file_reference, "options[{$key}]", $value);
	
	if($success) Ajax::success('Saved your settings successfully');
	Ajax::failed('Your settings could not be saved.');
}
$allowed[] = 'file_option';

function file_setdata($file_reference, $data, $relevant = array(), $seed = false, $mode = 'video') {
	if(empty($file_reference)) Ajax::failed();
	$evp = Registry::get('EVP');
	$evp_api = Registry::get('EVP_API');
	
//$data = unescape($data, true); // Unescape the data here
	
	$original_file_reference = $file_reference;
	if($mode != 'video') $file_reference = $mode.'::'.$file_reference;
	
	if(!empty($seed)) {
		$seed = escape(makeSeed($seed), true);
		$evp->insertDataKey("data[page_seed]", $seed, $file_reference, 'file');
	}
	
	if($mode == 'video') {
		$success = $evp_api->files_setdata($file_reference, $data, $relevant);
		
		$file = $evp->getFile($file_reference);
		$profile = 'default';
		
		if(strpos($file_reference, '::')) {
			$id = explode('::', $file_reference, 2);
			$profile = $id[1];
			if(empty($profile)) $profile = 'default';
		}
		
		$embed_code = $evp->videoEmbedCode($file, 'sd', $profile, '');
		$embed_url = $evp->videoEmbedURL($file, 'sd', $profile, 'player-genuine');
		$file_settings = $evp->collateFileSettings($file, $profile);
	} elseif($mode == 'playlist') {
		$file = $evp->getPlaylist($original_file_reference);
		
		if(empty($relevant) || in_array('page_widgets', $relevant)) $evp->deleteDataKey('page_widgets', $file_reference, 'file', true);
		if(empty($relevant) || in_array('options', $relevant)) $evp->deleteDataKey('options', $file_reference, 'file', true);
		
		$ssn = ssn_encode($data);
		foreach($ssn as $k => $v) $evp->insertDataKey($k, $v, $file_reference, 'file');
		
		$success = true;
		
	} elseif($mode == 'splittest') {
		$file = $evp->getSplitTest($original_file_reference);
		
		if(empty($relevant) || in_array('page_widgets', $relevant)) $evp->deleteDataKey('page_widgets', $file_reference, 'file', true);
		if(empty($relevant) || in_array('options', $relevant)) $evp->deleteDataKey('options', $file_reference, 'file', true);
		
		$ssn = ssn_encode($data);
		foreach($ssn as $k => $v) $evp->insertDataKey($k, $v, $file_reference, 'file');
		
		$success = true;
		
	}
	
	if($success) {
		// Let's build the sales tracking code
		$upsells = array();
		if($file_settings->options['sales_track'] == true) {
			$i = 1;
			while(!empty($file_settings->options['sales_track_product_upsell_'.$i])) {
				$name = $file_settings->options['sales_track_product_upsell_'.$i];
				$price = $file_settings->options['sales_track_product_upsell_price_'.$i];
				$upsells[$name] = $evp->salesTrackingImage($file_reference, $name, $price, true, 'conversion-insert');
				$i++;
			}
		}
		
		// And respond
		Ajax::success(array(
			'success' => true,
			'message' => 'Saved your settings successfully',
			'file' => $file,
			'embed_code' => $embed_code,
			'embed_url' => $embed_url,
			'sales_track' => $file_settings->options['sales_track'],
			'sales_tracking_upsells' => ($file_settings->options['sales_track'] == true && !empty($upsells)),
			'sales_tracking_main' => $evp->salesTrackingImage($file_reference, $file_settings->options['sales_track_product_main'], $file_settings->options['sales_track_product_main_price'], false),
			'sales_tracking_upsells' => $upsells
		));
	}
	Ajax::failed('Your settings could not be saved.');
}
$allowed[] = 'file_setdata';

function page_seed($file_reference, $seed, $mode = 'video') {
	if(empty($file_reference)) Ajax::failed();
	$evp = Registry::get('EVP');
	$evp_api = Registry::get('EVP_API');
	
	$seed = escape(makeSeed($seed), true);
	
	if($mode != 'video') $file_reference = $mode.'::'.$file_reference;
	
	$success = $evp->insertDataKey("data[page_seed]", $seed, $file_reference, 'file');
	
	if($success) Ajax::success('Saved your seed successfully');
	Ajax::failed('Your seed could not be saved.');
}
$allowed[] = 'page_seed';

function files_batch_profile($files = array(), $source_id) {
	$evp = Registry::get('EVP');
	$evp_api = Registry::get('EVP_API');
	$source = $evp->getFile($source_id);
	
	$p = explode('::', $source_id, 2);
	if(empty($p[1])) $profile = '';
	else $profile = $p[1];
	
	if($source) {
		
		$file_settings = $evp->collateFileSettings($source, $profile);
		
		// Let's copy over the settings
		$options = array();
		$options['options'] = $file_settings->options;
	//$options['events'] = $file_settings->events;
	//$options['chapters'] = $file_settings->chapters;
		$options['widgets'] = $file_settings->widgets;
		$options['plugins'] = $file_settings->plugins;
		$options['page_widgets'] = $file_settings->page_widgets;
		
		foreach($files as $idx => $file_id) {
			$evp_api->files_setdata($file_id, $options);
		}
		
		Ajax::success();
	}
	Ajax::failed();
}
$allowed[] = 'files_batch_profile';

function folders_batch_profile($folders = array(), $source_id) {
	$evp = Registry::get('EVP');
	$evp_api = Registry::get('EVP_API');
	$source = $evp->getFile($source_id);
	
	$p = explode('::', $source_id, 2);
	if(empty($p[1])) $profile = '';
	else $profile = $p[1];
	
	if($source) {
		
		$file_settings = $evp->collateFileSettings($source, $profile);
		
		// Let's copy over the settings
		$options = array();
		$options['options'] = $file_settings->options;
		$options['widgets'] = $file_settings->widgets;
		$options['plugins'] = $file_settings->plugins;
		$options['page_widgets'] = $file_settings->page_widgets;
		
		$files = array();
		
		if(!empty($folders)) {
			foreach($folders as $fid) {
				$get = getArray("SELECT `file_id` FROM {$evp->options->prefix}folders_relationships WHERE `folder_id` = '{$fid}'");
				$files = array_merge($files, flattenByKey($get, 'file_id'));
			}
		}
		
		if(!empty($files)) {
			foreach($files as $idx => $file_id) {
				$evp_api->files_setdata($file_id, $options);
			}
		}
		
		Ajax::success();
	}
	Ajax::failed();
}
$allowed[] = 'folders_batch_profile';

function folders_batch_delete($folders = array()) {
	$evp = Registry::get('EVP');
	$evp_api = Registry::get('EVP_API');
	$yes = true;
	foreach($folders as $idx => $folder_id) {
		if($yes) {
			$yes = $evp_api->folders_delete($folder_id);
		}
	}
	if($yes) Ajax::success();
	Ajax::failed();
}
$allowed[] = 'folders_batch_delete';

function folder_delete($folder_id) {
	if(empty($folder_id)) Ajax::failed();
	$evp = Registry::get('EVP');
	$evp_api = Registry::get('EVP_API');
	$yes = $evp_api->folders_delete($folder_id);
	if($yes) Ajax::success('Deleted successfully.');
	Ajax::failed('This folder could not be deleted');
}
$allowed[] = 'folder_delete';

function folder_create($name, $parent_id = false) {
	if(empty($name)) Ajax::failed();
	$evp = Registry::get('EVP');
	$evp_api = Registry::get('EVP_API');
	$id = $evp_api->folders_create($name, $parent_id);
	if($id) Ajax::success(json_encode(array('id' => $id, 'name' => $name, 'parent_id' => $parent_id)));
	Ajax::failed('This folder could not be created.');
}
$allowed[] = 'folder_create';

function folder_rename($folder_id, $new_name) {
	if(empty($folder_id)) Ajax::failed();
	$evp = Registry::get('EVP');
	$evp_api = Registry::get('EVP_API');
	$yes = $evp_api->folders_rename($folder_id, $new_name);
	if($yes) Ajax::success('Renamed successfully.');
	Ajax::failed('This folder could not be renamed.');
}
$allowed[] = 'folder_rename';

function folder_set_parent($folder_id, $parent_id = false) {
	if(empty($folder_id)) Ajax::failed();
	$evp = Registry::get('EVP');
	$evp_api = Registry::get('EVP_API');
	$yes = $evp_api->folders_set_parent($folder_id, $parent_id);
	if($yes) Ajax::success('Updated parent successfully.');
	Ajax::failed('This folder could not be assigned to this parent.');
}
$allowed[] = 'folder_set_parent';

function widget_config_load($file_reference, $widget_slug, $widget_idx = false) {
	if(empty($file_reference) || empty($widget_slug)) Ajax::failed();
	$evp = Registry::get('EVP');
	$evp_api = Registry::get('EVP_API');
	
	$file = $evp->getFile($file_reference);
	
	$widget = $evp->getWidget($widget_slug, $file, $widget_idx);
	if(!$widget) Ajax::failed('Could not identify this widget.');
	
	if(!empty($widget->config->configRemote)) {
		include($widget->path.'/'.$widget->config->configRemote);
	} else {
		Ajax::failed('This widget has not specified a remote configuration file.');
	}
}
$allowed[] = 'widget_config_load';

function page_widget_config_load($file_reference, $pageWidget_slug, $pageWidget_idx = false, $source_mode = 'video') {
	if(empty($file_reference) || empty($pageWidget_slug)) Ajax::failed();
	$evp = Registry::get('EVP');
	$evp_api = Registry::get('EVP_API');
	
	if($source_mode == 'video') $file = $evp->getFile($file_reference);
	elseif($source_mode == 'playlist') $file = $evp->getPlaylist($file_reference);
	elseif($source_mode == 'splittest') $file = $evp->getSplitTest($file_reference);
	
	$pageWidget = $evp->getPageWidget($pageWidget_slug, $file, $pageWidget_idx, ($source_mode == 'video' ? false : $file));
	if(!$pageWidget) Ajax::failed('Could not identify this widget.');
	
	if(!empty($pageWidget->config->configRemote)) {
		include($pageWidget->path.'/'.$pageWidget->config->configRemote);
	} else {
		Ajax::failed('This page widget has not specified a remote configuration file.');
	}
}
$allowed[] = 'page_widget_config_load';

function users_delete($user_id) {
	if(empty($user_id)) Ajax::failed();
	$evp = Registry::get('EVP');
	$evp_api = Registry::get('EVP_API');
	$yes = $evp_api->users_delete($user_id);
	if($yes) Ajax::success('Deleted successfully.');
	Ajax::failed('This user could not be deleted');
}
$allowed[] = 'users_delete';

function sitemap_delete($sitemap_id) {
	if(empty($sitemap_id)) Ajax::failed();
	$evp = Registry::get('EVP');
	$evp_api = Registry::get('EVP_API');
	$yes = $evp_api->sitemaps_delete($sitemap_id);
	if($yes) Ajax::success('Deleted successfully.');
	Ajax::failed('This sitemap could not be deleted');
}
$allowed[] = 'sitemap_delete';

function general_testsmtp($host, $port, $username, $password, $to) {
 $evp = Registry::get('EVP');
 $config = Registry::retrieve('config');
 
 $host = trim(strtolower($host));
 $port = trim(strtolower($port));
 $username = trim($username);
 $password = trim($password);
 
 $to = trim(strtolower($to));
 
 if(empty($host) || empty($port) || empty($username) || empty($password) || empty($to)) {
		Ajax::failed(encode(array('msg' => 'Please make sure you provide a host, port, username, password and a test recipient address.')));
 }
 
 if(!is_numeric($port)) {
		Ajax::failed(encode(array('msg' => 'Please enter an integer for the port - it\'s usually 28!')));
 }
 
 $email = new Email(array(
	'useSMTP' => true,
	'host' => $host,
	'port' => $port,
	'username' => $username,
	'password' => $password,
	'security' => ($port == 465)
 ));
 
 $yes = $email->send(array($to, $to), 'Test email', "<p>This is an email to test the SMTP settings in use in {$evp->options->project_name}, installed at: {$config['locations']['absolute']}</p><p>It was sent on ".date('d/m/Y H:i').".</p><p>Thanks,</p>");
 if($yes) {
		Ajax::success(encode(array('msg' => 'Test email delivered successfully.')));
 } else {
		Ajax::failed(encode(array('msg' => 'Your message could not be delivered. Please check that all your settings are valid and try again.')));
 }
}
$allowed[] = 'general_testsmtp';

function general_refreshcloudfront() {
	$evp = Registry::get('EVP');
	
	$d = $evp->s3->remote->getDistribution($evp->settings->cloudfront_distribution);
	if($d) {
		$evp->s3->remote->updateDistribution($d);
		Ajax::success('Refreshing successfully.');
	}
	
	Ajax::failed('The distribution could not be refreshed - does it still exist?');
}
$allowed[] = 'general_refreshcloudfront';

function general_deletedefaultsettings() {
	$evp = Registry::get('EVP');
	
	query("DELETE FROM {$evp->options->prefix}files_data WHERE `file_id` = 'default'");
	
	Ajax::success('Deleted successfully.');
}
$allowed[] = 'general_deletedefaultsettings';

function general_advanced_optimisemysql() {
	$evp = Registry::get('EVP');
	$config = Registry::retrieve('config');

	$total_tables = 0;
	$total_overhead = 0;
	
	$result = mysql_query('SHOW TABLES') or die('Cannot get tables');
 while($table = mysql_fetch_row($result)) {
 	$total_tables++;
 	$status = getRow("SHOW TABLE STATUS LIKE '{$table[0]}'");
 	
 	$total_overhead += $status->Data_free;
 	
		mysql_query('OPTIMIZE TABLE '.$table[0]) or die('Cannot optimize '.$table[0]);
	}
	
	Ajax::success(encode(array('success' => true, 'total_tables' => $total_tables, 'total_overhead' => $total_overhead, 'pretty_overhead' => formatRawSize($total_overhead))));
	
}
$allowed[] = 'general_advanced_optimisemysql';

function general_advanced_download_config() {
	$config = Registry::retrieve('config');
	header('Content-disposition: attachment; filename=config.php');
	header('Content-type: text/php');
	readfile($config['paths']['base'].'/config.php');
}
$allowed[] = 'general_advanced_download_config';

function general_advanced_emptycache() {
	$evp = Registry::get('EVP');
	
	$total = $evp->api->empty_cache();
 
 Ajax::success(encode(array('success' => true, 'files' => $total)));
}
$allowed[] = 'general_advanced_emptycache';

function general_advanced_emptyvideocache() {
	$evp = Registry::get('EVP');
	
	$evp->api->empty_video_cache();
 
 Ajax::success(encode(array('success' => true)));
}
$allowed[] = 'general_advanced_emptyvideocache';

function general_sendemail($message, $from, $page, $url, $file_ref) {
 $evp = Registry::get('EVP');
 $xs = Registry::get('UserManager');
 $config = Registry::retrieve('config');
 
 $message = trim($message);
 $from = trim($from);
 $to = 'support@webactix.zendesk.com';
 
	// Let's expose the default message for the help section
	$help_support_info = '';
	$help_support_info.= "Commercial: ".($evp->options->commercial === true ? 'Yes' : 'No')."\n";
	$help_support_info.= "Installed at: {$config['locations']['absolute']}\n";
	$help_support_info.= "Logged in username: {$xs->userData->username}\n";
	if(!$evp->options->hosted) {
		$help_support_info.= "{$evp->options->project_name} version: {$evp->options->project_version}\n";
		$help_support_info.= "PHP version: ".phpversion()."\n";
		$help_support_info.= "Path: {$config['paths']['base']}\n";
		$help_support_info.= "Enabled local hosting: ".($evp->local ? 'Yes' : 'No')."\n";
	}
	$help_support_info.= "Enabled Amazon S3: ".($evp->s3 ? "Yes ({$evp->settings->s3_bucket})" : 'No')."\n";
	$help_support_info.= "Enabled CloudFront: ".($evp->s3 && $evp->settings->use_cloudfront == '1' ? "Yes ({$evp->s3->options->type})" : 'No')."\n";
	$help_support_info.= "------------------\n";
	$help_support_info.= "Ticket submitted from: {$url}\n";
//if(!empty($file_ref)) $help_support_info.= "Current file ID (When submitting): {$file_ref}\n";
	$help_support_info.= "------------------\n\n";
	$help_support_info.= $message;
	$help_support_info.= "\n\n------------------\n";

	$s = $evp->settings;
	if(!empty($s->smtp_host) && !empty($s->smtp_port) && !empty($s->smtp_username) && !empty($s->smtp_password)) {
		$c = Registry::retrieve('config');
		$c['smtp'] = array(
		'host' => $evp->settings->smtp_host,
		'port' => $evp->settings->smtp_port,
		'username' => $evp->settings->smtp_username,
		'password' => $evp->settings->smtp_password,
		'security' => ($port == 465)
		);
		Registry::store('config', $c);
	}
	
	$ticket = '<p>'.nl2br($help_support_info).'</p>';
	
	$subject = $evp->options->project_name.': Help request';
 
 $yes = sendEmail(array($to, $to), $from, $subject, $ticket);
 if($yes) {
		Ajax::success(encode(array('msg' => 'Ticket submitted successfully.', 'ticket' => $ticket)));
 } else {
		Ajax::failed(encode(array('to' => $to, 'ticket' => $ticket)));
 }
}
$allowed[] = 'general_sendemail';

$input_method = (empty($_POST) ? $_GET : $_POST);

if(in_array($input_method['method'], $allowed)) {
  call_user_func_array($input_method['method'], array_remove_keys($input_method, array('method')));
} else {
  echo dump($_GET);
  echo dump($_POST);
  echo dump($input_method);
  Ajax::failed(encode(new Object(array('msg' => 'Method not allowed.', 'type' => 'failed'))));
}
?>