<?php
session_start(); // Start a session

// Load up the config
$config = array();

// Some path configuration
$config['paths'] = array();
$config['paths']['base'] = realpath('.');
$config['paths']['folder_base'] = realpath('.');
$config['paths']['script'] = realpath($_SERVER['SCRIPT_FILENAME']);

// Now we have the paths, so we need to include things
require($config['paths']['base'].'/libraries/base.php');
require($config['paths']['base'].'/libraries/registry.php');
require($config['paths']['base'].'/helpers/general.php');
require($config['paths']['base'].'/helpers/files.php');
require($config['paths']['base'].'/helpers/natives.php');
require($config['paths']['base'].'/helpers/string.php');
require($config['paths']['base'].'/libraries/database.php');

// Manually load our libraries
require($config['paths']['base'].'/libraries/ajax.php');
require($config['paths']['base'].'/libraries/email.phpmailer.php');
require($config['paths']['base'].'/libraries/email.pop3.php');
require($config['paths']['base'].'/libraries/email.smtp.php');
require($config['paths']['base'].'/libraries/email.wrapper.php');
require($config['paths']['base'].'/libraries/eternity.php');
require($config['paths']['base'].'/libraries/mysql.php');
require($config['paths']['base'].'/libraries/page.php');
require($config['paths']['base'].'/libraries/project.php');
require($config['paths']['base'].'/libraries/request.php');
require($config['paths']['base'].'/libraries/s3.php');
require($config['paths']['base'].'/libraries/session.php');
require($config['paths']['base'].'/libraries/template.php');
require($config['paths']['base'].'/libraries/uploader.php');
require($config['paths']['base'].'/libraries/usermanager.php');
require($config['paths']['base'].'/libraries/validator.php');
//require($config['paths']['base'].'/libraries/webactix_geo.php');
//require($config['paths']['base'].'/libraries/webactix_validate.php');
//require($config['paths']['base'].'/libraries/zip.php');

// Manually load helpers
require($config['paths']['base'].'/helpers/debug.php');
require($config['paths']['base'].'/helpers/environment.php');
require($config['paths']['base'].'/helpers/html.php');
require($config['paths']['base'].'/helpers/misc.php');
require($config['paths']['base'].'/helpers/time.php');

// Manually load project files
require($config['paths']['base'].'/project/evp.php');
require($config['paths']['base'].'/project/evp.api.php');
require($config['paths']['base'].'/project/evp.stats.php');

// Include the main config
require($config['paths']['base'].'/config.php');

error_reporting(0);

// If this is a hosted service, overwrite some config with custom values
// specific to each subdomain
if(EVP::$defaults['hosted'] === true) {
	$correct_location = false;
	$domain = $_SERVER['SERVER_NAME'];
	if(stripos($domain, 'www.') !== false) $domain = str_replace('www.', '', $domain);
	if(stripos($domain, '.evsuite.com') !== false) {
		$domain = explode('.', $domain);
		$subdomain = $domain[0];
	} else {
		$correct_location = true;
		$domain = basename($domain);
		// Let's grab the alias folder, which exposes the $subdomain variable!
		$alias_folder = $config['paths']['base'].'/../aliases/'.$domain;
		if(is_dir($alias_folder)) require($alias_folder.'/config.php');
	}
	if($subdomain) {
		$subdomain_path = $config['paths']['base'].'/../configurations/'.$subdomain;
		if(!empty($subdomain) && is_dir($subdomain_path)) {
			require($subdomain_path.'/config.php');
			$config['paths']['subdomain'] = $subdomain_path;
			$config['locations']['subdomain'] = $subdomain;
			if(empty($config['locations']['absolute'])) { // If this hasn't been overridden per-subdomain, set
				$config['locations']['absolute'] = "http://{$subdomain}.evsuite.com/";
			}

			if($correct_location === true) { // This will be set if using a domain alias
				$config['locations']['absolute'] = "http://{$domain}/";
			}

		} else {
			die('An invalid subdomain ('.$subdomain.') was provided to this hosted version of '.EVP::$defaults['project_name']);
		}
	} else {
		die('The account you are looking for could not be identified.');
	}
}

// Monkey-patch for 2.x compatibility
if(empty($config['evp'])) $config['evp'] = $config['serenity'];

// SSL support
$config['locations']['protocol'] = 'http';
if((!empty($_SERVER['HTTPS']) && $_SERVER['HTTPS'] != 'off') || (!empty($_SERVER['HTTP_X_FORWARDED_PROTO']) && $_SERVER['HTTP_X_FORWARDED_PROTO'] === 'https')) {
	$config['locations']['absolute'] = str_replace('http://', 'https://', $config['locations']['absolute']);
	$config['locations']['protocol'] = 'https';
}

Registry::store('config', $config); // Write to the Registry

// Add some extra location data
$config['locations']['script'] = 'http://'.$_SERVER['SERVER_NAME'].$_SERVER['SCRIPT_NAME'];
$config['locations']['folder_name'] = '';
$config['locations']['folder_relative'] = '/';
$config['locations']['folder_absolute'] = url($config['locations']['absolute'].(empty($config['locations']['folder_name']) ? '' : $config['locations']['folder_name'].'/'));

Registry::store('config', $config); // And save the updates

if(isset($config['database'])) {
 Registry::store('database', new Database($config['database']['type']));
 if($config['database']['autoconnect'] == true) {
	Registry::retrieve('database')->connect($config['database']);
 }
}

if(isset($config['users'])) {
 Registry::store('users', new UserManager($config['users']));
	$xs = Registry::retrieve('users');
}
?>
