<?php
/**
 * Returns the array or object with utf8 encoded strings
 * @return array
 */
function utf8_encode_array($value) {
	if(is_array($value) || is_object($value)) {
		foreach($value as &$v) $v = utf8_encode_array($v);
		return $value;
	}
	return utf8_encode($value);
}

/**
 * Returns the array or object with utf8 decoded strings
 * @return array
 */
function utf8_decode_array($value) {
	if(is_array($value) || is_object($value)) {
		foreach($value as &$v) $v = utf8_decode_array($v);
		return $value;
	}
	return utf8_decode($value);
}

/**
 * Returns the current IP address, even if it's behind an AWS load balancer
 * <code>
 * $ip = getRemoteIP(); // 10.451.141.3
 * </code>
 * @return string
 */
function getRemoteIP() {
	return (empty($_SERVER['HTTP_X_CLUSTER_CLIENT_IP']) ? $_SERVER['REMOTE_ADDR'] : $_SERVER['HTTP_X_CLUSTER_CLIENT_IP']);
}

/**
 * @todo Document this function
 */
function pass_create($url = false) {
	if($url === false) $url = currentUrl();
	else $url = strip_tags($url);
	return urlencode(base64_encode($url));
}

/**
 * @todo Document this function
 */
function pass_resolve($pass) {
	return strip_tags(base64_decode(urldecode($pass)));
}

/**
 * @todo Document this function
 */
function url($url = false, $query_data = false, $resolve = true) {
	$config = Registry::retrieve('config');
	if($resolve == true && substr($url, 0, 7) != 'http://' && substr($url, 0, 8) != 'https://') {
		$url = ltrim($url, '/');
		$url = $config['locations']['absolute'].$url;
	}
	if($url === false) $url = $config['locations']['absolute'];
	if(!empty($query_data)) {
		$parsed = parse_url($url, PHP_URL_QUERY);
		if($parsed) {
			parse_str($parsed, $existing);
			$existing = array_merge($existing, $query_data);
			$existing = http_build_query($existing);
			$base = explode('?', $url);
			$url = $base[0].'?'.$existing;
		} else {
			$url.= '?'.http_build_query($query_data);
		}
	}
	return $url;
}

function getWidget($widget_name, $data = array()) {
	$config = Registry::retrieve('config');
	return url($config['locations']['folder_name'].'/themes/'.$config['misc']['theme'].'/widgets/'.$widget_name.'.php', $data);
}

function getWidgetInclude($widget_name) {
	$config = Registry::retrieve('config');
	return $config['paths']['base'].'/'.$config['locations']['folder_name'].'/themes/'.$config['misc']['theme'].'/widgets/'.$widget_name.'.php';
}

/**
 * Redirects the user and exits execution
 * Can also resolve urls to the current site
 * <code>
 * redirect(); // go to your absolute url
 * redirect('test.php'); // go to http://thissite.com/test.php
 * redirect('test.php', false, false); // go to test.php, relative to here
 * redirect('http://google.com'); // will still go to google.com - you don't need to do anything else
 * </code>
 * @param string $url
 * @param bool $resolve Resolve to the current site
 */
function redirect($url = false, $query_data = false, $resolve = true) {
	$url = url($url, $query_data, $resolve);
	header("Location: {$url}");
	exit;
}
 
/**
 * Returns the current url
 * <code>
 * $currentPage = currentUrl(); // http://defraction.net/
 * </code>
 * @return string
 */
function currentUrl($getQueryString = true) {
  $domain = $_SERVER['HTTP_HOST'];
  $path = $_SERVER['SCRIPT_NAME'];
  $qs = $_SERVER['QUERY_STRING'];
  $url = ($_SERVER['HTTPS'] == 'on' ? 'https://' : 'http://').$domain.$path;
	if($getQueryString === true && !empty($qs)) {
		$url.= '?'.$qs;
	}
  return $url;
	/* // This will not return 'www.' correctly!
	$pageURL = 'http';
	if(isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] == 'on') $pageURL .= 's';
	$pageURL .= '://';
	if($_SERVER['SERVER_PORT'] != '80') $pageURL .= "{$_SERVER['SERVER_NAME']}:{$_SERVER['SERVER_PORT']}{$_SERVER['REQUEST_URI']}";
	else $pageURL .= $_SERVER['SERVER_NAME'].$_SERVER['REQUEST_URI'];
	if($getQueryString === false && strpos($pageURL, '?') !== false) {
		$pageURL = explode('?', $pageURL);
		$pageURL = grab($pageURL, 0);
	}
	return $pageURL;
	*/
}

/**
 * @todo Document this function
 */
function currentDomain() {
	$config = Registry::retrieve('config');
	$current_domain = parse_url($config['locations']['absolute'], PHP_URL_HOST);
	$current_domain = str_replace('www.', '', strtolower($current_domain));
	return 'http://'.$current_domain.'/';
}

/**
 * @todo Document this function
 */
function resolveUrl($base, $href) {
	if(!$href) return $base;
	$rel_parsed = parse_url($href);
	if(array_key_exists('scheme', $rel_parsed)) return $href;
	$base_parsed = parse_url("{$base} ");
	if(!array_key_exists('path', $base_parsed)) $base_parsed = parse_url("{$base}/ ");
	if($href{0} === '/') $path = $href;
	else $path = dirname($base_parsed['path'])."/{$href}";
	$path = preg_replace('~/\./~', '/', $path);
	$parts = array();
	foreach(explode('/', preg_replace('~/+~', '/', $path)) as $part) {
		if($part === "..") array_pop($parts);
		elseif($part) $parts[] = $part;
	}
	return ((array_key_exists('scheme', $base_parsed)) ? $base_parsed['scheme'].'://'.$base_parsed['host'] : '').'/'.implode('/', $parts);
}

/**
 * Formats the passed data to a query string
 * <code>
 * $queryString = toQueryString($array); // ?somekey=value&whatever=test
 * </code>
 * @see http://php.net/http_build_query
 * @param array|object|Object $data
 * @return string
 */
function toQueryString($data, $numeric_prefix = '', $arg_separator = false) {
	if(is_object($data)) $data = (array)$data;
	return '?'.http_build_query($data);
}

/**
 * Expands a query string back into an array
 * This will fail on more complex strings
 * <code>
 * $queryData = fromQueryString('somekey=value&amp;whatever=test');
 * </code>
 * @param string $str
 * @return array
 * @todo Improve this function to handle array expansion, etc
 */
function fromQueryString($str) {
	$str = html_entity_decode($str);
	$var = explode('&', $str);
	if(empty($var)) $var = array($var);
	$arr = array();
	foreach($var as $val) {
		$x = explode('=', $val);
		$arr[$x[0]] = $x[1];
	}
	return $arr;
}

/**
 * Returns the headers from a url
 * <code>
 * $headers = getHeaders('www.test.com/woof.gif', 'gif');
 * </code>
 * @param string $url The url to check
 * @param string|array $ext Only return headers if they meet this file extension
 * @return int|array Error code or headers array
 * @link http://uk2.php.net/manual/en/function.file-exists.php#76246
 */
function getHeaders($file, $ext = false) {
	$file = ereg_replace(' +', '%20', trim($file));
	if(substr($file, 0, 7) !== "http://") $file = "http://{$file}";
	
	if($ext) {
		$fileExt = getExtension($file);
		if(is_array($ext)) {
			if(!in_array($fileExt, $ext)) return 1;
		} elseif($fileExt !== $ext) return 1;
	}
	
	$domain = substr($file, 7); // lose the http prefix
	$tld = strtoupper(array_pop(explode('.', substr($domain, 0, strpos($domain, '/'))))); // Get the TLD
	$types = array('AC', 'AD', 'AE', 'AERO', 'AF', 'AG', 'AI', 'AL', 'AM', 'AN', 'AO', 'AQ', 'AR', 'ARPA', 'AS', 'ASIA', 'AT', 'AU', 'AW', 'AX', 'AZ', 'BA', 'BB', 'BD', 'BE', 'BF', 'BG', 'BH', 'BI', 'BIZ', 'BJ', 'BM', 'BN', 'BO', 'BR', 'BS', 'BT', 'BV', 'BW', 'BY', 'BZ', 'CA', 'CAT', 'CC', 'CD', 'CF', 'CG', 'CH', 'CI', 'CK', 'CL', 'CM', 'CN', 'CO', 'COM', 'COOP', 'CR', 'CU', 'CV', 'CX', 'CY', 'CZ', 'DE', 'DJ', 'DK', 'DM', 'DO', 'DZ', 'EC', 'EDU', 'EE', 'EG', 'ER', 'ES', 'ET', 'EU', 'FI', 'FJ', 'FK', 'FM', 'FO', 'FR', 'GA', 'GB', 'GD', 'GE', 'GF', 'GG', 'GH', 'GI', 'GL', 'GM', 'GN', 'GOV', 'GP', 'GQ', 'GR', 'GS', 'GT', 'GU', 'GW', 'GY', 'HK', 'HM', 'HN', 'HR', 'HT', 'HU', 'ID', 'IE', 'IL', 'IM', 'IN', 'INFO', 'INT', 'IO', 'IQ', 'IR', 'IS', 'IT', 'JE', 'JM', 'JO', 'JOBS', 'JP', 'KE', 'KG', 'KH', 'KI', 'KM', 'KN', 'KR', 'KW', 'KY', 'KZ', 'LA', 'LB', 'LC', 'LI', 'LK', 'LOCAL', 'LR', 'LS', 'LT', 'LU', 'LV', 'LY', 'MA', 'MC', 'MD', 'MG', 'MH', 'MIL', 'MK', 'ML', 'MM', 'MN', 'MO', 'MOBI', 'MP', 'MQ', 'MR', 'MS', 'MT', 'MU', 'MUSEUM', 'MV', 'MW', 'MX', 'MY', 'MZ', 'NA', 'NAME', 'NC', 'NE', 'NET', 'NF', 'NG', 'NI', 'NL', 'NO', 'NP', 'NR', 'NU', 'NZ', 'OM', 'ORG', 'PA', 'PE', 'PF', 'PG', 'PH', 'PK', 'PL', 'PM', 'PN', 'PR', 'PRO', 'PS', 'PT', 'PW', 'PY', 'QA', 'RE', 'RO', 'RU', 'RW', 'SA', 'SB', 'SC', 'SD', 'SE', 'SG', 'SH', 'SI', 'SJ', 'SK', 'SL', 'SM', 'SN', 'SO', 'SR', 'ST', 'SU', 'SV', 'SY', 'SZ', 'TC', 'TD', 'TEL', 'TF', 'TG', 'TH', 'TJ', 'TK', 'TL', 'TM', 'TN', 'TO', 'TP', 'TR', 'TRAVEL', 'TT', 'TV', 'TW', 'TZ', 'UA', 'UG', 'UK', 'UM', 'US', 'UY', 'UZ', 'VA', 'VC', 'VE', 'VG', 'VI', 'VN', 'VU', 'WF', 'WS', 'YE', 'YT', 'YU', 'ZA', 'ZM', 'ZW');
	
	if(!in_array($tld, $types)) return 2;
	
	$fileHeaders = @get_headers($file);
	if(!$fileHeaders) return 3;
	
	if($fileHeaders[0] == 'HTTP/1.1 404 Not Found') return 404;
	else return $fileHeaders;
}

/**
 * Forwards a POST or GET request
 * <code>
 * $response = sendToHost('http://google.com', 'post', '/someFile.php', 'something=value&var=whatever');
 * </code>
 * @param string $host The url to forward the request to
 * @param string $method 'get' or 'post'
 * @param string $path
 * @param array|object|Object $data
 * @return string
 */

function sendToHost($host, $method = 'post', $path, $data) {
	if(!is_string($data)) {
		$data = (array)$data;
		$data = toQueryString($data);
	}
	$fp = fsockopen($host, '80');
	if($fp) {
		fputs($fp, "POST {$path} HTTP/1.1\r\n");
		fputs($fp, "Host: {$host}\r\n");
		fputs($fp, "Content-type: application/x-www-form-urlencoded\r\n");
		fputs($fp, "Content-length: ".strlen($data)."\r\n");
		fputs($fp, "Connection: close\r\n\r\n");
		fputs($fp, $data . "\r\n\r\n");
		while(!feof($fp)) $buf .= fgets($fp, 1024);
		fclose($fp);
	}
	return $buf;
}

$countryCodes = array ( 'AF' => 'Afghanistan', 'AX' => 'land Islands', 'AL' => 'Albania', 'DZ' => 'Algeria', 'AS' => 'American Samoa', 'AD' => 'Andorra', 'AO' => 'Angola', 'AI' => 'Anguilla', 'AQ' => 'Antarctica', 'AG' => 'Antigua And Barbuda', 'AR' => 'Argentina', 'AM' => 'Armenia', 'AW' => 'Aruba', 'AU' => 'Australia', 'AT' => 'Austria', 'AZ' => 'Azerbaijan', 'BS' => 'Bahamas', 'BH' => 'Bahrain', 'BD' => 'Bangladesh', 'BB' => 'Barbados', 'BY' => 'Belarus', 'BE' => 'Belgium', 'BZ' => 'Belize', 'BJ' => 'Benin', 'BM' => 'Bermuda', 'BT' => 'Bhutan', 'BO' => 'Bolivia', 'BA' => 'Bosnia And Herzegovina', 'BW' => 'Botswana', 'BV' => 'Bouvet Island', 'BR' => 'Brazil', 'IO' => 'British Indian Ocean Territory', 'BN' => 'Brunei Darussalam', 'BG' => 'Bulgaria', 'BF' => 'Burkina Faso', 'BI' => 'Burundi', 'KH' => 'Cambodia', 'CM' => 'Cameroon', 'CA' => 'Canada', 'CV' => 'Cape Verde', 'KY' => 'Cayman Islands', 'CF' => 'Central African Republic', 'TD' => 'Chad', 'CL' => 'Chile', 'CN' => 'China', 'CX' => 'Christmas Island', 'CC' => 'Cocos (keeling) Islands', 'CO' => 'Colombia', 'KM' => 'Comoros', 'CG' => 'Congo', 'CD' => 'Congo, The Democratic Republic Of The', 'CK' => 'Cook Islands', 'CR' => 'Costa Rica', 'CI' => 'Cote D\'ivoire', 'HR' => 'Croatia', 'CU' => 'Cuba', 'CY' => 'Cyprus', 'CZ' => 'Czech Republic', 'DK' => 'Denmark', 'DJ' => 'Djibouti', 'DM' => 'Dominica', 'DO' => 'Dominican Republic', 'EC' => 'Ecuador', 'EG' => 'Egypt', 'SV' => 'El Salvador', 'GQ' => 'Equatorial Guinea', 'ER' => 'Eritrea', 'EE' => 'Estonia', 'ET' => 'Ethiopia', 'FK' => 'Falkland Islands (malvinas)', 'FO' => 'Faroe Islands', 'FJ' => 'Fiji', 'FI' => 'Finland', 'FR' => 'France', 'GF' => 'French Guiana', 'PF' => 'French Polynesia', 'TF' => 'French Southern Territories', 'GA' => 'Gabon', 'GM' => 'Gambia', 'GE' => 'Georgia', 'DE' => 'Germany', 'GH' => 'Ghana', 'GI' => 'Gibraltar', 'GR' => 'Greece', 'GL' => 'Greenland', 'GD' => 'Grenada', 'GP' => 'Guadeloupe', 'GU' => 'Guam', 'GT' => 'Guatemala', 'GG' => 'Guernsey', 'GN' => 'Guinea', 'GW' => 'Guinea-bissau', 'GY' => 'Guyana', 'HT' => 'Haiti', 'HM' => 'Heard Island And Mcdonald Islands', 'VA' => 'Holy See (vatican City State)', 'HN' => 'Honduras', 'HK' => 'Hong Kong', 'HU' => 'Hungary', 'IS' => 'Iceland', 'IN' => 'India', 'ID' => 'Indonesia', 'IR' => 'Iran, Islamic Republic Of', 'IQ' => 'Iraq', 'IE' => 'Ireland', 'IM' => 'Isle Of Man', 'IL' => 'Israel', 'IT' => 'Italy', 'JM' => 'Jamaica', 'JP' => 'Japan', 'JE' => 'Jersey', 'JO' => 'Jordan', 'KZ' => 'Kazakhstan', 'KE' => 'Kenya', 'KI' => 'Kiribati', 'KP' => 'Korea, Democratic People\'s Republic Of', 'KR' => 'Korea, Republic Of', 'KW' => 'Kuwait', 'KG' => 'Kyrgyzstan', 'LA' => 'Lao People\'s Democratic Republic', 'LV' => 'Latvia', 'LB' => 'Lebanon', 'LS' => 'Lesotho', 'LR' => 'Liberia', 'LY' => 'Libyan Arab Jamahiriya', 'LI' => 'Liechtenstein', 'LT' => 'Lithuania', 'LU' => 'Luxembourg', 'MO' => 'Macao', 'MK' => 'Macedonia, The Former Yugoslav Republic Of', 'MG' => 'Madagascar', 'MW' => 'Malawi', 'MY' => 'Malaysia', 'MV' => 'Maldives', 'ML' => 'Mali', 'MT' => 'Malta', 'MH' => 'Marshall Islands', 'MQ' => 'Martinique', 'MR' => 'Mauritania', 'MU' => 'Mauritius', 'YT' => 'Mayotte', 'MX' => 'Mexico', 'FM' => 'Micronesia, Federated States Of', 'MD' => 'Moldova, Republic Of', 'MC' => 'Monaco', 'MN' => 'Mongolia', 'MS' => 'Montserrat', 'MA' => 'Morocco', 'MZ' => 'Mozambique', 'MM' => 'Myanmar', 'NA' => 'Namibia', 'NR' => 'Nauru', 'NP' => 'Nepal', 'NL' => 'Netherlands', 'AN' => 'Netherlands Antilles', 'NC' => 'New Caledonia', 'NZ' => 'New Zealand', 'NI' => 'Nicaragua', 'NE' => 'Niger', 'NG' => 'Nigeria', 'NU' => 'Niue', 'NF' => 'Norfolk Island', 'MP' => 'Northern Mariana Islands', 'NO' => 'Norway', 'OM' => 'Oman', 'PK' => 'Pakistan', 'PW' => 'Palau', 'PS' => 'Palestinian Territory, Occupied', 'PA' => 'Panama', 'PG' => 'Papua New Guinea', 'PY' => 'Paraguay', 'PE' => 'Peru', 'PH' => 'Philippines', 'PN' => 'Pitcairn', 'PL' => 'Poland', 'PT' => 'Portugal', 'PR' => 'Puerto Rico', 'QA' => 'Qatar', 'RE' => 'Reunion', 'RO' => 'Romania', 'RU' => 'Russian Federation', 'RW' => 'Rwanda', 'SH' => 'Saint Helena', 'KN' => 'Saint Kitts And Nevis', 'LC' => 'Saint Lucia', 'PM' => 'Saint Pierre And Miquelon', 'VC' => 'Saint Vincent And The Grenadines', 'WS' => 'Samoa', 'SM' => 'San Marino', 'ST' => 'Sao Tome And Principe', 'SA' => 'Saudi Arabia', 'SN' => 'Senegal', 'CS' => 'Serbia And Montenegro', 'SC' => 'Seychelles', 'SL' => 'Sierra Leone', 'SG' => 'Singapore', 'SK' => 'Slovakia', 'SI' => 'Slovenia', 'SB' => 'Solomon Islands', 'SO' => 'Somalia', 'ZA' => 'South Africa', 'GS' => 'South Georgia And The South Sandwich Islands', 'ES' => 'Spain', 'LK' => 'Sri Lanka', 'SD' => 'Sudan', 'SR' => 'Suriname', 'SJ' => 'Svalbard And Jan Mayen', 'SZ' => 'Swaziland', 'SE' => 'Sweden', 'CH' => 'Switzerland', 'SY' => 'Syrian Arab Republic', 'TW' => 'Taiwan, Province Of China', 'TJ' => 'Tajikistan', 'TZ' => 'Tanzania, United Republic Of', 'TH' => 'Thailand', 'TL' => 'Timor-leste', 'TG' => 'Togo', 'TK' => 'Tokelau', 'TO' => 'Tonga', 'TT' => 'Trinidad And Tobago', 'TN' => 'Tunisia', 'TR' => 'Turkey', 'TM' => 'Turkmenistan', 'TC' => 'Turks And Caicos Islands', 'TV' => 'Tuvalu', 'UG' => 'Uganda', 'UA' => 'Ukraine', 'AE' => 'United Arab Emirates', 'GB' => 'United Kingdom', 'US' => 'United States', 'UM' => 'United States Minor Outlying Islands', 'UY' => 'Uruguay', 'UZ' => 'Uzbekistan', 'VU' => 'Vanuatu', 'VE' => 'Venezuela', 'VN' => 'Viet Nam', 'VG' => 'Virgin Islands, British', 'VI' => 'Virgin Islands, U.s.', 'WF' => 'Wallis And Futuna', 'EH' => 'Western Sahara', 'YE' => 'Yemen', 'ZM' => 'Zambia', 'ZW' => 'Zimbabwe');
?>