<?php
/**
 * Recursively returns the paths of all the files in a directory
 * <code>
 * $files = getFiles('somedir');
 * </code>
 * @param string $from The directory to start scanning at
 * @return array The files found
 */
function getFiles($from = '.') {
	if(!is_dir($from)) return false;
	$files = array();
	if($dh = opendir($from)) {
		while(false !== ($file = readdir($dh))) {
			if($file == '.' || $file == '..' || $file == '.svn' || $file == 'Thumbs.db') continue;
			$path = $from.DIRECTORY_SEPARATOR.$file;
			if(is_dir($path)) $files = array_merge($files, getFiles($path));
			else $files[] = realpath($path);
		}
		closedir($dh);
	}
	return $files;
}

/**
 * Recursively returns the paths of all the subdirectories of a given directory
 * <code>
 * $dirs = getDirectories('somedir');
 * </code>
 * @param string $from The directory to start scanning at
 * @return array The directories found
 */
function getDirectories($from = '.', $recurse = true) {
	if(!is_dir($from)) return false;
	$dirs = array();
	if($dh = opendir($from)) {
		while(false !== ($file = readdir($dh))) {
			if($file == '.' || $file == '..' || $file == '.svn' || $file == 'Thumbs.db') continue;
			$path = $from.DIRECTORY_SEPARATOR.$file;
			if(is_dir($path)) {
				if($recurse == true) {
					$nest = getDirectories(realpath($path));
					$dirs[realpath($path)] = $nest;
				} else {
					$dirs[] = realpath($path);
				}
			}
		}
		closedir($dh);
	}
	return $dirs;
}

/**
 * Gets the extension of a passed file
 * <code>
 * $ext = getExtension('someFile.php'); // $ext is 'php'
 * </code>
 * @todo Expand to pass an actual file resource
 * @param string $file
 * @return string
 */
function getExtension($file) {
	if(strpos($file, '?')) {
		$file = explode('?', $file);
		$file = $file[0];
	}
	return substr(strtolower(strrchr($file, '.')), 1);
}

/**
 * Gets the filename from a url or string
 * <code>
 * $name = getFilename('testing_page.php?id=45'); // $name is 'testing_page.php'
 * </code>
 * @param string $filepath
 * @return string
 */
function getFilename($filepath) {
	preg_match('/[^?]*/', $filepath, $matches); 
	$string = $matches[0]; 
	$pattern = preg_split('/\./', $string, -1, PREG_SPLIT_OFFSET_CAPTURE); 
	$lastdot = $pattern[count($pattern)-1][1]; 
	$filename = basename($string);
	return $filename;
}

/**
 * Formats a raw byte size into a readable (4.5GB, 3MB, etc) size
 * <code>
 * $size = formatRawSize(filesize('somefile.jpg')); // 1.4GB
 * </code>
 * @param string|int $size
 * @return string
 */
function formatRawSize($size) {
	if($size == 0) {
	  return '0 bytes';
	}
	if(is_numeric($size)) {
		$s = array('bytes', 'kb', 'MB', 'GB', 'TB', 'PB');
		$e = floor(log($size) / log(1024));
		$output = sprintf('%.2f '.$s[$e], ($size / pow(1024, floor($e))));
		return $output;
	}
	return false;
}

/**
 * Converts php.ini-style '2M' etc into actual bytes
 * @param string $val
 * @return int
 */
function formatToBytes($val) {
	$val = trim($val);
	$last = strtolower($val[strlen($val)-1]);
	switch($last) {
		case 'g':
			$val *= 1024;
		case 'm':
			$val *= 1024;
		case 'k':
			$val *= 1024;
	}
	return $val;
}
?>