<?php
/**
 * Returns the first true argument, or false
 * Uses an == check
 * <code>
 * echo pick(false, null, 'Yeah', false); // 'Yeah'
 * </code>
 * @param mixed $x.. Any number of values
 */
function pick() {
	$args = func_get_args();
	if(!$args) return false;
	foreach($args as $arg) if(isset($arg) && $arg == true) return $arg;
	return false;
}

/**
 * Checks if a key exists inside an array or an object
 * <code>
 * var_dump(keyExists('test', array('test' => 'woo'))); // Returns true
 * </code>
 * @param string $key Key to check
 * @param array|object $data Array/object to look inside
 * @return bool
 * @see http://php.net/array_key_exists
 */
function keyExists($key, $data) {
	if(is_array($data)) {
		return array_key_exists($key, $data);
	} elseif(is_object($data)) {
		return array_key_exists($key, (array)$data);
	}
	return false;
}

/**
 * @todo Document this function
 */
function search($array, $expression) {
	$result = array();
	$expression = preg_replace("/([^\s]+?)(=|<|>|!)/", "\$a['$1']$2", $expression);
	foreach ($array as $a) if(eval("return {$expression};")) $result[] = $a;
	return $result;
}

/**
 * @todo Document this function
 */
function object_search($object, $expression) { 
	$result = array();
	$expression = preg_replace("/([^\s]+?)(=|<|>|!)/", "\$a['$1']$2", $expression);
	foreach($object as $a) {
		if($a instanceof Object) $a = $a->asArray();
		if(eval("return {$expression};")) $result[] = new Object($a);
	}
	return $result;
}

/**
 * @todo Document this function
 */
function array_keys_recursive($input, $search_value = null){ 
 $output = ($search_value !== null ? array_keys($input, $search_value) : array_keys($input));
 foreach($input as $sub) {
		if(is_array($sub)) {
			$output = ($search_value !== null ? array_merge($output, array_keys_recursive($sub, $search_value)) : array_merge($output, array_keys_recursive($sub)));
		}
	}
 return $output;
} 

/**
 * @todo Document this function
 */
function array_remove_keys($array, $keys) {
	if(is_object($array)) $array = (array)$array;
	foreach($array as $key => $value) {
		if(in_array($key, $keys)) unset($array[$key]);
	}
	return $array;
}

/**
 * @todo Document this function
 */
function array_empty($array) {
	if(empty($array)) return $array;
	$array = (array)$array;
	foreach($array as $key => $value) {
		if($value == '') unset($array[$key]);
	}
	return $array;
}

/**
 * @todo Document this function
 */
function array_closest($array, $num) {
	$difference = abs($num - $array[0]);
	$index = 0;
	$max = sizeof($array);
	for($i = 0; $i < $max; $i++) {
		$tmp = abs($num - $array[$i]);
		if($difference > $tmp) {
			$difference = $tmp;
			$index = $i;
		}
	}
	return $index;
}

/**
 * Shuffle the order of an array without losing key => value relationships
 * Based on the function by Antonio Ognio
 * <code>
 * $array = array(
	   'key' => 'value',
		 'something' => 'else',
		 'dog' => 'woof'
	 );
 	 echo debug(array_shuffle($array)); // Same key/value relationships, just ordered differently!
 * </code>
 * @param array $array Array of values
 * @return array
 * @see http://php.net/shuffle
 * @see http://php.net/manual/en/function.shuffle.php#83007
 */
function array_shuffle($array) {
	$aux = array();
	$keys = array_keys($array);
	shuffle($keys);
	foreach($keys as $key) {
		$aux[$key] = $array[$key];
		unset($array[$key]);
	}
	return $aux;
}

/**
 * Merges all arguments left to right into an Object
 * <code>
 * $obj = merge($options, $defaults); // The keys in $options will replace those in $defaults
 * </code>
 * @param mixed $data,...
 * @return Object The merged output as an Object
 */
function merge() {
	$output = new Object();
	$merging = func_get_args();
	$merging = array_reverse($merging, true);
	foreach((array)$merging as $merge) {
		if(!is_array($merge) && !is_object($merge)) continue;
		foreach($merge as $key => $value) $output->{$key} = $value;
	}
	return $output;
}

/**
 * @todo Document this function
 */
function array_merge_add_recursive($one, $two) {
	$return = array();
	foreach($one as $key => $value) {
		if(is_array($value) && is_array($two)) {
			$return[$key] = array_merge_add_recursive($value, $two[$key]);
		} elseif(is_object($value) && is_object($two)) {
			$return[$key] = array_merge_add_recursive($value, $two->{$key});
		} else {
			if(is_numeric($value)) {
				$return[$key] = $value;
				if(is_object($two)) {
					if(is_numeric($two->{$key})) $return[$key] += $two->{$key};
				} else if(is_numeric($two[$key])) {
					$return[$key] += $two[$key];
				}
			} else {
				$return[$key] = $value;
			}
		}
	}
	
	foreach($two as $key => $value) {
		if(is_array($value) && is_array($one)) {
			$return[$key] = array_merge_add_recursive($value, $one[$key]);
		} elseif(is_object($value) && is_object($one)) {
			$return[$key] = array_merge_add_recursive($value, $one->{$key});
		} else if(empty($return[$key])) {
			if(is_numeric($value)) {
				$return[$key] = $value;
				if(is_object($one)) {
					if(is_numeric($one->{$key})) $return[$key] += $one->{$key};
				} else if(is_numeric($one[$key])) {
					$return[$key] += $one[$key];
				}
			} else {
				$return[$key] = $value;
			}
		}
	}
	
	return $return;
}

/**
 * If the argument is an array, return it. If not, put the argument into an array
 * <code>
 * $var = splat('test'); // $var = array('test');
 * </code>
 * @param mixed $value,...
 * @return array
 */
function splat($value) {
	if(is_array($value)) return $value;
	return array($value);
}

/**
 * Grab a certain index from an array, or a key from an object
 * <code>
 * $data = array('one', 'two', 'three');
 * echo grab($data, 2); // Displays 'three'
 * </code>
 * @param array|object|Object $array
 * @param int $idx
 * @return mixed The value at $idx
 */
function grab($array, $idx = 0) {
	if(is_array($array)) {
		if(isset($array[$idx])) return $array[$idx];
	} elseif(is_object($array)) {
		if(array_key_exists($idx, $array)) return $array->{$idx};
	}
	return false;
}

/**
 * Fuses the contents of two arrays
 * Similar to array_merge_recursive() except 
 * <code>
 * $one = array(
 *   'key' => array(
 *      'test' => 'value',
 *      'woof' => 'something'
 *   )
 * );
 * $two = array(
 *   'key' => array(
 *      'woof' => 'something'
 *   )
 * );
 * print_r(array_fuse($one, $two));
 * </code>
 * @param array $array
 * @param array $fuse
 * @return array The fused arrays
 * @todo Finish the documentation
 */
function array_fuse($arr, $fuse) {
	if(is_array($arr)) {
		if(is_array($fuse)) {
			foreach($fuse as $k => $v) {
				if(isset($arr[$k]) && is_array($v) && is_array($arr[$k])) {
					$arr[$k] = array_fuse($arr[$k], $v);
				} else {
					$arr[$k] = $v;
				}
			}
		} elseif(!is_array($arr) && (strlen($arr) == 0 || $arr == 0)) {
			$arr = $fuse;
		}
	}
	ksort($arr); // This is to recreate SSN-based storage, mostly
	return $arr;
}
/* // This is 3x faster, but it makes arrays if both arrays have the same key, which is bad
function array_fuse($arr, $fuse) {
	$arr = array_merge_recursive($arr, $fuse);
	ksort($arr); // This is to recreate SSN-based storage, mostly
	return $arr;
}
*/

/**
 * Similar to array_key_exists, but allowing for multiple keys
 * <code>
 * $array = array(
 *   'key' => 'value',
 *   'test' => 'something'
 * );
 * if(array_keys_exist($array, array('test', 'hoof')) echo 'yeah'; // This will not show, cos 'hoof' isn't a key!
 * </code>
 * @param array $array
 * @param array|string $keys Array of keys to check exist, or a string (Will be splat-ed into an array)
 * @return bool
 */
function array_keys_exist($array, $keys) {
	if(!is_array($keys)) $keys = splat($keys);
	if(count(array_intersect($keys, array_keys((array)$array))) == count($keys)) {
		return true;
	}
	return false;
}

/**
 * Similar to array_key_exists, but allowing for multiple keys, and also checks that they are not null
 * <code>
 * $array = array(
 *   'key' => 'value',
 *   'test' => 'something'
 * );
 * if(array_keys_set($array, array('test', 'hoof')) echo 'yeah'; // This will not show, cos 'hoof' isn't a key!
 * </code>
 * @param array $array
 * @param array|string $keys Array of keys to check exist, or a string (Will be splat-ed into an array)
 * @return bool
 */
function array_keys_set($array, $keys) {
	if(!is_array($keys)) $keys = splat($keys);
	foreach($keys as $k) {
		if(!isset($array[$k]) || strlen($array[$k]) == 0) return false;
	}
	return true;
}

/**
 * Implodes an array by key
 * Will typecast anything else into an array too (i.e. Objects)
 * <code>
 * $array = array(
 *   array(
 *     'name' => 'Marc',
 *     'age' => 18
 *   ),
 *   array(
 *     'name' => 'Karl',
 *     'age' => '18'
 *   )
 * );
 * $array = implodeByKey($array, 'name', ','); // Returns "Marc,Karl"
 * </code>
 * @param mixed $value,...
 * @return array
 */
function implodeByKey($array, $key, $glue = '') {
	if(empty($array) || !is_array($array)) return false;
	$implode = array();
	foreach($array as $i) {
		$i = (array)$i;
		if(array_key_exists($key, $i)) array_push($implode, $i[$key]);
	}
	return implode($glue, $implode);
}

/**
 * Wrap all values of an array with a string
 * <code>
 * $array = array('name', 'age', 'dog');
 * $array = array_wrap($array, "'"); // Same as array("'name'", "'age'", "'dog'");
 * </code>
 * @param array $array
 * @param string $str The string to wrap
 * @return array
 */
function array_wrap($array, $str = '') {
	$array = splat($array);
	foreach($array as &$v) {
		$v = "{$str}{$v}{$str}";
	}
	return $array;
}

/**
 * Chunk an array into $p partitions of whatever size is necessary
 * <code>
 * $array = array('name', 'age', 'dog', 'test', 'value', testing');
 * $array = array_partition($array, 2); // Same as array(array('name', 'age', 'dog'), array('test', 'value', 'testing')) - 2 arrays
 * </code>
 * @param array $array
 * @param int $p
 * @return array
 */
function array_partition($array, $p) {
  $arraylen = count($array);
  $partlen = floor($arraylen / $p);
  $partrem = $arraylen % $p;
  $partition = array();
  $mark = 0;
  for($px = 0; $px < $p; $px++) {
		$incr = ($px < $partrem) ? $partlen + 1 : $partlen;
		$partition[$px] = array_slice($array, $mark, $incr);
		$mark += $incr;
  }
  return $partition;
}

/**
 * Flattens an array into an array of certain keys
 * Will typecast anything else into an array too (i.e. Objects)
 * <code>
 * $array = array(
 *   array(
 *     'name' => 'Marc',
 *     'age' => 18
 *   ),
 *   array(
 *     'name' => 'Karl',
 *     'age' => '18'
 *   )
 * );
 * $array = flattenByKey($array, 'name'); // Returns equivalent of array('Marc', 'Karl');
 * </code>
 * @param mixed $value,...
 * @return array
 */
function flattenByKey($array, $key) {
	if(!is_array($array)) $array = (array)$array;
	if(empty($array) || !is_array($array)) return false;
	$implode = array();
	foreach($array as $i) {
		$i = (array)$i;
		if(array_key_exists($key, $i)) array_push($implode, $i[$key]);
	}
	return $implode;
}

/**
 * Picks a random number between $floor and $ceiling (Inclusive)
 * Wrapper for the PHP function mt_rand
 * <code>
 * $randomNumber = random(4, 50); // $randomNumber will be between 4 and 50
 * </code>
 * @param int $floor
 * @param int $ceiling
 * @return int
 * @see http://php.net/mt_rand
 */
function random($floor = false, $ceiling = false) {
	return mt_rand($floor, $ceiling);
}

/**
 * Generates a random string
 * <code>
 * $randomString = randomStr(5, 'alpha'); // $randomString is 5 alphabetical characters long
 * </code>
 * @param int $length
 * @param string $type 'alphanumeric', 'alpha', 'num'
 * @return string
 */
function randomStr($length = 10, $type = 'alphanumeric') {
	if($type == 'alphanumeric') $chars = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789';
	elseif($type == 'alpha' || $type == 'letters') $chars = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ';
	elseif($type == 'num' || $type == 'numeric' || $type == 'numbers') $chars = '0123456789';
	elseif($type == 'chars') $chars = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789#*^!?<>_+-=';
	elseif($type == 'mixedalpha' || $type == 'mixedletters') $chars = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz';
	elseif($type == 'mixedalphanumeric') $chars = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789';
	$charLength = strlen($chars) - 1;
	for($i = 0; $i < $length; $i++) $string .= $chars{random(0, $charLength)};
	return $string;
}

/**
 * Decodes a loosely typed JSON string
 * <code>
 * decode("{key:'value', anotherKey:'I am some kind of value', marcIsCool:true}");
 * </code>
 * @param $string
 * @return mixed The decoded data
 */
function decode($string) {
	if(is_object($string) || is_array($string)) return $string;
	if($done = json_decode($string)) return $done;
	$str = str_replace("'", '"', $string);
	$str = preg_replace("/([\w]+?):\"/", "\"$1\":\"", $str);
	$str = preg_replace("/([\w]+?):(true|false)/", "\"$1\":$2", $str);
//$str = preg_replace("/([a-zA-Z0-9\[\]'!]+):('|true|false)/", "\"$1\":", $string);
//$str = preg_replace("/:'([a-zA-Z0-9:\\/]+)'/", ":\"$1\"", $str);
	$str = json_decode($str);
	return $str;
}

/**
 * Encodes data to a JSON string
 * @see php.net/json_encode
 * @param mixed $data
 * @return string The JSON string
 */
function encode($data) {
	return json_encode($data);
}

$_eternity_ssn_cache = array();

/**
 * Decodes an array of SSN (Simple Storage Notation) arrays into a standard array
 * @param mixed $data
 * @return array The array of info
 */
function ssn_decode($ssn) {
	$return = array();
	
	foreach($ssn as $k => $v) {
		$s = ssn_decode_single(array($k, $v));
		$return = array_fuse($return, $s);//array_merge_recursive($return, $s);
	}
	
	return $return;
}

/**
 * Decodes a single SSN (Simple Storage Notation) array into its' array format
 * @param mixed $data
 * @return array $ssn
 */
function ssn_decode_single($ssn_entry) {
	$hash = md5(print_r($ssn_entry, true));
	if(!empty($_eternity_ssn_cache[$hash])) return $_eternity_ssn_cache[$hash];
	
	$return = array();
	
	$k = $ssn_entry[0];
	$v = $ssn_entry[1];
	
	$root = before('[', $k);
		
	if(empty($root)) { // Nothing before [? Means [ doesn't exist, meaning it's its' own root
		$root = $k;
		$return[$root] = $v;
	} else {
		$return[$root] = array();
		$chunks = explode('[', $k);
		unset($chunks[0]); // Remove the first; this is always the root
		$last_chunk = trim(array_pop($chunks), ']');
		
		$append = &$return[$root]; // This is where the value will go
		
		if(!empty($chunks)) { // There are middle chunks - we need to pad the array
			foreach($chunks as $idx => $chunk) {
				$chunk = trim($chunk, ']');
				if(is_numeric($chunk)) { // Something is wrong here with numeric indexes; this has a ] on the end??
					$p = trim($chunks[$idx], ']');
					$append = &$append[$p];
				} else {
					$append = &$append[$chunk];
				}
			}
		}
		
		// Now we just append the last chunk to the end of the deepest thing inside $return; $append
		$append[$last_chunk] = $v;
		
		if($v == 'true' || $v == 'false') {
			$append[$last_chunk] = stringToBool($v);
		}
		
		if($v[0] == '{' || $v[0] == '[') {
			$append[$last_chunk] = json_decode($v);
		}
	}
	
	$_eternity_ssn_cache[$hash] = $return;
	
	return $return;
}

/**
 * Encodes data to an array of SSN (Simple Storage Notation) arrays
 * @param mixed $data
 * @return string The SSN string
 */
function ssn_encode($array) {
	
	$return = array();
	
	foreach($array as $k => $v) {
		ssn_encode_key($k, $v, $k, $return);
	}
	
	return $return;
}

/**
 * @todo Document this function
 */
function ssn_encode_key($k, $v, $key, &$return) {
	if(is_object($v) || is_array($v)) {
		foreach($v as $sk => $sv) {
			$rk = $key."[{$sk}]";
			ssn_encode_key($sk, $sv, $rk, $return);
		}
	} else {
		$return[$key] = $v;
	}
}

/**
 * Escapes a string or array/object
 * <code>
 * $secured = escape("I am going 'in' the database now! Oh, and spend some £.. or whatever");
 * </code>
 * @param string|array|object $value
 * @param bool $encode Encode the html entities or not
 * @return string
 */
function escape($value, $encode = false) {
	if(is_array($value) || is_object($value)) {
		foreach($value as &$v) $v = escape($v, $encode);
		return $value;
	}
	if($encode) return addslashes(htmlentities($value, ENT_COMPAT, 'UTF-8'));
	return addslashes($value);
}

/**
 * Unescapes a string or array/object
 * <code>
 * $unsecured = unescape("I am going \'in\' the database now! Oh, and spend some £.. or whatever");
 * </code>
 * @param string|array|object $value
 * @param bool $decode Decode the html entities or not
 * @return string
 */
function unescape($value, $decode = false) {
	if(is_array($value) || is_object($value)) {
		foreach($value as &$v) $v = unescape($v, $decode);
		return $value;
	}
	if($decode) return html_entity_decode(stripslashes($value), ENT_COMPAT, 'UTF-8');
	return stripslashes($value);
}
?>