<?php

// hash_hmac — Generate a keyed hash value using the HMAC method
// (PHP 5 >= 5.1.2, PECL hash >= 1.1)
if (!function_exists('hash_hmac')) {
	// based on: http://www.php.net/manual/en/function.sha1.php#39492
	function hash_hmac($algo, $data, $key, $raw_output = false) {
		$blocksize = 64;
			if (strlen($key) > $blocksize)
					$key = pack('H*', $algo($key));

		$key = str_pad($key, $blocksize, chr(0x00));
			$ipad = str_repeat(chr(0x36), $blocksize);
			$opad = str_repeat(chr(0x5c), $blocksize);
			$hmac = pack('H*', $algo(($key^$opad) . pack('H*', $algo(($key^$ipad) . $data))));

		return $raw_output ? $hmac : bin2hex($hmac);
	}
}

/**
 * Generates a human-readable password
 * @param int $length Must be an even number
 * @return array
 */
function generate_password($length = 6) {
	$consonants = array('b','c','d','f','g','h','j','k','l','m','n','p','r','s','t','v','w','x','y','z');
	$vowels = array('a','e','i','o','u');
	$password = '';
	srand((double)microtime()*1000000);
	$max = $length / 2;
	for($i=1; $i <= $max; $i++) {
		$password.= $consonants[rand(0,19)];
		$password.= $vowels[rand(0,4)];
	}
	$password.= random(0, 99);
	return $password;
}

/**
 * Transform a boolean into a string 'true', usually for direct echo into JavaScript
 * @param bool $bool
 * @return string
 */
function boolToString($bool) {
 if($bool === true || $bool === '1' || $bool === 'true') return 'true';
 return 'false';
}

/**
 * Transform a string 'true' into boolean true
 * @param string $string
 * @return bool
 */
function stringToBool($string) {
 return ($string == 'true' || $string === true);
}

/**
 * Returns an array of any valid urls found inside a string
 * @return array
 */
function getUrls($string) {
	preg_match_all('@(https?://([-\w\.]+)+(:\d+)?(/([\w/_\.\-]*(\?\S+)?)?)?)@', $string, $matches, PREG_SET_ORDER);
	return $matches;
}

/**
 * Returns a string with any urls replaced by html anchors
 * @param string
 * @param bool $newWindow If true, 'target="_blank"' will be added to the anchors matched
 * @return string
 */
function urlsToAnchors($string, $newWindow = false) {
	$replace = '<a href="$1">$1</a>';
	if($newWindow) $replace = '<a href="$1" target="_blank">$1</a>';
	return preg_replace('@(https?://([-\w\.]+)+(:\d+)?(/([\w/_\.\-]*(\?\S+)?)?)?)@', $replace, $string);
}

/**
 * Returns the string with multiple whitespaces collapsed into one
 * @return string
 */
function string_whitespace($string) {
  return trim(preg_replace("/\\s+/", ' ', $string));
}

/**
 * Returns the string with all whitespace removed
 * @return string
 */
function string_whitespace_strip($string) {
  return trim(preg_replace("/\\s+?/", '', $string));
}

/**
 * Returns any alphanumeric characters and spaces from a string, suitable for a name
 * @return string
 */
function string_name($string) {
	return trim(preg_replace("/[^a-zA-Z]\s0-9/", '', $string));
}

/**
 * Returns any alphanumeric characters from a string
 * @return string
 */
function string_alphanumeric($string) {
	return preg_replace("/[^a-zA-Z]/", '', $string);
}

/**
 * Returns any numeric characters from a string
 * @return string
 */
function string_numbers($string) {
	return preg_replace("/[^0-9]/", '', $string);
}

/**
 * Returns any numeric characters and . from a string
 * @return string
 */
function string_float($string) {
	return preg_replace("/[^0-9\.]/", '', $string);
}

/**
 * Substitutes keys with values
 * <code>
 * echo substitute("I am a {string}", array('string' => 'integer')); // returns "I am a integer"
 * </code>
 * @param string $string
 * @param array $data Multi-dimensional array of keys and values to replace
 * @return string
 */
function substitute($string, $data = array()) {
	if(empty($data)) return $string;
	foreach($data as $key => $value) $string = str_replace("{{$key}}", $value, $string);
	return $string;
}

/**
 * @todo Document this function
 */
function string_trim($str, $length, $suffix = '&hellip;', $prefix = false) {
	if(!$suffix) $suffix = '';
	if(!$prefix) $prefix = '';
	if(strlen($str) > $length) return $prefix.substr($str, 0, $length).$suffix;
	return $prefix.substr($str, 0, $length);
}

/**
 * @todo Document this function
 */
function plural($string) {
	$last = substr($string, -1);
	if($last == 's') return $string;
	return $string.'s';
}

/**
 * @todo Document this function
 */
function singular($string) {
	$last = substr($string, -1);
	if($last != 's') return $string;
	return substr($string, 0, -1);
}

/**
 * @todo Document this function
 */
function space($string) {
	return trim(strtolower(preg_replace('/[A-Z]+?/', ' $0', $string)));
}

/**
 * @todo Document this function
 */
function camelcase($string, $dontStrip = array()) {
	return lcfirst(str_replace(' ', '', ucwords(preg_replace('/^a-z0-9'.implode('',$dontStrip).']+/', ' ',$string))));
}

/**
 * @todo Document this function
 */
function truncate($string, $length) {
	if(strlen($string) > $length) {
		while($string{$length} != ' ') {
			$length--;
		}
		return substr($string, 0, $length);
	}
	return $string;
}

/**
 * @todo Document this function
 */
function after($this, $inthat) {
	if(!is_bool(strpos($inthat, $this)))
	return substr($inthat, strpos($inthat, $this) + strlen($this));
}

/**
 * @todo Document this function
 */
function after_last($this, $inthat) {
	if(!is_bool(strrevpos($inthat, $this)))
	return substr($inthat, strrevpos($inthat, $this) + strlen($this));
}

/**
 * @todo Document this function
 */
function before($this, $inthat) {
	return substr($inthat, 0, strpos($inthat, $this));
}

/**
 * @todo Document this function
 */
function before_last($this, $inthat) {
	return substr($inthat, 0, strrevpos($inthat, $this));
}

/**
 * @todo Document this function
 */
function between($this, $that, $inthat) {
	return before($that, after($this, $inthat));
}

/**
 * @todo Document this function
 */
function between_last ($this, $that, $inthat) {
	return after_last($this, before_last($that, $inthat));
}

/**
 * @todo Document this function
 */
function strrevpos($instr, $needle) {
	$rev_pos = strpos(strrev($instr), strrev($needle));
	if($rev_pos === false) return false;
	else return strlen($instr) - $rev_pos - strlen($needle);
}
?>
