<?php
require('application.php');
$eternity = new Eternity();

function ajaxErrorHandler($errno, $errstr, $errfile, $errline) {
	switch ($errno) {
		case E_USER_ERROR:
		case E_USER_WARNING:
			Ajax::failed($errstr);
			break;
		case E_NOTICE:
		case E_USER_NOTICE:
		case E_WARNING:
		case E_ERROR:
		case E_STRICT:
			//
			break;
    default:
			echo "Unknown error: [$errno] {$errstr}<br />\n";
			break;
	}
	// Kill the internal error handler
	return true;
}
set_error_handler('ajaxErrorHandler');

function test_database($config) {
	$data = array();
	parseData(json_decode($config), $data);
			
	if($data['database']['type'] == 'MySQL') {
		$mysql = new MySQL();
		$mysql->addEvent('onConnectFailed', 'test_database_connect_failed');
		$mysql->addEvent('onSelectFailed', 'test_database_select_failed');
		$mysql->addEvent('onSelectDatabase', 'test_database_select_success');
		$success = $mysql->connect($data['database']);
		
		if($success == true) Ajax::success('We have connected to your database successfully.');
	} else {
		Ajax::failed('This database type is not supported.');
	}
}

function test_database_select_success($event, $database) {
	//
}

function test_database_connect_failed($event, $config) {
	Ajax::failed("Could not connect using the username '{$config['username']}', with the password '{$config['password']}'. Please check your username and password.");
}

function test_database_select_failed($event, $config) {
	Ajax::failed("The database '{$config['database']}' could not be selected. Please check that it exists, and '{$config['username']}' has the required permissions.");
}

function test_s3($config) {
	$data = array();
	parseData(json_decode($config), $data);
	
	$data['misc']['s3_accesskey'] = trim($data['misc']['s3_accesskey']);
	$data['misc']['s3_secretkey'] = trim($data['misc']['s3_secretkey']);

	if(!empty($data['misc']['s3_accesskey']) && !empty($data['misc']['s3_secretkey'])) {
		$s3 = new S3_STD($data['misc']['s3_accesskey'], $data['misc']['s3_secretkey'], false);
		if($s3) {
			$evps3 = new EVP_S3($data['misc']['s3_accesskey'], $data['misc']['s3_secretkey'], '');
			$bucket = $evps3->createBucket();
			if(!empty($bucket)) {
				Ajax::success($bucket['bucket']);
			} else {
				Ajax::failed('Your default S3 bucket could not be created. Please make sure you have less than the maximum allowed by Amazon.');
			}
		} else {
			Ajax::failed('We could not connect to your S3 account. Please check your Access Key ID and Secret Key.');
		}
	}
	Ajax::success('');
}

$allowed = array('test_database', 'test_s3');

if(in_array($_POST['method'], $allowed)) {
	call_user_func_array($_POST['method'], array_remove_keys($_POST, array('method')));
} else {
	Ajax::failed(encode(new Object(array('msg' => 'Method not allowed.', 'type' => 'failed'))));
}
?>