CREATE TABLE IF NOT EXISTS `{$prefix}cache_users` (
  `cached` datetime NOT NULL,
  `ip` varchar(30) NOT NULL,
  `data` text NOT NULL,
  UNIQUE KEY `ip` (`ip`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

CREATE TABLE IF NOT EXISTS `{$prefix}chats` (
  `id` int(10) unsigned NOT NULL AUTO_INCREMENT,
  `operator_id` tinyint(10) unsigned NOT NULL,
  `file_ref` varchar(255) NOT NULL,
  `date` datetime NOT NULL,
  `uuid` varchar(60) NOT NULL,
  `client_name` varchar(150) NOT NULL,
  `client_email` varchar(100) NOT NULL,
  `client_country` varchar(255) NOT NULL,
  `url` varchar(255) NOT NULL,
  `status` tinyint(1) unsigned NOT NULL COMMENT '0 for awaiting op; 1 for awaiting client; 2 for closed',
  PRIMARY KEY (`id`),
  KEY `file_ref` (`file_ref`),
  KEY `operator_id` (`operator_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

CREATE TABLE IF NOT EXISTS `{$prefix}chats_msgs` (
  `id` int(100) unsigned NOT NULL AUTO_INCREMENT,
  `chat_id` int(10) unsigned NOT NULL,
  `date` datetime NOT NULL,
  `type` tinyint(1) unsigned NOT NULL COMMENT '0 for operator; 1 for client',
  `read` tinyint(1) unsigned NOT NULL COMMENT '0 for unread; 1 for read',
  `body` text NOT NULL,
  PRIMARY KEY (`id`),
  KEY `chat_id` (`chat_id`,`type`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

CREATE TABLE IF NOT EXISTS `{$prefix}comments` (
  `id` int(10) unsigned NOT NULL AUTO_INCREMENT,
  `parent_id` int(10) unsigned DEFAULT NULL,
  `file_ref` varchar(255) NOT NULL,
  `isOperator` tinyint(1) NOT NULL DEFAULT '0',
  `date` datetime NOT NULL,
  `name` varchar(150) NOT NULL,
  `email` varchar(150) NOT NULL,
  `body` text NOT NULL,
  `likes` text,
  PRIMARY KEY (`id`),
  KEY `parent_id` (`parent_id`,`file_ref`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

CREATE TABLE IF NOT EXISTS `{$prefix}conversions` (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `file_id` varchar(255) NOT NULL,
  `converted` tinyint(1) NOT NULL,
  `date` datetime NOT NULL,
  `name` varchar(150) NOT NULL,
  `price` double(10,2) NOT NULL,
  `upsell` tinyint(1) NOT NULL,
  `ip` varchar(30) NOT NULL,
  `country` varchar(50) NOT NULL,
  `region` varchar(50) NOT NULL,
  `city` varchar(50) NOT NULL,
  PRIMARY KEY (`id`),
  KEY `converted` (`converted`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

CREATE TABLE IF NOT EXISTS `{$prefix}files` (
  `idx` int(10) unsigned NOT NULL AUTO_INCREMENT,
  `id` varchar(255) NOT NULL,
  `parent_id` varchar(255) DEFAULT NULL,
  `child_type` varchar(30) DEFAULT NULL,
  `file_name` varchar(100) NOT NULL,
  `name` varchar(255) NOT NULL,
  `location` varchar(255) NOT NULL,
  `size` int(15) unsigned NOT NULL,
  `time` int(15) unsigned NOT NULL,
  `extension` varchar(5) NOT NULL,
  `type` varchar(10) NOT NULL,
  `source` tinyint(1) NOT NULL DEFAULT '1',
  PRIMARY KEY (`idx`),
  KEY `id` (`id`),
  KEY `parent_id` (`parent_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

CREATE TABLE IF NOT EXISTS `{$prefix}files_data` (
  `idx` int(10) unsigned NOT NULL AUTO_INCREMENT,
  `file_id` varchar(255) NOT NULL,
  `key` varchar(100) NOT NULL,
  `value` text NOT NULL,
  PRIMARY KEY (`idx`),
  KEY `key` (`key`),
  KEY `id` (`file_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

CREATE TABLE IF NOT EXISTS `{$prefix}folders` (
  `id` int(11) unsigned NOT NULL AUTO_INCREMENT,
  `parent_id` int(10) unsigned DEFAULT NULL,
  `file_count` int(10) NOT NULL DEFAULT '0',
  `name` varchar(150) NOT NULL,
  PRIMARY KEY (`id`),
  KEY `parent_id` (`parent_id`),
  KEY `file_count` (`file_count`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

CREATE TABLE IF NOT EXISTS `{$prefix}folders_relationships` (
  `idx` int(10) unsigned NOT NULL AUTO_INCREMENT,
  `folder_id` int(11) NOT NULL,
  `file_id` varchar(255) NOT NULL,
  PRIMARY KEY (`idx`),
  KEY `file_id` (`file_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

CREATE TABLE IF NOT EXISTS `{$prefix}groups` (
  `id` int(10) unsigned NOT NULL AUTO_INCREMENT,
  `seed` varchar(15) NOT NULL,
  `name` varchar(255) NOT NULL,
  PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

CREATE TABLE IF NOT EXISTS `{$prefix}groups_data` (
  `idx` int(10) unsigned NOT NULL AUTO_INCREMENT,
  `group_id` int(10) unsigned NOT NULL,
  `key` varchar(255) NOT NULL,
  `value` text NOT NULL,
  PRIMARY KEY (`idx`),
  KEY `group_id` (`group_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

CREATE TABLE IF NOT EXISTS `{$prefix}likes` (
  `idx` int(10) unsigned NOT NULL AUTO_INCREMENT,
  `type` varchar(15) NOT NULL COMMENT 'file, comment, chat, etc',
  `reference` varchar(255) NOT NULL,
  `name` varchar(200) DEFAULT NULL,
  `email` varchar(200) NOT NULL,
  `date` datetime NOT NULL,
  PRIMARY KEY (`idx`),
  KEY `type` (`type`,`reference`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

CREATE TABLE IF NOT EXISTS `{$prefix}playlists` (
  `id` int(10) NOT NULL AUTO_INCREMENT,
  `name` varchar(150) NOT NULL,
  `list_position` tinyint(1) unsigned NOT NULL DEFAULT '0' COMMENT '0 for none; 1 for left; 2 for right',
  `files` text NOT NULL,
  `labels` longtext NOT NULL,
  `descriptions` longtext NOT NULL,
  PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

CREATE TABLE IF NOT EXISTS `{$prefix}ratings` (
  `idx` int(10) unsigned NOT NULL AUTO_INCREMENT,
  `file_ref` varchar(255) CHARACTER SET latin1 NOT NULL,
  `uuid` varchar(50) CHARACTER SET latin1 NOT NULL,
  `date` datetime NOT NULL,
  `rating` tinyint(1) unsigned NOT NULL,
  PRIMARY KEY (`idx`),
  KEY `file_ref` (`file_ref`,`rating`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

CREATE TABLE IF NOT EXISTS `{$prefix}settings` (
  `idx` int(10) unsigned NOT NULL AUTO_INCREMENT,
  `name` varchar(255) NOT NULL,
  `value` text,
  PRIMARY KEY (`idx`),
  KEY `name` (`name`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

CREATE TABLE IF NOT EXISTS `{$prefix}sitemaps` (
  `id` smallint(5) unsigned NOT NULL,
  `domain` varchar(150) NOT NULL,
  PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

CREATE TABLE IF NOT EXISTS `{$prefix}sitemaps_data` (
  `idx` int(10) unsigned NOT NULL AUTO_INCREMENT,
  `sitemap_id` smallint(5) unsigned NOT NULL,
  `key` varchar(255) NOT NULL,
  `value` text NOT NULL,
  PRIMARY KEY (`idx`),
  KEY `sitemap_id` (`sitemap_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

CREATE TABLE IF NOT EXISTS `{$prefix}split_tests` (
  `id` int(10) NOT NULL AUTO_INCREMENT,
  `name` varchar(150) NOT NULL,
  `date_created` datetime NOT NULL,
  `date_ending` datetime DEFAULT NULL,
  `metric` varchar(20) DEFAULT NULL,
  `files` text NOT NULL,
  `best_file` varchar(255) DEFAULT NULL,
  PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

CREATE TABLE IF NOT EXISTS `{$prefix}surveys` (
  `idx` int(10) unsigned NOT NULL AUTO_INCREMENT,
  `file_ref` varchar(255) NOT NULL,
  `uuid` varchar(60) NOT NULL,
  `date` datetime NOT NULL,
  `survey_name` varchar(100) NOT NULL,
  `question_idx` tinyint(1) unsigned NOT NULL,
  `answer_idx` tinyint(1) unsigned NOT NULL,
  PRIMARY KEY (`idx`),
  KEY `file_ref` (`file_ref`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

CREATE TABLE IF NOT EXISTS `{$prefix}track` (
  `idx` int(10) unsigned NOT NULL AUTO_INCREMENT,
  `id` varchar(255) NOT NULL,
  `type` varchar(20) NOT NULL,
  `uuid` varchar(60) DEFAULT NULL,
  `player_time` float NOT NULL,
  `time` datetime NOT NULL,
  `referrer_root` varchar(255) NOT NULL,
  `referrer` text NOT NULL,
  `ip` varchar(30) NOT NULL,
  `country` varchar(50) NOT NULL,
  `region` varchar(50) NOT NULL,
  `city` varchar(50) NOT NULL,
  PRIMARY KEY (`idx`),
  KEY `id` (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

CREATE TABLE IF NOT EXISTS `{$prefix}track_progression` (
  `idx` int(10) unsigned NOT NULL AUTO_INCREMENT,
  `file_ref` varchar(255) NOT NULL,
  `uuid` varchar(60) NOT NULL,
  `ip` varchar(30) DEFAULT NULL,
  `player_time` tinyint(4) NOT NULL,
  `time` datetime NOT NULL,
  PRIMARY KEY (`idx`),
  KEY `file_ref` (`file_ref`),
  KEY `time` (`time`),
  KEY `uuid` (`uuid`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

CREATE TABLE IF NOT EXISTS `{$prefix}track_static` (
  `id` int(10) unsigned NOT NULL AUTO_INCREMENT,
  `file_ref` varchar(255) NOT NULL,
  `starts` int(10) unsigned DEFAULT NULL,
  `completes` int(10) unsigned DEFAULT NULL,
  `unique_starts` int(10) unsigned DEFAULT NULL,
  `unique_completes` int(10) unsigned DEFAULT NULL,
  `start_platform_desktop` int(10) unsigned DEFAULT NULL,
  `start_platform_ios` int(10) unsigned DEFAULT NULL,
  `start_platform_ios-legacy` int(10) unsigned DEFAULT NULL,
  `start_platform_android` int(10) unsigned DEFAULT NULL,
  `start_platform_msie` int(10) unsigned DEFAULT NULL,
  `start_platform_webkit` int(10) unsigned DEFAULT NULL,
  `start_platform_opera` int(10) unsigned DEFAULT NULL,
  `start_platform_firefox` int(10) unsigned DEFAULT NULL,
  PRIMARY KEY (`id`),
  UNIQUE KEY `file_ref` (`file_ref`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

CREATE TABLE IF NOT EXISTS `{$prefix}track_total_completes` (
  `idx` int(10) unsigned NOT NULL AUTO_INCREMENT,
  `file_ref` varchar(255) NOT NULL,
  `uuid` varchar(60) NOT NULL,
  `ip` varchar(30) NOT NULL,
  `month` varchar(10) NOT NULL,
  `day` varchar(9) NOT NULL,
  `hour` tinyint(2) NOT NULL,
  `time` datetime NOT NULL,
  `country` varchar(100) NOT NULL,
  `referrer` varchar(255) DEFAULT NULL,
  PRIMARY KEY (`idx`),
  KEY `file_ref` (`file_ref`),
  KEY `time` (`time`),
  KEY `uuid` (`uuid`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

CREATE TABLE IF NOT EXISTS `{$prefix}track_total_starts` (
  `idx` int(10) unsigned NOT NULL AUTO_INCREMENT,
  `file_ref` varchar(255) NOT NULL,
  `uuid` varchar(60) NOT NULL,
  `ip` varchar(30) NOT NULL,
  `month` varchar(10) NOT NULL,
  `day` varchar(9) NOT NULL,
  `hour` tinyint(2) NOT NULL,
  `time` datetime NOT NULL,
  `country` varchar(100) NOT NULL,
  PRIMARY KEY (`idx`),
  KEY `file_ref` (`file_ref`),
  KEY `time` (`time`),
  KEY `uuid` (`uuid`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

CREATE TABLE IF NOT EXISTS `{$prefix}track_unique_completes` (
  `idx` int(10) unsigned NOT NULL AUTO_INCREMENT,
  `file_ref` varchar(255) NOT NULL,
  `uuid` varchar(60) NOT NULL,
  `ip` varchar(30) NOT NULL,
  `month` varchar(10) NOT NULL,
  `day` varchar(9) NOT NULL,
  `hour` tinyint(2) NOT NULL,
  `time` datetime NOT NULL,
  `country` varchar(100) NOT NULL,
  PRIMARY KEY (`idx`),
  KEY `file_ref` (`file_ref`),
  KEY `uuid` (`uuid`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

CREATE TABLE IF NOT EXISTS `{$prefix}track_unique_starts` (
  `idx` int(10) unsigned NOT NULL AUTO_INCREMENT,
  `file_ref` varchar(255) NOT NULL,
  `uuid` varchar(60) NOT NULL,
  `ip` varchar(30) NOT NULL,
  `month` varchar(10) NOT NULL,
  `day` varchar(9) NOT NULL,
  `hour` tinyint(2) NOT NULL,
  `time` datetime NOT NULL,
  `country` varchar(100) NOT NULL,
  PRIMARY KEY (`idx`),
  KEY `file_ref` (`file_ref`),
  KEY `time` (`time`),
  KEY `uuid` (`uuid`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

CREATE TABLE IF NOT EXISTS `{$prefix}users` (
  `id` int(10) unsigned NOT NULL AUTO_INCREMENT,
  `seed` varchar(15) NOT NULL,
  `username` varchar(255) NOT NULL,
  `password` varchar(255) NOT NULL,
  PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

CREATE TABLE IF NOT EXISTS `{$prefix}users_data` (
  `id` int(255) unsigned NOT NULL AUTO_INCREMENT,
  `user_id` int(10) unsigned NOT NULL,
  `key` varchar(255) NOT NULL,
  `value` text NOT NULL,
  PRIMARY KEY (`id`),
  KEY `user_id` (`user_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

CREATE TABLE IF NOT EXISTS `{$prefix}users_relationships` (
  `id` int(255) unsigned NOT NULL AUTO_INCREMENT,
  `user_id` int(10) unsigned NOT NULL,
  `group_id` int(10) unsigned NOT NULL,
  PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

CREATE TABLE IF NOT EXISTS `{$prefix}users_relationships_data` (
  `id` int(255) unsigned NOT NULL AUTO_INCREMENT,
  `rel_id` int(255) unsigned NOT NULL,
  `key` varchar(255) NOT NULL,
  `value` text NOT NULL,
  PRIMARY KEY (`id`),
  KEY `relid` (`rel_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

CREATE TABLE IF NOT EXISTS `{$prefix}users_tracker` (
  `id` int(255) unsigned NOT NULL AUTO_INCREMENT,
  `user_id` int(10) NOT NULL,
  `useCookie` tinyint(1) unsigned NOT NULL,
  `sessionId` varchar(255) NOT NULL,
  `ip` varchar(15) NOT NULL,
  `lastActivity` timestamp NULL DEFAULT NULL,
  PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

CREATE TABLE IF NOT EXISTS `{$prefix}users_tracker_data` (
  `id` int(255) unsigned NOT NULL AUTO_INCREMENT,
  `tracker_id` int(10) unsigned NOT NULL,
  `key` varchar(255) NOT NULL,
  `value` text NOT NULL,
  PRIMARY KEY (`id`),
  KEY `trackerid` (`tracker_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

INSERT INTO `{$prefix}groups` (`id`, `seed`, `name`) VALUES(1, 'JFDL842NGKF83BA', 'Administrators');
INSERT INTO `{$prefix}settings` (`name`, `value`) VALUES('site_name', 'EasyVideoSuite');
INSERT INTO `{$prefix}settings` (`name`, `value`) VALUES('theme', '3.0');
INSERT INTO `{$prefix}settings` (`name`, `value`) VALUES('upgrade_evs1_1_survey', '1');