<?php
/**
 * Allows JavaScript to send a JSON string and call allowed PHP functions
 * @author Marc Fowler <marc.fowler@defraction.net>
 * @version 0.1
 * @package Eternity
 */
class Ajax extends Base {
	/**
	 * The allowed arguments
	 * @var array
	 */
	protected $args;

	/**
	 * Sets the functions which can be called through JavaScript, and calls them if the correct mode is specified
	 * The general idea is to write a requests.php file which contains functions to handle all of your AJAX requests
	 * This saves multiple instances in your JavaScript, and makes modification simpler
	 * <b>PHP</b>
	 * <code>
	 * new Ajax('aFunction', 'anotherFunction', array('SomeClass', 'function'));
	 * </code>
	 * <b>JS</b> (Using Request.Eternity for MooTools)
	 * <code>
	 * new Request.Eternity({url:'yourUrl.php'}).addEvent('onSuccess', function(responseText) {
	 *  alert(responseText);
	 * }).send(['SomeClass', 'function'], ['test', 'value']); // The second array is the arguments passed to the function
	 * </code>
	 * @param string|array $allowedFunctions, ... Strings or arrays of functions allowed to be called
	 */

	public function initialise() {
		$this->args = func_get_args();
		if(sizeof($this->args) == 1 && is_array($this->args[0])) $this->args = $this->args[0];
		if($_GET['AjaxRequest'] == 'ajax') {
			$function = json_decode($_POST['Function']);
			if(!$function) $function = $_POST['Function'];
			$function = trim(html_entity_decode($function), '" ');
			$args = json_decode($_POST['Arguments']);
			if(in_array($function, $this->args)) call_user_func_array($function, $args);
			else Ajax::failed("Error: The function '{$function}' is not allowed to be called via Ajax.");
		}
	}

	public function setResponseType($type = 'html') {
		switch($type) {
			case 'json':
				header('Content-type: application/json');
				break;
			default:
				header('Content-type: text/html');
				break;
		}
	}

	/**
	 * Fails the function and sends a 503 error to trigger the onFailure event in JS
	 * <code>
	 * Ajax::failed();
	 * Ajax::failed('Something went wrong');
	 * </code>
	 * @param string $string A string describing the error
	 */

	public static function failed($string = false) {
		header(' ', true, 503);
		if(is_array($string) || is_object($array)) {
			echo json_encode($string);
		} else {
			if($string) echo $string;
		}
		exit;
	}

	/**
	 * Marks the function as successful and sends a 200 OK header to trigger onSuccess in JS
	 * <code>
	 * Ajax::success();
	 * Ajax::success('Saved your changes');
	 * </code>
	 * @param string $string A string describing the successful action
	 */

	public static function success($string = false) {
		header(' ', true, 200);
		if(is_array($string) || is_object($array)) {
			echo json_encode($string);
		} else {
			if($string) echo $string;
		}
		exit;
	}
}
?>
