<?php
class API extends Base {
	public $defaults = array(
		'responseType' => 'json',
		'sendContentType' => false,
		'xmlRoot' => 'root'
	);
	
	public $responseData = array();
	public $sendContentType = false;
	
	public function initialise($options = false) {
		$this->setOptions($options);
		$this->setResponseType($this->options->responseType);
	}
	
	public function sendContentType($yes = true) {
		$this->options->sendContentType = $yes;
		return $this;
	}
	
	public function setResponseType($type) {
		$this->options->responseType = $type;
		return $this;
	}
	
	public function setXMLRoot($root = 'root') {
		$this->options->xmlRoot = $root;
		return $this;
	}
	
	public function addResponseData($data) {
		$this->responseData = array_merge($this->responseData, $data);
		return $this;
	}
	
	public function setResponseData($data) {
		$this->responseData = $data;
		return $this;
	}
	
	public function emptyResponseData() {
		$this->responseData = array();
		return $this;
	}
	
	public function array2xml($array, $xml = false) {
		if($xml === false) $xml = new SimpleXMLElement('<'.$this->options->xmlRoot.'/>');
		foreach($array as $key => $value) {
			if(is_array($value)) {
				$this->array2xml($value, $xml->addChild($key));
			} else {
				$xml->addChild($key, $value);
			}
		}
		return $xml->asXML();
	}
	
	public function respond() {
		switch($this->options->responseType) {
			case 'xml':
				if($this->options->sendContentType) header('Content-type: text/xml; charset=utf-8');
				echo $this->array2xml($this->responseData);
				break;
			case 'blank-image':
				header('Content-type: image/png');
				$img = imagecreatetruecolor(1, 1);
				imagepng($img);
				imagedestroy($img);
				break;
			case 'pretty':
				$response = $this->responseData;
				echo debug($response);
				break;
			case 'json':
				if($this->options->sendContentType) header('Content-type: application/json; charset=utf-8');
				$response = json_encode($this->responseData);
				echo $response;
				break;
			case 'jsonp':
				if($this->options->sendContentType) header('Content-type: application/javascript; charset=utf-8');
				$jsonp = $_REQUEST['callback'];
				$response = json_encode($this->responseData);
				echo "{$jsonp}({$response});";
				break;
		}
		
		$database = Registry::retrieve('database');
		if($database) $database->close();
		
	}
}
?>