<?php

/**
 * All database abstractions (Such as the default MySQL) must extend this to ensure compatibility
 * @author Marc Fowler <marc.fowler@defraction.net>
 * @version 0.1
 * @package Eternity
 * @see MySQL
 */

interface DatabaseInterface {
	public function initialise();
	public function connect($config);
	public function selectDatabase($database);
	public function query($query);
	public function getInsertId();
	public function getRow($query);
	public function getArray($query);
	public function close();
}

/**
 * Provides an anonymous method of calling database-related functions dependant on the mode
 * @author Marc Fowler <marc.fowler@defraction.net>
 * @version 0.1
 * @package Eternity
 * @see MySQL
 */

class Database extends Base {

	/**
	 * Allows the selected type of abstraction to be initialised
	 * <code>
	 * $db = new Database('mysql'); // If the class 'MySQL' exists, it will be used in place of this one
	 * Registry::get('database')->getArray('SELECT...');
	 * </code>
	 * @param string $type Used to identify the class to use (MySQL, SQLite..)
	 * @return mixed The initialised class
	 * @see Base
	 */
	 
	public function initialise($type) {
		if(class_exists($type)) {
			$this->klass = new $type;
			return $this->klass;
		}
	}
	
	public function __call($name, $arguments) {
		return call_user_func_array(array($this->klass, $name), $arguments);
	}
}

// Once a connection is established, these global functions will grab the 'main' configured database connection
// You'll need to grab your own reference if you want to use an alternative connection and use the class methods!

function query($query, $continueOnError = false) {
	$database = Registry::retrieve('database');
	if(!$database || !$database instanceof Database) return false;
	return $database->query($query, $continueOnError);
}

function getRow($query, $objType = false) {
	$database = Registry::retrieve('database');
	if(!$database || !$database instanceof Database) return false;
	return $database->getRow($query, $objType);
}

function getArray($query, $objType = false) {
	$database = Registry::retrieve('database');
	if(!$database || !$database instanceof Database) return false;
	return $database->getArray($query);
}

function getInsertId() {
	$database = Registry::retrieve('database');
	if(!$database || !$database instanceof Database) return false;
	return $database->getInsertId($query);
}

function getLastQuery() {
  $database = Registry::retrieve('database');
  if(!$database || !$database instanceof Database) return false;
  return $database->getLastQuery();
}

function getQueries() {
  $database = Registry::retrieve('database');
  if(!$database || !$database instanceof Database) return false;
  return $database->getQueries();
}
?>