<?php
/**
 * Provides the primary page control
 * Can be overridden to provide fallbacks
 * @author Marc Fowler <marc.fowler@defraction.net>
 * @version 0.1
 * @package Eternity
 */
class Page extends Base {
	public $defaults = array(
	  'theme' => 'default',
	  'version' => '1.0'
	);
	
	public function initialise($template_file, $master_file = 'master.php', $options = false) {
  $this->setOptions($options);
  $config = Registry::retrieve('config');
  $this->template = new Template($config['paths']['folder_base']."/themes/{$this->options->theme}/{$template_file}", false);
  $this->master = ($master_file ? new Template($config['paths']['folder_base']."/themes/{$this->options->theme}/{$master_file}", false) : false);;
  Registry::store('Page', $this);
  return $this;
	}
	
	public function javaScript($ref, $options = false) {
	 if(is_array($ref)) {
			foreach($ref as $i) {
				$this->javascripts[] = $i;
				$this->javascript_options[] = $options;
			}
	 } else {
			$this->javascripts[] = $ref;
			$this->javascript_options[] = $options;
		}
	 return $this;
	}
	
	public function js($ref, $options = false) { return $this->javaScript($ref, $options); }
	
	public function getJavaScript() {
	 if(empty($this->javascripts)) return '';
		$config = Registry::retrieve('config');
  $return = array();
  $scripts = $this->javascripts;
  foreach($scripts as $idx => $script) {
			$options = $this->javascript_options[$idx];
			if(substr($script, 0, 4) == 'http') {
				$c = "<script type=\"text/javascript\" src=\"{$script}?v={$this->options->version}\"></script>";
				if($options['position'] == 'start') array_unshift($return, $c);
				else $return[] = $c;
				continue;
			}
			if(stripos($script, '.js') !== false) {
		//if(substr($script, -3) == '.js') {
				$c = "<script type=\"text/javascript\" src=\"{$config['locations']['folder_absolute']}themes/{$this->options->theme}/".$script."?v={$this->options->version}\"></script>";
				if($options['position'] == 'start') array_unshift($return, $c);
				else $return[] = $c;
				continue;
			}
			// Do stuff depending on options
			if($options['domready'] === true) {
				$script = "window.addEvent('domready', function() { {$script} });";
			}
			$c = "<script type=\"text/javascript\">\n{$script}\n</script>";
			if($options['position'] == 'start') array_unshift($return, $c);
			else $return[] = $c;
  }
  return implode("\n", $return)."\n";
	}
	
	public function CSS($ref, $options = false) {
  if(is_array($ref)) {
			foreach($ref as $i) {
				$this->css[] = $i;
				$this->css_options[] = $options;
			}
  } else {
			$this->css[] = $ref;
			$this->css_options[] = $options;
  }
  return $this;
	}
	
	public function getCSS() {
	 if(empty($this->css)) return '';
		$config = Registry::retrieve('config');
  $return = array();
  $scripts = $this->css;
  foreach($scripts as $idx => $script) {
	  $options = $this->css_options[$idx];
			if(substr($script, 0, 4) == 'http') {
				$c = "<link rel=\"stylesheet\" href=\"{$script}?v={$this->options->version}\" />";
				if($options['position'] == 'start') array_unshift($return, $c);
				else $return[] = $c;
				continue;
			}
			if(stripos($script, '.css') !== false) {
		//if(substr($script, -4) == '.css') {
				$c = "<link rel=\"stylesheet\" href=\"{$config['locations']['folder_absolute']}themes/{$this->options->theme}/".$script."?v={$this->options->version}\" />";
				if($options['position'] == 'start') array_unshift($return, $c);
				else $return[] = $c;
				continue;
			}
			$c = "<style type=\"text/css\">\n{$script}\n</style>";
			if($options['position'] == 'start') array_unshift($return, $c);
			else $return[] = $c;
  }
  return implode("\n", $return)."\n";
	}
	
	public function removeCSS($remove) {
		if(empty($this->css)) return $this;
		foreach($this->css as $idx => $file) {
			if($file == $remove) unset($this->css[$idx]);
		}
		return $this;
	}
	
	public function title($title) {
  if(is_array($title)) {
			foreach($title as $t) $this->titles[] = $t;
  } else {
			$this->titles[] = $title;
  }
  return $this;
	}
	
	public function getTitle() {
	  return implode(' &raquo; ', (empty($this->titles) ? array('') : $this->titles));
	}
	
	public function meta($type, $value = '') {
  if(is_array($type)) {
			$this->meta[$type[0]] = $type[1];
  } else {
			$this->meta[$type] = $value;
  }
  return $this;
	}
	
	public function getMeta() {
  $meta = $this->meta;
  if(empty($meta)) return '';
  $return = '';
  foreach($meta as $type => $value) {
  	if(stripos($type, 'og:') === false) {
				$return[] = "<meta name=\"{$type}\" content=\"{$value}\" />";
			} else {
				$return[] = "<meta property=\"{$type}\" content=\"{$value}\" />";
			}
  }
  return implode("\n", $return)."\n";
	}
	
	public function getWidgetInclude($widget_name) {
		return getWidgetInclude($widget_name);
	}
	
	public function getWidget($widget_name, $data = array()) {
		return getWidget($widget_name, $data);
	}
	
	public function set() {
		$args = func_get_args();
		if(sizeof($args) == 1 && is_array($args[0])) {
			foreach($args[0] as $key => $value) {
				$this->template->set($key, $value);
			}
		} else {
			return $this->template->set($args[0], $args[1]);
		}
	}
	
	public function parse() {
	 if($this->master instanceof Template) {
			$this->fireEvent('onTemplateMerge', array($this->master, $this->template, $this));
			$variables = array_merge((array)$this->template->variables, (array)$this->master->variables);
			$html = str_replace('<!-- APP -->', $this->template->view, $this->master->view);
			$bufferContents = ob_get_clean();
			$view = new HTMLTemplate($html, true, $bufferContents);
			if(!empty($variables)) foreach($variables as $k => $v) $view->set($k, $v);
			$view->set('page', $this);
			echo $view->parse();
	 } elseif($this->template instanceof Template) {
			echo $this->template->parse();
	 }
	}
	
	public function __toString() {
	  return $this->parse();
	}
}
?>