<?php
/**
 * The global registry
 * @author Marc Fowler <marc.fowler@defraction.net>
 * @version 0.1
 * @package Eternity
 */
 
class Registry extends Singleton {
	/**
	 * The references held by the Registry
	 * @var array
	 */
	protected $references = array();
	protected $storage = array();
	public $registryStorage = array();
	
	/**
	 * Add a new class to the registry
	 * This is done automatically by all classes extending Base
	 * <code>
	 * Registry::set('SomeClass', $myInstance);
	 * </code>
	 * @param string $name Used to later {@link Registry::get()} from the registry
	 * @param mixed $reference Reference to the instance
	 * @return mixed The reference
	 * @see Base
	 */
	 
	public static function set($name, $reference) {
		$name = strtolower($name);
		$klass = self::fetch();
		if(!isset($klass->references[$name])) $klass->references[$name] = array();
		$n = sizeof($klass->references[$name]);
		$klass->references[$name][$n] = &$reference;
		$klass->storage[$name][$n] = new Object();
		$reference->internals->storage = &$klass->storage[$name][$n];
		return $klass->references[$name][$n];
	}
	
	/**
	 * Return the $n-th instance of specified class in the registry
	 * <code>
	 * $userManager = Registry::get('UserManager', 0);
	 * </code>
	 * @param string $name
	 * @param int $n If (bool)false, the array of all $name's will be returned, but defaults to the 0th
	 * @return mixed The reference or an array
	 */
	
	public static function get($name, $n = 0) {
		$name = strtolower($name);
		$klass = self::fetch();
		if(!isset($klass->references[$name])) return false;
		if(is_int($n)) return $klass->references[$name][$n];
		return $klass->references[$name];
	}
	
	/**
	 * Remove from the registry
	 * <code>
	 * Registry::remove('UserManager', 0);
	 * </code>
	 * @param string $name
	 * @param int $n If (bool)false, every reference by $name will be removed
	 * @return Registry
	 */
	
	public static function remove($name, $n = false) {
		$name = strtolower($name);
		$klass = self::fetch();
		if(is_int($n)) unset($klass->references[$n]->{$name}, $klass->storage[$n]->{$name});
		else unset($klass->references->{$name}, $klass->storage->{$name});
		return $this;
	}
	
	/**
	 * Saves a value in the Registry storage
	 * Used to contain global values such as configuration
	 * @param string $key
	 * @param mixed $value
	 * @return The $value
	 */
	 
	public static function store($key, $value) {
		$klass = self::fetch();
		$storage = &$klass->registryStorage;
		if(!is_object($storage)) $storage = new stdClass;
		$storage->{$key} = $value;
		return $value;
	}
	
	/**
	 * Return a key from the Registry storage
	 * @param string $key
	 * @param mixed $default If the key isn't found, return this instead (And set key to this)
	 */
	
	public static function retrieve($key, $default = false) {
		$klass = self::fetch();
		$storage = &$klass->registryStorage;
		if(isset($storage->{$key})) return $storage->{$key};
		$storage->{$key} = $default;
		return $storage->{$key};
	}
}
?>