<?php
/**
 * Provides a simple templating mechanism
 * Can be overridden to provide fallbacks
 * @author Marc Fowler <marc.fowler@defraction.net>
 * @version 0.2
 * @package Eternity2
 */
class Template extends Base {
	public $variables = array();
	public $buffer = false;
	public $bufferContents = '';
	
	public function isiPhone() {
		return !!strstr($_SERVER['HTTP_USER_AGENT'], 'iPhone');
	}
	
	public function isiPad() {
		return !!strstr($_SERVER['HTTP_USER_AGENT'], 'iPad');
	}
	
	public function isiOS() {
		return ($this->isiPhone() || $this->isiPad());
	}
	
	/**
	 * Load the content of the required view
	 * <code>
	 * $template = new Template('someFile.php');
	 * </code>
	 * @param string $name
	 * @param bool $buffer Turn on output buffering or not
	 */
	 
	public function initialise($name, $buffer = true, $bufferContents = false, $minifyInlineJS = false) {
		$this->name = $name;
		$this->minifyInlineJS = $minifyInlineJS;
		if(file_exists($name)) {
			$this->view = file_get_contents($name);
		} else {
			unset($this);
			return false;
		}
		if($buffer) {
			$this->fireEvent('onBeforeBuffer');
			$this->buffer = true;
			ob_start();
			$this->fireEvent('onBuffer');
		}
		if($bufferContents !== false) {
			$this->bufferContents = $bufferContents;
		}
	}
	
	/**
	 * Registers a variable to be used inside the template
	 * <code>
	 * $template->set('someVar', $someVar); // $someVar will be accessible inside the template
	 * </code>
	 * @param string $name
	 * @param mixed $value The reference to be accessed
	 * @return $this
	 */
	
	public function set($key, $value) {
		/*
		$args = func_get_args();
		if(sizeof($args) == 1 && is_array($args[0])) { // Array
			foreach($args[0] as $k => $v) $this->variables[$k] = &$v;
			return $this;
		} else {
			$key = $args[0];
			$value = $args[1];
		}
		$this->variables[$key] = &$value;
		$this->fireEvent('onSet', array($key, $value));
		return $this;
		*/
		$this->variables[$key] = $value;
		return $this;
		
	}
	
	/**
	 * Parses thetemplate and extracts the variables
	 * Inside the template itself, the variable '$outputBuffer' allows access to the current buffer
	 * @return The parsed template
	 * @see Template::set()
	 */
	
	public function parse() {
		$this->fireEvent('onBeforeParse', $this);
		$outputBuffer = '';
		if($this->buffer) $outputBuffer = ob_get_clean();
		if($this->bufferContents) $outputBuffer = $this->bufferContents;
		ob_start();
		extract($this->variables);
		$this->view = str_replace('<!-- BUFFER -->', $outputBuffer, $this->view);
		eval('?>'.$this->view);
		$view = ob_get_clean();
		
		/*
		if($this->minifyInlineJS === true && class_exists('JSMin')) {
			//
		}
		*/
		
		$this->fireEvent('onParse', array($this, $view));
		return $view;
	}
	
	public function __toString() {
		return $this->parse();
	}
}

class HTMLTemplate extends Template {
	public function initialise($html, $buffer = true, $bufferContents = '', $minifyInlineJS = false) {
		$this->view = $html;
		if($buffer) {
			$this->fireEvent('onBeforeBuffer');
			$this->buffer = true;
			ob_start();
			$this->fireEvent('onBuffer');
		}
		if($bufferContents !== false) {
			$this->bufferContents = $bufferContents;
		}
	}
}

function set($key, $value) {
	$template = Registry::retrieve('template');
	if(!$template || !$template instanceof Template) return false;
	return $template->set($key, $value);
}
?>