<?php
class Uploader extends Base {
	public $defaults = array(
		'overwrite' => true,
		'maxSize' => false,
		'required' => true,
		'mode' => 'normal',
		'key' => '', // Use this if you have to
		'rename' => true
	);
	
	public function initialise($key, $options = false) {
		$this->key = $key;
		$this->setOptions($options);
	}
	
	public function isEmpty() {
		if($this->options->mode == 'custom') return empty($_FILES['tmp_name']);
		return empty($_FILES[$this->key]);
	}
	
	public function upload($destination, $allowedFormats = array(), $filename = false) {
		if($this->options->mode == 'custom') {
		  return $this->upload_custom($destination, $allowedFormats, $filename);
		} else if($this->options->mode == 'key') {
		  return $this->upload_key($destination, $allowedFormats, $filename);
		}
		if($this->isEmpty() && $this->options->required) {
			$this->fireEvent('onFailure', array('No file has been uploaded.', true));
			return false;
		} elseif($this->isEmpty()) {
			$this->fireEvent('onEmpty');
			return false;
		}
		$config = Registry::retrieve('config');
		$this->destination = $config['paths']['base'].$destination;
		if(!is_dir($this->destination)) {
			if(!@mkdir($this->destination)) {
				$this->fireEvent('onFailure', 'The destination directory does not exist, and could not be created.');
				return false;
			}
		}
		if(!is_writeable($this->destination)) {
			if(!@chmod($this->destination, 0777)) {
				$this->fireEvent('onFailure', 'The destination directory could not be written to.');
				return false;
			}
		}
		$this->filename = $filename;
		$this->allowedFormats = $allowedFormats;
		$this->completed = array();
		// Ok, let's get going
		$files = $_FILES[$this->key];
		foreach($files['name'] as $idx => $name) {
			$error = false;
			switch ($files['error'][$idx]) {
				case 0 : break;
				case 1 : $error = $files['name'][$idx].' is larger than the maximum allowed filesize.'; break;
				case 2 : $error = $files['name'][$idx].' is larger than the maximum allowed filesize specified in the form.'; break;
				case 3 : $error = $files['name'][$idx].' was only partially uploaded.'; break;
				case 4 : $error = 'No file was uploaded.'; break;
				case 6 : $error = 'The temporary folder does not exist.'; break;
				case 7 : $error = 'Failed to write '.$files['name'][$idx].' to disk.'; break;
				case 8 : $error = $files['name'][$idx].' stopped by extension.'; break;
				default : $error = 'Unidentified error, caused by '.$files['name'][$idx].'.'; break;
			}
			if($files['size'][$idx] <= 0) $error = $files['name'][$idx].' is empty';
			$ext = getExtension($name);
			if(!in_array($ext, $this->allowedFormats)) {
				$this->fireEvent('onFailure', 'This file format is not allowed.');
				return false;
			}
			if($this->filename == false) { // Rename this file!
				$filename = substr($name, 0, -strlen($ext) - 1);
				if($this->options->rename == true) {
					$filename = makeSeed($filename).'-'.randomStr(5, 'alphanumeric').'.'.$ext;
				} else {
					$filename = makeSeed($filename).'.'.$ext;
				}
			} else {
				$filename = $this->filename.'.'.$ext;
			}
			if(file_exists($this->destination.$filename) && !$this->options->overwrite) $error = "The file {$filename} already exists.";
			if($this->options->maxSize && $files['size'][$idx] > $this->options->maxSize) $error = "{$name} is larger than the maximum allowed filesize.";
			if($error) {
				$this->fireEvent('onFailure', $error);
				return false;
			} else {
				if(@move_uploaded_file($files['tmp_name'][$idx], $this->destination.$filename)) {
					$details = array($idx, $this->destination.$filename, $filename);
					array_push($this->completed, array('key' => $idx, 'file' => $details[1], 'file_relative' => $details[2]));
					$this->fireEvent('onSuccess', $details);
				} else {
					$this->fireEvent('onFailure', 'This file could not be moved to its destination directory.');
					return false;
				}
			}
		}
		$this->fireEvent('onComplete', array($this->completed));
		return true;
	}
	
	private function upload_custom($destination, $allowedFormats = array(), $filename = false) {
		if($this->isEmpty() && $this->options->required) {
			$this->fireEvent('onFailure', array('No file has been uploaded.', true));
			return false;
		} elseif($this->isEmpty()) {
			$this->fireEvent('onEmpty');
			return false;
		}
		$config = Registry::retrieve('config');
		$this->destination = $config['paths']['base'].$destination;
		if(!is_dir($this->destination)) {
			if(!@mkdir($this->destination)) {
				$this->fireEvent('onFailure', 'The destination directory does not exist, and could not be created.');
				return false;
			}
		}
		if(!is_writeable($this->destination)) {
			if(!chmod($this->destination, 0777)) {
				$this->fireEvent('onFailure', 'The destination directory could not be written to.');
				return false;
			}
		}
		$this->filename = $filename;
		$this->allowedFormats = $allowedFormats;
		$this->completed = array();
		// Ok, let's get going
		$files = $_FILES[$this->key];
		foreach($files['name'] as $idx => $name) {
			$error = false;
			switch ($files['error'][$idx]) {
				case 0 : break;
				case 1 : $error = $files['name'][$idx].' is larger than the maximum allowed filesize.'; break;
				case 2 : $error = $files['name'][$idx].' is larger than the maximum allowed filesize specified in the form.'; break;
				case 3 : $error = $files['name'][$idx].' was only partially uploaded.'; break;
				case 4 : $error = 'No file was uploaded.'; break;
				case 6 : $error = 'The temporary folder does not exist.'; break;
				case 7 : $error = 'Failed to write '.$files['name'][$idx].' to disk.'; break;
				case 8 : $error = $files['name'][$idx].' stopped by extension.'; break;
				default : $error = 'Unidentified error, caused by '.$files['name'][$idx].'.'; break;
			}
			if($files['size'][$idx] <= 0) $error = $files['name'][$idx].' is empty';
			$ext = getExtension($name);
			if($this->filename == false) { // Rename this file!
				$filename = substr($name, 0, -strlen($ext) - 1);
				$filename = makeSeed($filename).'-'.randomStr(5, 'alphanumeric').'.'.$ext;
			} else {
				$filename = $this->filename.'.'.$ext;
			}
			if(file_exists($this->destination.$filename) && !$this->options->overwrite) $error = "The file {$filename} already exists.";
			if($this->options->maxSize && $files['size'][$idx] > $this->options->maxSize) $error = "{$name} is larger than the maximum allowed filesize.";
			if($error) {
				$this->error = $error;
				$this->fireEvent('onFailure', $error);
				return false;
			} else {
				if(@move_uploaded_file($files['tmp_name'][$idx], $this->destination.$filename)) {
					$details = array($idx, $this->destination.$filename, $filename);
					array_push($this->completed, array('key' => $idx, 'file' => $details[1], 'file_relative' => $details[2]));
					$this->fireEvent('onSuccess', $details);
				} else {
					$this->fireEvent('onFailure', 'This file could not be moved to its destination directory.');
					return false;
				}
			}
		}
		$this->fireEvent('onComplete', array($this->completed));
		return true;
	}
	
	public function upload_key($destination, $allowedFormats = array(), $filename = false) {
		if($this->isEmpty() && $this->options->required) {
		  $this->fireEvent('onFailure', array('No file has been uploaded.', true));
		  return false;
		} elseif($this->isEmpty()) {
		  $this->fireEvent('onEmpty');
		  return false;
		}
		$config = Registry::retrieve('config');
		$this->destination = $config['paths']['base'].$destination;
		if(!is_dir($this->destination)) {
		  if(!@mkdir($this->destination)) {
			$this->fireEvent('onFailure', 'The destination directory does not exist, and could not be created.');
			return false;
		  }
		}
		if(!is_writeable($this->destination)) {
		  if(!@chmod($this->destination, 0777)) {
			$this->fireEvent('onFailure', 'The destination directory could not be written to.');
			return false;
		  }
		}
		$this->filename = $filename;
		$this->allowedFormats = $allowedFormats;
		$this->completed = array();
		// Ok, let's get going
		$files = $_FILES[$this->key];
		foreach($files['name'] as $idx => $obj) {
		  $name = $obj[$this->options->key];
		  if(empty($name)) continue;
		  $error = false;
		  switch ($files['error'][$idx][$this->options->key]) {
			case 0 : break;
			case 1 : $error = $files['name'][$idx][$this->options->key].' is larger than the maximum allowed filesize.'; break;
			case 2 : $error = $files['name'][$idx][$this->options->key].' is larger than the maximum allowed filesize specified in the form.'; break;
			case 3 : $error = $files['name'][$idx][$this->options->key].' was only partially uploaded.'; break;
			case 4 : $error = 'No file was uploaded.'; break;
			case 6 : $error = 'The temporary folder does not exist.'; break;
			case 7 : $error = 'Failed to write '.$files['name'][$idx][$this->options->key].' to disk.'; break;
			case 8 : $error = $files['name'][$idx][$this->options->key].' stopped by extension.'; break;
			default : $error = 'Unidentified error, caused by '.$files['name'][$idx][$this->options->key].'.'; break;
		  }
		  if($files['size'][$idx] <= 0) $error = $files['name'][$idx][$this->options->key].' is empty';
		  $ext = getExtension($name);
		  if(!in_array($ext, $this->allowedFormats)) {
			$this->fireEvent('onFailure', 'This file format is not allowed.');
			return false;
		  }
		  if($this->filename == false) { // Rename this file!
			$filename = substr($name, 0, -strlen($ext) - 1);
			if($this->options->rename == true) {
			  $filename = makeSeed($filename).'-'.randomStr(5, 'alphanumeric').'.'.$ext;
			} else {
			  $filename = makeSeed($filename).'.'.$ext;
			}
		  } else {
			$filename = $this->filename.'.'.$ext;
		  }
		  if(file_exists($this->destination.$filename) && !$this->options->overwrite) $error = "The file {$filename} already exists.";
		  if($this->options->maxSize && $files['size'][$idx] > $this->options->maxSize) $error = "{$name} is larger than the maximum allowed filesize.";
		  if($error) {
			$this->fireEvent('onFailure', $error);
			return false;
		  } else {
			if(@move_uploaded_file($files['tmp_name'][$idx][$this->options->key], $this->destination.$filename)) {
			  $details = array($idx, $this->destination.$filename, $filename);
			  array_push($this->completed, array('key' => $idx, 'file' => $details[1], 'file_relative' => $details[2]));
			  $this->fireEvent('onSuccess', $details);
			} else {
			  $this->fireEvent('onFailure', 'This file could not be moved to its destination directory.');
			  return false;
			}
		}
	  }
	  $this->fireEvent('onComplete', array($this->completed));
	  return true;
	}
}
?>