EVP3.players.extend('EVP3.players.jwplayer', {
	'embed': function() {
		var player_api = this;
		var loc = this.player_location;
		var file = this.file;
		this.removed_cuepoints = [];
		var klass = this;

		var config = this.getConfig();

		if(EVP.helpers.isHTML5() && config.prettySource == 'youtube') {
			return new EVP3.players.youtube(this.container_element, this.video_element, this.file, this.evp);
		}

		var hasMetaData = !(file.metadata == false);

		if(hasMetaData) {
			setTimeout(function() {
				player_api.evp.fireEvent('onMetaData', [player_api.file.metadata, false]);
			}, 500);
		}

		var control_position = 'over';
		if(this.file.options.player_controls == 'hide') control_position = 'none';
		if(this.file.config.type == 'audio' || EVP.helpers.isAndroid() || EVP.helpers.isBlackberry()) control_position = 'bottom';
		if(this.file.options.player_controls == 'fauxlive') control_position = 'none';

		var icons = true;
		if(this.file.config.type == 'audio' && this.file.options.related_image_mode == 'splash' && this.file.options.related_image_play == false) icons = false;
		if(control_position == 'over' && (config.auto_buffer == true || config.auto_play == true)) icons = false; // Don't show the main play icon if we have controls and we're buffering

		// Set the related image?
		var related_image = false;
		if(typeof this.file.thumbnails != 'undefined' && typeof this.file.thumbnails[0] != 'undefined') {
			related_image = this.file.thumbnails[0];
		}

		if(EVP.helpers.isMobileOS() && this.file.options.related_image != false && typeof this.file.options.related_image != 'undefined') {
			related_image = this.file.options.related_image;
		}

		if(config.auto_buffer && !EVP.helpers.isMobileOS()) related_image = false; // No image if you're auto-buffering on desktop

		if(this.file.coverphoto) related_image = this.file.coverphoto;

		if(related_image && related_image.indexOf('/player/images/no-image.png') !== -1) related_image = false;

		var saveMetaData = function(klass) {
			if(!hasMetaData) {
				setTimeout(function() {
					var metaData = klass.getMeta();
					var meta = {
						'nativeWidth': metaData.width || 640,
						'nativeHeight': metaData.height || 360,
						'duration': metaData.duration || klass.getDuration(),
						'codec': {
							'framerate': metaData.videoframerate,
							'codec': metaData.videocodecid,
							'audioChannels': metaData.audiochannels,
							'audioCodec': metaData.audiocodecid,
							'audioSampleRate': metaData.audiosamplerate
						}
					};
					// Fire off a JSONP request?
					$.ajax({
						'url': EVP3.settings.api_location,
						'dataType': 'jsonp',
						'data': {'method': 'public-metadata-save', 'responseType': 'jsonp', 'file_id': file.file_id, 'metadata': meta},
						'complete': function() {
							// Fire an event
							player_api.file.metadata = meta;
							player_api.evp.fireEvent('onMetaData', [meta, true]);
						}
					});
				}, 200); // You have to wait so long for audio files with busted metadata
				hasMetaData = true;
			}
		};

		if(typeof this.file.options.watermark !== 'undefined' && this.file.options.watermark.indexOf('/player/images/no-image.png') !== -1) this.file.options.watermark_enabled = false; // Don't show the placeholder

		if(EVP.helpers.isMobileOS()) this.file.options.watermark_enabled = false; // Don't show the watermark on mobile devices

		var readyFired = player_api.evp.readyFired = false;

		var player_options = {
			'wmode': 'transparent',
			'width': (player_api.evp.isResponsiveVideo() ? '100%' : config.player_width),
			'height': (player_api.evp.isResponsiveVideo() ? '100%' : config.player_height),
			'autostart': (this.file.options.related_image_mode === 'lightbox' ? false : config.auto_play), // Only autoplay if it's not a lightbox
			'stretching': (this.file.options.player_stretching || 'exactfit'),
			'controlbar.position': control_position,
			'icons': icons,
			'playlist.position': 'none',
			'plugins': {},
			'logo.margin': 10,
			'logo.hide': true,
			'abouttext': 'EasyVideoSuite',
			'aboutlink': EVP3.settings.affiliate_url || 'http://easyvideosuite.com/',
			'events': {
				'onPlay': function() {
					var readyFired = player_api.evp.readyFired;
					if(!readyFired) player_api.evp.fireEvent('onPlayerReady');
					readyFired = player_api.evp.readyFired = player_api.evp.isPlayerReady = true;

					if(!player_api.evp.isResponsiveVideo()) { // This dirty hack prevents the video being wrongly sized in HTML5 players..
						$('#'+klass.container_id+'-video_video').css({'width': player_options.width, 'height': player_options.height});
					}
				},
				'onReady': function() {
					var readyFired = player_api.evp.readyFired;
					if(!readyFired) player_api.evp.fireEvent('onPlayerReady');
					readyFired = player_api.evp.readyFired = player_api.evp.isPlayerReady = true;
				},
				'onMeta': function() {
					saveMetaData(this);
				}
			}
		};

		if(player_api.evp.isResponsiveVideo()) player_options.stretching = 'uniform';

		player_options.modes = [{'type': 'html5', 'provider': 'video'}, {'type': 'flash', 'src': loc+'/player.swf?v=3'}];

		if(EVP.helpers.isMobileOS()) {
			player_options['controlbar.position'] = 'none';
			player_options.icons = true;
		} else {
			player_options.skin = config.player_skin; // Skins in HTML5 desktop mode (requires CORS on the bucket)
		}

		player_options.auto_buffer = true; // Always turn this on for mobiles as it dramatically improves their loading/initialization speed and prevents having to tap multiple times!
		if(EVP.helpers.isAndroid()) player_options.auto_buffer = false; // But off for Android as it messes up the loading spinner
		if(EVP.helpers.isMobileOS()) player_options.autostart = false;

		// Watermark support
		if(this.file.options.watermark_enabled && typeof this.file.options.watermark == 'string') {
			player_options['logo.position'] = this.file.options.watermark_position || 'bottom-left';
			player_options['logo.hide'] = false;
			player_options['logo.file'] = this.file.options.watermark;
			player_options['logo.link'] = '';
		}

		// Audio player support
		// Hide the play icon if there's no splash
		if(this.file.config.type == 'audio' && typeof config.player_image == 'undefined') {
			player_options.icons = false;
		}

		// Do we need to assign a child?
		var getsLowRes = (EVP.helpers.isMobileOS() || EVP.helpers.isFirefox());

		if(EVP.helpers.isFirefox() && (typeof this.file.children === 'undefined' || typeof this.file.children['480x360'] === 'undefined')) {
			if(typeof console !== 'undefined') console.info('Falling back to Flash player, as no child video is present!');
			player_options.modes = [player_options.modes[1]]; // Make Firefox use Flash if there isn't a child video
			EVP.helpers.isHTML5 = function() { // Override
				return false;
			};
		}

		var switchToChild = function(location, children) {
			var http_file = location;
			if(typeof children !== 'undefined') {
				if(typeof children['480x360'] != 'undefined' && getsLowRes === true) http_file = children['480x360'].location || base64_decode(children['480x360'].encoded_location);
			}
			return http_file;
		}

		if(this.file.playlist) { // Playlist support

			var list = [];
			$.each(this.file.playlist, function(idx, pl) {
				var location = switchToChild(pl.location || base64_decode(pl.encoded_location), pl.children);
				var obj = {'file': location, 'title': pl.label};
				list.push(obj);
			});

			player_options.playlist = list;
			player_options.repeat = 'list';
		} else { // Support for intro videos, or just straight videos
			var playlist_intro = {};
			var playlist_main = {'file': config.http_location};
			var playlist_outro = {};
			var list = [];

			var usingCF = false;
			if(config.rtmp_location && !EVP.helpers.isHTML5() && !this.file.options.disable_pause) { // Cannot use RTMP with disabled pausing as it doesn't work
				usingCF = true;
				player_options.provider = 'rtmp';
				player_options.streamer = 'rtmp://'+config.rtmp_domain+'/cfx/st';

				playlist_intro.provider = 'rtmp';
				playlist_intro.streamer = 'rtmp://'+config.rtmp_domain+'/cfx/st';

				playlist_main.provider = 'rtmp';
				playlist_main.streamer = 'rtmp://'+config.rtmp_domain+'/cfx/st';

				playlist_outro.provider = 'rtmp';
				playlist_outro.streamer = 'rtmp://'+config.rtmp_domain+'/cfx/st';
			}

			if(related_image != false) {
				playlist_intro.image = related_image;
				playlist_main.image = related_image;
				playlist_outro.image = related_image;
			}

			if(this.file.options.intro_video) {
				playlist_intro.file = (usingCF ? this.file.options.intro_video_config.rtmp_location : switchToChild(this.file.options.intro_video_location || base64_decode(this.file.options.intro_video_encoded_location), this.file.options.intro_video_children));
			}

			if(this.file.options.outro_video) {
				playlist_outro.file = (usingCF ? this.file.options.outro_video_config.rtmp_location : switchToChild(this.file.options.outro_video_location || base64_decode(this.file.options.outro_video_encoded_location), this.file.options.outro_video_children));
			}

			playlist_main.file = (usingCF ? config.rtmp_location : switchToChild(config.http_location, this.file.children));

			// Thumbnail support
			if(related_image !== false && typeof related_image != 'undefined') {
				playlist_main.image = related_image;
			}

			// Build the playlist itself
			if(playlist_intro.file) list.push(playlist_intro);
			if(playlist_main.file) list.push(playlist_main);
			if(playlist_outro.file) list.push(playlist_outro);

			player_options.playlist = list;
			player_options.repeat = 'list';
		}

		// Delayed start? Remember it must be specified for only the main video, hence some logic here
		if(config.delayed_start) {
			if(player_options.playlist) {
				if(this.file.options.intro_video && this.file.options.outro_video) {
					player_options.playlist[1].start = config.delayed_start;
				} else if(this.file.options.outro_video) {
					player_options.playlist[0].start = config.delayed_start;
				} else if(this.file.options.intro_video) {
					player_options.playlist[1].start = config.delayed_start;
				} else {
					player_options.playlist[0].start = config.delayed_start;
				}
			} else {
				player_options.start = config.delayed_start;
			}
		}

		// Lights out functionality?
		if(this.file.options.related_image_mode !== 'lightbox' && !EVP.helpers.isMobileOS()) {
			if(this.file.options.player_lightsout == 'on' || this.file.options.player_lightsout == true) {
				player_options.plugins = EVP3.settings.location+'/player/assets/jwplayer-5.10/plugins/lightsout/lightsout.js?v=14.6';
				player_options['lightsout.dockicon'] = false;
				player_options['lightsout.onidle'] = 'off';
				player_options['lightsout.onplay'] = 'off';
				player_options['lightsout.onpause'] = 'on';
				player_options['lightsout.oncomplete'] = 'on';
				player_options['lightsout.onfadeclick'] = 'on'; // What to do when clicking the black area itself?
			}
		}

		this.evp.addEvent('onBarBuilt', function(evt, bar) {
			$(bar).css('z-index', 301); // This pushes the bar so it's visible above the darkened overlay
		});

		var player = this.api = this.getAPI()(this.container_id+'-video').setup(player_options);

		// Desktop? Let's add a class when entering fullscreen
		if(!EVP.helpers.isMobileOS()) {
			klass.addEvent('fullscreen', function(evt) {
				player_api.evp.fireEvent('onFullscreen', [evt.fullscreen]);
				if(evt.fullscreen) {
					$('#'+klass.container_id).addClass('evs-fullscreen');
				} else {
					$('#'+klass.container_id).removeClass('evs-fullscreen');
				}
			});
		}

		// Are we a mobile device? If so, auto buffering is turned on and that causes splash images to not register a click event, so we have to remove it manually
		if(EVP.helpers.isMobileOS()) {
			klass.addEvent('play', function() {
				$('#'+klass.container_id).find('.evp-splash-wrap').remove();
			});
		}

		// Should we be autoplaying? Let's make sure it works
		if(player_options.autostart && !EVP.helpers.isMobileOS() && this.file.options.related_image_mode !== 'lightbox' && (this.file.options.player_lightsout !== 'on' && this.file.options.player_lightsout !== true)) {
			klass.addEvent('ready', function() {
				var x = 0, i = setInterval(function() {
					if($('#'+klass.container_id).height() > 0) {
						player.play();
						if(player.getState() === 'PLAYING') {
							clearInterval(i);
						}
					}
					x++;
					if(x > 200) clearInterval(i);
				}, 10);
			});
		}

		// Some audio files have broken data and onMeta doesn't fire
		// This should trigger the getDuration manually for them after 2 seconds of playing
		if(this.file.config.type == 'audio') {
			this.addEvent('play', function() {
				var klass = this;
				setTimeout(function() {
					saveMetaData(klass);
				}, 2000);
			});
		}

		// Disable pausing?
		if(this.file.options.disable_pause) {
			player.onPause(function(event) {
				if(event.oldstate !== 'BUFFERING') {
					this.play(true);
				}
			});
		}

		// Extra playlist stuff for 'roll' videos
		klass.currentRollItem = false;

		if(this.file.options.intro_video || this.file.options.outro_video) {
			klass.currentRollItem = 0;
			var offsetRollItem = 0;
			if(!this.file.options.intro_video) {
				klass.currentRollItem = 1;
				offsetRollItem = 1;
			}
			this.addEvent('playlistItem', function(pl) {
				klass.currentRollItem = pl.index + offsetRollItem;
			});
		}

		// We autobuffering the video? Faux autobuffering for JWP
		// Only if lights out isn't on
		if(!EVP.helpers.isMobileOS() && this.file.options.related_image_mode !== 'lightbox' && control_position !== 'none' && (this.file.options.player_lightsout !== 'on' && this.file.options.player_lightsout !== true)) {
			if(config.auto_buffer == true && config.auto_play != true) {
				var fauxBuffered = false;
				this.addEvent('play', function() {
					if(!fauxBuffered) player.pause();
					fauxBuffered = true;
				});
				player.play();
			}
		}

		// Pause when focus is lost?
		if(this.file.options.pause_focus_lost == true && !EVP.helpers.isMobileOS()) {
			try {
				var pause = function() {
					if(typeof player !== 'undefined' && player.getState() == 'PLAYING') {
						player.play(false); // Pause it
						$(window).focus(function() {
							player.play(true); // Resume it
						});
					} else {
						$(window).unbind('focus');
					}
				}
				$(window).bind('blur.evp', pause);
			} catch(e) {
				EVP.log('Exception thrown with focus events', e, 'error');
			}
		}

		// Set the volume?
		if(!EVP.helpers.isHTML5()) player.setVolume(config.volume);

		if(EVP.helpers.isAndroid()) {
			var playTimer = 0;
			var playInt;
			this.addEvent('play', function() {
				playTimer = 0;
				playInt = setInterval(function() {
					playTimer++;
				}, 995);
			});
		}

		// Statistic tracking
		this.addEvent('play', function(args) {
			if((klass.currentRollItem === 0)) return; // Don't track for the intro
			if(args.oldstate != 'PAUSED') {
				var platform = EVP.helpers.getPlatform();
				klass.evp.track('start', {'uuid': EVP3.settings.uuid, 'platform': platform});
			}
		});

		this.addEvent('complete', function() {
			if((klass.currentRollItem === 0)) return; // Don't track for the intro

			var time = this.getPosition();
			if(typeof playTimer != 'undefined') {
				time = playTimer;
				clearInterval(playInt); // Stop the Android timer
			}

			// This section basically turns off complete tracking for Android only if the current timer is more than 5 seconds away from the duration of the video
			// because complete events are fired even if it's just a stop-start
			// hopefully we can port this to only legacy Android devices rather than all
			var shouldTrack = true;
			if(typeof playTimer != 'undefined' && typeof klass.file.metadata != 'undefined' && (Math.ceil(parseInt(klass.file.metadata.duration || 0)) - time) >= 5) shouldTrack = false;

			if(shouldTrack) klass.evp.track('complete', {'uuid': EVP3.settings.uuid, 'time': time, 'referrer': $.base64Encode(window.location.href)});
		});
	},
	'getConfig': function() {
		var config = this._super();
		config = $.extend(true, config, {
			'player_skin': this.player_location+'/skins/'+(config.player_skin)+'/'+(config.player_skin)+'.xml?v=1.2'
		});
		return config;
	},
	'addEvent': function(event, callback) {
		var ucEvent = event.charAt(0).toUpperCase() + event.substr(1);
		return this.api['on'+ucEvent](callback);
	},
	'getState': function() { // Standardised way of grabbing the current state
		var state = this.api.getState();
		var ret = '';

		if(state == 'PLAYING') ret = 'playing';
		else if(state == 'PAUSED') ret = 'paused';
		else if(state == 'BUFFERING') ret = 'buffering';

		return ret;
	},
	'onSeek': function(cb) {
		return this.api.onSeek(cb);
	},
	'onCuepoint': function(time, callback) { // Brilliant. Don't forget time is a single-second, not ms
		var klass = this;

		time = Math.floor(time);

		var obj = {'time': time, 'callback': callback};
		klass.removed_cuepoints.push(obj);

		var exec = false;
		var r = function() { exec = false; };
		this.api.onSeek(r).onComplete(r); // Bind these so local r gets reset
		this.api.onTime(function(event) {
			var second = Math.floor(event.position);
			if((klass.currentRollItem === 1 || klass.currentRollItem === false) && second == time && exec === false && $.inArray(obj, klass.removed_cuepoints) !== -1) { // Only fire it once!
				exec = true;
				callback(second);
			}
		});
	},
	'removeCuepoint': function(time, callback) {
		var klass = this;

		time = Math.floor(time);

		var obj = {'time': time, 'callback': callback};
		klass.removed_cuepoints.push(obj);

		return this;
	}
});
