<?php
// This file can access $file, $file_settings and $plugin
// You can assign things to $plugin if you need to
// File-specific stuff goes in $plugin->file_options
?>
EVP3.players.extend('EVP3.plugins.<?php echo $plugin->slug; ?>', {
	'init': function(evp, file, plugin, file_options) {
		this.evp = evp;
		this.plugin = plugin;
		this.file = file;
		this.file_options = file_options;
		this.fn_list = [];
		var klass = this;
		
		this.maximum = 10;
		
		this.recordingActive = ($.cookie('evp_prog_'+klass.file.active_file_ref) !== 'true');
		
		if(this.recordingActive) {
			// Now, are we using a playlist? If so, we need to implement this for all videos in a list
			if(this.file.playlist) {
				this.file.player.api.onPlaylistItem(function(d) {
					var idx = d.index;
					var entry = klass.file.playlist[idx];
					
					if(!entry || parseInt(entry.raw_duration) <= 0) return false;
					
					klass.detach();
					klass.calculate(parseInt(entry.raw_duration));
					klass.attach();
					
				});
			} else {
				this.evp.addEvent('onMetaData', function() {
					klass.calculate();
					klass.attach();
				});
			}
		}
	},
	'calculate': function(duration) {
		var duration = Math.ceil(duration || this.file.metadata.duration);
		if(duration < 10) return; // We won't track videos less than 10 seconds because it gets stupid
		
		var interval = (duration / this.maximum); // Let's fire a maximum of X intervals, so when should they happen?
		
		this.interval = interval;
		this.duration = duration;
	},
	'attach': function() {
		var klass = this;
		var counter = 1;
		var hasCookied = false;
		
		for(x = this.interval; x < this.duration; x+=this.interval) {
			var fn = function(current_time) {
				if(!hasCookied) { // Only set the cookie that we have played this file the one time
					var exp = new Date();
					exp.setHours(exp.getHours() + 4500);
					$.cookie('evp_prog_'+klass.file.active_file_ref, 'true', {'expires': exp, 'path': '/'});
					hasCookied = true;
				}
				
				klass.evp.track('progress', {'uuid': EVP3.settings.uuid, 'time': counter, 'player_time': current_time});
				counter++;
			};
			
			var obj = {'time': x, 'callback': fn};
			
			klass.fn_list.push(obj);
			
			klass.evp.file.player.onCuepoint(x, fn);
		}
		
		var fired = [];
		klass.evp.file.player.addEvent('play', function() {
			if(fired.indexOf('start') !== -1) return;
			
			klass.evp.track('progress', {'uuid': EVP3.settings.uuid, 'time': 0, 'player_time': 0});
			
			fired.push('start');
		});
		
		klass.evp.file.player.addEvent('complete', function() {
			if(fired.indexOf('complete') !== -1) return;

			klass.evp.track('progress', {'uuid': EVP3.settings.uuid, 'time': 10, 'player_time': klass.duration});

			fired.push('complete');
		});
	},
	'detach': function() {
		var klass = this;
		$.each(klass.fn_list, function(idx, obj) {
			klass.evp.file.player.removeCuepoint(obj.time, obj.callback);
		});
	}
});