<?php
class EVP_API extends Project {
	public function initialise() {
		$this->evp = Registry::get('EVP');
	}

	public function authenticate($username, $password) {
		$xs = Registry::retrieve('users');
		$username = strtolower(trim($username));
		$password = trim($password); // Matches SHA1 of password
		$user = $xs->getUser($username, 'username');
		if($user) {
			$user->group = grab($user->groups, 0);
		}
		if($user && $user->password == $password){/* && ($user->group->name == 'Administrators' || $user->group->name == 'Managers' || $user->group->name == 'Users')) {*/
			$user->isClient = ($user->group->name != 'Administrators' && $user->group->name != 'Managers');
			return $user;
		}
		return false;
	}

	public function empty_cache() {
		$config = Registry::retrieve('config');

		$cache_files = getFiles($config['paths']['cacheDir']);
	 if(empty($cache_files)) return 0;

	 $total = sizeof($cache_files);

	 foreach($cache_files as $file) {
	 	unlink($file);
	 }

	 return $total;
	}

	public function empty_video_cache() {
		// Empty the _files table
		query("DELETE FROM {$this->evp->options->prefix}files WHERE `parent_id` IS NULL");

		// Delete any metadata keys
		query("DELETE FROM {$this->evp->options->prefix}files_data WHERE `key` LIKE 'metadata[%'");

		return true;
	}

	public function live_chat_prune() {
		$evp = $this->evp;

		$toMarkAsClosed = array();
		$doNotClose = array();

		$floor = gmdate('Y-m-d H:i:s', gmstrtotime('-10 minutes'));

		// First, let's get all unclosed chats which were started below the floor
		$activeChats = getArray("SELECT `id` FROM {$evp->options->prefix}chats WHERE `status` != '2' && `date` <= '{$floor}'");
		if(!empty($activeChats)) {

			$floor = gmstrtotime('-10 minutes');

			// Now let's get the latest messages for each chat
		//$latestMessages = getArray("SELECT `chat_id`, UNIX_TIMESTAMP(`date`) AS `date`, `body` FROM {$evp->options->prefix}chats_msgs WHERE `type` = '1' && `chat_id` IN(".implodeByKey($activeChats, 'id', ',').") GROUP BY `chat_id` ORDER BY `date` DESC");
			$latestMessages = getArray("SELECT `chat_id`, MAX(UNIX_TIMESTAMP(`date`)) AS date FROM {$evp->options->prefix}chats_msgs WHERE `chat_id` IN(".implodeByKey($activeChats, 'id', ',').") GROUP BY `chat_id` ORDER BY `date` DESC");

			// And check for ones that were sent under the floor
			if(!empty($latestMessages)) {
				foreach($latestMessages as $m) {
					if($m->date <= $floor) $toMarkAsClosed[] = $m->chat_id;
					else $doNotClose[] = $m->chat_id;
				}
			}

			foreach($activeChats as $c) if(!in_array($c->id, $doNotClose)) $toMarkAsClosed[] = $c->id;
		}

		if(!empty($toMarkAsClosed)) {
			query("UPDATE {$evp->options->prefix}chats SET `status` = '2' WHERE `id` IN (".implode(',', array_unique($toMarkAsClosed)).")");
		}
	}

	public function files_import($url) {
		if(empty($url)) return false;
		$evp = $this->evp;
		$config = Registry::retrieve('config');

		if(strtolower(substr($url, 0, 8)) !== 'https://' && strtolower(substr($url, 0, 7)) !== 'http://') {
			$url = 'http://'.$url;
		}

		// Check if it's a YouTube embed URL
		if(stripos($url, 'youtube.com/embed/') !== false || stripos($url, 'youtu.be') !== false) {
			$url = trim($url, '/');
			$ytd = explode('/', $url);
			$ytid = array_pop($ytd);
			$url = 'http://youtube.com/watch?v='.$ytid;
		}

		// Check if it's a remote YouTube video
		if(stripos($url, 'youtube.com') !== false) { // Huzzah
			parse_str(parse_url($url, PHP_URL_QUERY), $yt_vars);
			if(!empty($yt_vars) && !empty($yt_vars['v'])) {
				$youtube = true;
				$youtube_id = $yt_vars['v'];
				// Let's try to grab the name quickly
				$req = new Request_Curl(array('timeout' => 25, 'url' => 'http://www.youtube.com/watch'));
				$yt_page = $req->send(array('v' => $youtube_id));
				if(!empty($yt_page)) {
					preg_match('/<title>([^>]*)<\/title>/si', $yt_page, $yt_title);
					if(!empty($yt_title) && !empty($yt_title[1]) && strtolower(string_whitespace($yt_title[1])) != 'youtube - broadcast yourself.') {
						$file_name = $yt_title[1];
					} else {
						return false;
					//$api->setResponseData(array('success' => false, 'message' => 'The title could not be extracted; the video likely does not exist or is unavailable.'))->respond();
					}
				} else {
					return false;
				//$api->setResponseData(array('success' => false, 'message' => 'Your server could not communicate with YouTube.'))->respond();
				}
				if(!isset($yt_title)) { // We did not modify the name in any way
					$file_name = 'YouTube Imported Video';
				}
				$file_name = str_replace('youtube---', '', makeSeed($file_name));
				$extension = 'flv';
				$data = array(
					'page_seed' => makeSeed($file_name)
				);
			}
		} else {
			$file_name = getFilename($url);
			$extension = getExtension($url);
			$data = array(
				'page_seed' => makeSeed($file_name)
			);
		}

		// Let's check for existing files with this name
		$idx = 1;
		$o_file_name = ''.$file_name;
		$use_this_name = $file_name.= '-'.$idx;
		$use_this_id = base64_encode($use_this_name);
		while(getRow("SELECT `id` FROM {$evp->options->prefix}files WHERE `id` = '{$use_this_id}'")) {
			$idx++;
			$use_this_name = $o_file_name.= '-'.$idx;
			$use_this_id = base64_encode($use_this_name);
		}

		$file = new Object(array(
			'id' => $use_this_id,
			'name' => $file_name,
			'file_name' => $file_name,
			'time' => gmdate('U'),
			'size' => '',
			'extension' => $extension,
			'type' => 'video',
			'location' => $url,
			'data' => $data,
			'source' => ($youtube ? 3 : 4)
		));

		$evp->insertMySQLObject('files', $file);

		return $file;
	}

	public function files_stats_remote_youtube($file_id) {
		if(empty($file_id)) return false;
		$evp = $this->evp;
		$config = Registry::retrieve('config');
		$obj = $evp->getFile($file_id);
		if(!$obj) return false;

		// Let's get the v param from the location
		parse_str(parse_url($obj->location, PHP_URL_QUERY), $qs);
		if(empty($qs)) return false;

		// Let's get the XML from YT
		$req = new Request_Curl(array('timeout' => 25, 'url' => 'http://gdata.youtube.com/feeds/api/videos/'.$qs['v']));
		$result = $req->send(array('v' => 2));
		if(empty($result)) return false;

		// Parse the result
		$xml = simplexml_load_string($result);

		$yt = $xml->children('yt', true);//'http://gdata.youtube.com/schemas/2007');
		$attrs = array();

		foreach($yt as $k => $c) $attrs[(string)$k] = new Object($c->attributes());

		// Format the stats
		$return = array(
			'favourites' => (string)$attrs['statistics']->favoriteCount,
			'views' => (string)$attrs['statistics']->viewCount,
			'likes' => (string)$attrs['rating']->numLikes,
			'dislikes' => (string)$attrs['rating']->numDislikes
		);

		return $return;
	}

	public function files_metadata_remote_youtube($file_id) {
		if(empty($file_id)) return false;
		$evp = $this->evp;
		$config = Registry::retrieve('config');
		$obj = $evp->getFile($file_id);
		if(!$obj) return false;

		// Let's get the v param from the location
		parse_str(parse_url($obj->location, PHP_URL_QUERY), $qs);
		if(empty($qs)) return false;

		// Let's get the XML from YT
		$req = new Request_Curl(array('timeout' => 25, 'url' => 'http://gdata.youtube.com/feeds/api/videos/'.$qs['v']));
		$result = $req->send(array('v' => 2));
		if(empty($result)) return false;

		// Parse the result
		$xml = simplexml_load_string(trim($result));

		$yt = $xml->children('media', true);
		$attrs = ($yt->group->content ? $yt->group->content->attributes() : array());

		// Format the metadata, and default to a 360p sizing
		$return = new Object(array(
			'duration' => (string)$attrs['duration'],
			'nativeWidth' => 640,
			'nativeHeight' => 360
		));

		return $return;
	}

	public function files_rename($file_id, $new_name) {
		if(empty($file_id)) return false;
		$evp = $this->evp;
		$config = Registry::retrieve('config');
		$obj = $evp->getFile($file_id);
		if(!$obj) return false;

		$new_name = string_trim($new_name, 60, '...');

		$new_name = escape($new_name, true);
		$yes = query("UPDATE {$evp->options->prefix}files SET `name` = '{$new_name}' WHERE `id` = '{$obj->id}' LIMIT 1");

		return !!$yes;
	}

	public function files_delete($file_id) {
		if(empty($file_id)) return false;
		$evp = $this->evp;
		$config = Registry::retrieve('config');
		$bucket = $evp->settings->s3_bucket;
		$obj = $evp->getFile($file_id);
		if(!$obj) return false;
		if($obj->prettySource == 'local') {
			$yes = @unlink($config['paths']['base'].'/assets/videos/'.$obj->file_name);
		} elseif($obj->prettySource == 'remote' || $obj->prettySource == 'youtube') {
			// Nothing to do here
			$yes = true;
		} elseif($obj->prettySource == 's3' && $evp->s3) {
		//$yes = @$evp->s3->remote->deleteObject($bucket, $obj->file_name);
			$resp = $evp->s3->aws->s3->delete_object($bucket, $obj->file_name);
			if($resp->isOK()) $yes = true;
			$children_raw = getArray("SELECT `file_name` FROM {$evp->options->prefix}files WHERE `parent_id` = '{$obj->id}'");
			if(!empty($children_raw)) {
				foreach($children_raw as $cf) {
					$evp->s3->aws->s3->delete_object($bucket, $cf->file_name);
				//@$evp->s3->remote->deleteObject($bucket, $cf->file_name);
				}
			}
		}
		if($yes === true) {
			query("DELETE FROM {$evp->options->prefix}track_progression WHERE `file_ref` = '{$obj->id}' || `file_ref` LIKE '{$obj->id}::%'");
			query("DELETE FROM {$evp->options->prefix}track_static WHERE `file_ref` = '{$obj->id}' || `file_ref` LIKE '{$obj->id}::%'");
			query("DELETE FROM {$evp->options->prefix}track_total_completes WHERE `file_ref` = '{$obj->id}' || `file_ref` LIKE '{$obj->id}::%'");
			query("DELETE FROM {$evp->options->prefix}track_total_starts WHERE `file_ref` = '{$obj->id}' || `file_ref` LIKE '{$obj->id}::%'");
			query("DELETE FROM {$evp->options->prefix}track_unique_completes WHERE `file_ref` = '{$obj->id}' || `file_ref` LIKE '{$obj->id}::%'");
			query("DELETE FROM {$evp->options->prefix}track_unique_starts WHERE `file_ref` = '{$obj->id}' || `file_ref` LIKE '{$obj->id}::%'");
			query("DELETE FROM {$evp->options->prefix}conversions WHERE `file_id` = '{$obj->id}' || `file_id` LIKE '{$obj->id}::%'");
			query("DELETE FROM {$evp->options->prefix}files_data WHERE `file_id` = '{$obj->id}' || `file_id` LIKE '{$obj->id}::%'");
			query("DELETE FROM {$evp->options->prefix}files WHERE `id` = '{$obj->id}'");
			query("DELETE FROM {$evp->options->prefix}files WHERE `parent_id` = '{$obj->id}'");
			query("DELETE FROM {$evp->options->prefix}chats WHERE `file_ref` = '{$obj->id}' || `file_ref` LIKE '{$obj->id}::%'");
			query("DELETE FROM {$evp->options->prefix}comments WHERE `file_ref` = '{$obj->id}' || `file_ref` LIKE '{$obj->id}::%'");
			query("DELETE FROM {$evp->options->prefix}folders_relationships WHERE `file_id` = '{$obj->id}' || `file_id` LIKE '{$obj->id}::%'");
			return true;
		}
		return false;
	}

	public function files_profiles($file_id) {
		if(empty($file_id)) return false;
		$evp = $this->evp;
		return $evp->getProfiles($file_id);
	}

	public function files_profile($file_id, $profile) {
		if(empty($file_id)) return false;
		$evp = $this->evp;
		return $evp->getProfile($file_id, $profile);
	}

	public function files_embed_code($file_id, $profile) {
		if(empty($file_id)) return false;
		$evp = $this->evp;
		return $evp->videoEmbedCode($file_id, 'sd', $profile);
	}

	public function files_profile_new($file_id, $profile) {
		if(empty($file_id) || empty($profile)) return false;
		$evp = $this->evp;
		$profile_seed = makeSeed($profile);
		$data = new Object(array('full_name' => $profile));
		$file_reference = "{$file_id}::{$profile_seed}";
		$evp->insertDataKey('profile_data', $data, $file_reference, 'file');

		// Get the file
		$file = $evp->getFile($file_reference);

		$evp->insertDataKey('data[page_seed]', $file->data['page_seed'], $file_reference, 'file');
		return $file_reference;
	}

	public function files_profile_delete($file_id, $profile) {
		if(empty($file_id) || empty($profile)) return false;
		$evp = $this->evp;
		$profile_seed = makeSeed($profile);

		$old = $file_id.'::'.$profile_seed;

		$evp->deleteDataKey('profile_data', $old, 'file');

		query("DELETE FROM {$evp->options->prefix}track_progression WHERE `file_ref` = '{$old}'");
		query("DELETE FROM {$evp->options->prefix}track_static WHERE `file_ref` = '{$old}'");
		query("DELETE FROM {$evp->options->prefix}track_total_completes WHERE `file_ref` = '{$old}'");
		query("DELETE FROM {$evp->options->prefix}track_total_starts WHERE `file_ref` = '{$old}'");
		query("DELETE FROM {$evp->options->prefix}track_unique_completes WHERE `file_ref` = '{$old}'");
		query("DELETE FROM {$evp->options->prefix}track_unique_starts WHERE `file_ref` = '{$old}'");
		query("DELETE FROM {$evp->options->prefix}conversions WHERE `file_id` = '{$old}'");
		query("DELETE FROM {$evp->options->prefix}files_data WHERE `file_id` = '{$old}'");
		query("DELETE FROM {$evp->options->prefix}chats WHERE `file_ref` = '{$old}'");
		query("DELETE FROM {$evp->options->prefix}comments WHERE `file_ref` = '{$old}'");
		query("DELETE FROM {$evp->options->prefix}folders_relationships WHERE `file_id` = '{$old}'");

		return true;
	}

	public function files_profile_rename($file_id, $profile, $new_name) {
		if(empty($file_id) || empty($profile) || empty($new_name)) return false;
		$evp = $this->evp;
		$profile_seed = makeSeed($profile);
		$new_seed = makeSeed($new_name);
		$data = new Object(array('full_name' => $new_name));

		$old = $file_id.'::'.$profile_seed;
		$new = $file_id.'::'.$new_seed;

		$evp->insertDataKey('profile_data', $data, $new, 'file');
		$evp->deleteDataKey('profile_data', $old, 'file');

		query("UPDATE {$evp->options->prefix}track_progression SET `file_ref` = '{$new}' WHERE `file_ref` = '{$old}'");
		query("UPDATE {$evp->options->prefix}track_static SET `file_ref` = '{$new}' WHERE `file_ref` = '{$old}'");
		query("UPDATE {$evp->options->prefix}track_total_completes SET `file_ref` = '{$new}' WHERE `file_ref` = '{$old}'");
		query("UPDATE {$evp->options->prefix}track_total_starts SET `file_ref` = '{$new}' WHERE `file_ref` = '{$old}'");
		query("UPDATE {$evp->options->prefix}track_unique_completes SET `file_ref` = '{$new}' WHERE `file_ref` = '{$old}'");
		query("UPDATE {$evp->options->prefix}track_unique_starts SET `file_ref` = '{$new}' WHERE `file_ref` = '{$old}'");
		query("UPDATE {$evp->options->prefix}conversions SET `file_id` = '{$new}' WHERE `file_id` = '{$old}'");
		query("UPDATE {$evp->options->prefix}files_data SET `file_id` = '{$new}' WHERE `file_id` = '{$old}'");
		query("UPDATE {$evp->options->prefix}chats SET `file_ref` = '{$new}' WHERE `file_ref` = '{$old}'");
		query("UPDATE {$evp->options->prefix}comments SET `file_ref` = '{$new}' WHERE `file_ref` = '{$old}'");
		query("UPDATE {$evp->options->prefix}folders_relationships SET `file_id` = '{$new}' WHERE `file_id` = '{$old}'");

		// Get the file
		$file = $evp->getFile($new);
		$evp->insertDataKey('data[page_seed]', $file->data['page_seed'], $new, 'file');

		return $new;
	}

	public function files_profile_duplicate($file_id, $source_profile, $target_name) {
		if(empty($file_id) || empty($target_name)) return false;
		$evp = $this->evp;
		$profile_seed = (strpos($source_profile, '::') === false ? makeSeed($source_profile) : makeSeed(grab(explode('::', $source_profile), 1)));
		$new_seed = makeSeed($target_name);
		$data = new Object(array('full_name' => $target_name));

		// Let's check for the target name existing already
		$existing_profiles = $evp->getProfiles($file_id, false, true);
		if($new_seed == 'default' || $new_seed == 'default-profile' || in_array($new_seed, $existing_profiles)) return false;

		// We are fine - let's move on
		$old = $file_id.(($profile_seed == 'default' || empty($profile_seed)) ? '' : '::'.$profile_seed);
		$new = $file_id.'::'.$new_seed;

		$evp->insertDataKey('profile_data', $data, $new, 'file');
	//$evp->deleteDataKey('profile_data', $old, 'file');

		// Clone over the data
		// Note that only settings are copied and not stats, etc etc
		$original_data = getArray("SELECT `file_id`, `key`, `value` FROM {$evp->options->prefix}files_data WHERE `file_id` = '{$old}'");
		if(!empty($original_data)) {
			foreach($original_data as $od) {
				query("INSERT INTO {$evp->options->prefix}files_data (`file_id`, `key`, `value`) VALUES ('{$new}', '{$od->key}', '".$od->value."')");
			}
		}

		// Get the file
		$file = $evp->getFile($new);
		$l = strlen($file->extension);
		$page_seed = makeSeed(substr($file->file_name, 0, -$l).' '.$target_name);
		$evp->insertDataKey('data[page_seed]', $page_seed, $new, 'file'); // Refresh the page seed with new name

		return $new;
	}

	public function files_options($file_ref, $options) {
		if(empty($file_ref)) return false;
		$evp = $this->evp;
		$file = $evp->getFile($file_ref);
		if(!$file) return false;

		// We should delete the cache files
		$config = Registry::retrieve('config');
		if(file_exists($config['paths']['cacheDir'].'/cache.'.$file_ref.'.js')) unlink($config['paths']['cacheDir'].'/cache.'.$file_ref.'.js');

		$success = true;
		if(!empty($options)) {
			foreach($options as $key => $value) {
				if($key == 'events') { // This must be stored in a very flexible way
					// Let's delete all the existing events
					query("DELETE FROM {$evp->options->prefix}files_data WHERE `file_id` = '{$file_ref}' && `key` LIKE 'events[%'");
					if(!empty($value)) {
						foreach($value as $oidx => $o) {
							foreach($o as $k => $v) {
								if(is_array($v) || is_object($v)) {
									foreach($v as $s => $sv) {
										if(is_array($sv) || is_object($sv)) {
											foreach($sv as $bk => $bv) {
												if(is_array($bv) || is_object($bv)) {
													foreach($bv as $fk => $fv) {
														$success = $this->files_option($file_ref, "{$key}[{$oidx}][{$k}][{$s}][{$bk}][{$fk}]", $fv);
													}
												} else {
													$success = $this->files_option($file_ref, "{$key}[{$oidx}][{$k}][{$s}][{$bk}]", $bv);
												}
											}
										} else {
											$success = $this->files_option($file_ref, "{$key}[{$oidx}][{$k}][{$s}]", $sv);
										}
									}
								} else {
									$success = $this->files_option($file_ref, "{$key}[{$oidx}][{$k}]", $v);
								}
							}
						}
					}
				} elseif(in_array($key, array('sales_track_product_upsell', 'sales_track_product_upsell_price', 'widgets', 'plugins', 'page_widgets'))) { // Store in an alternative way
					if(!empty($value)) {
						foreach($value as $oidx => $o) {
							foreach($o as $k => $v) {
								$success = $this->files_option($file_ref, "{$key}[{$oidx}][{$k}]", $v);
							}
						}
					}
				} else {
					$success = $this->files_option($file_ref, "options[{$key}]", $value);
				}
			}
		}

		return $success;
	}

	public function files_option($file_ref, $key, $value = '') {
		if(empty($file_ref)) return false;
		$evp = $this->evp;
		// This is removed because it means we can store things like playlist::5 etc in the same place
	//$file = $evp->getFile($file_ref);
	//if(!$file) return false;
		$evp->insertDataKey($key, $value, $file_ref, 'file');
		return true;
	}

	// $relevant is array('options', 'widgets'); - the values you ARE setting. If empty, ALL WILL BE SET
	public function files_setdata($file_ref, $data, $relevant = array()) { // $data is array('options' => array('auto_play => true), 'chapters' => array('test' => array('start' => '30'))) etc etc
		if(empty($file_ref)) return false;
		$evp = $this->evp;
		$file = $evp->getFile($file_ref);
		if(!$file) return false;

		// We should delete the cache files
		$config = Registry::retrieve('config');
		if(file_exists($config['paths']['cacheDir'].'/cache.'.$file_ref.'.js')) @unlink($config['paths']['cacheDir'].'/cache.'.$file_ref.'.js');

		// Delete the existing values
		if(empty($relevant) || in_array('options', $relevant)) $evp->deleteDataKey('options', $file_ref, 'file', true);
		if(empty($relevant) || in_array('events', $relevant)) $evp->deleteDataKey('events', $file_ref, 'file', true);
		if(empty($relevant) || in_array('chapters', $relevant)) $evp->deleteDataKey('chapters', $file_ref, 'file', true);
		if(empty($relevant) || in_array('surveys', $relevant)) $evp->deleteDataKey('surveys', $file_ref, 'file', true);
		if(empty($relevant) || in_array('widgets', $relevant)) $evp->deleteDataKey('widgets', $file_ref, 'file', true);
		if(empty($relevant) || in_array('page_widgets', $relevant)) $evp->deleteDataKey('page_widgets', $file_ref, 'file', true);
		if(empty($relevant) || in_array('plugins', $relevant)) $evp->deleteDataKey('plugins', $file_ref, 'file', true);

		$ssn = ssn_encode($data);
		foreach($ssn as $k => $v) $evp->insertDataKey($k, $v, $file_ref, 'file');

		return true;
	}

	public function folders_delete($folder_id) {
		if(empty($folder_id)) return false;
		$evp = $this->evp;
		$config = Registry::retrieve('config');
		$bucket = $evp->settings->s3_bucket;
		$obj = $evp->getFolder($folder_id);
		if(!$obj) return false;
		query("DELETE FROM {$evp->options->prefix}folders WHERE `id` = '{$obj->id}'");
		query("UPDATE {$evp->options->prefix}folders SET `parent_id` = NULL WHERE `parent_id` = '{$obj->id}'");
		query("DELETE FROM {$evp->options->prefix}folders_relationships WHERE `folder_id` = '{$obj->id}'");
		return true;
	}

	public function folders_create($name, $parent_id = false) {
		if(empty($name)) return false;
		$evp = $this->evp;

		$name = escape($name, true);
		$parent_id = escape($parent_id, true);

		if(!empty($parent_id)) $parent_id = "'{$parent_id}'";
		else $parent_id = 'NULL';

		if(query("INSERT INTO {$evp->options->prefix}folders (`parent_id`, `name`) VALUES ({$parent_id}, '{$name}')")) {
			$id = getInsertId();
			return $id;
		}
		return false;
	}

	public function folders_rename($folder_id, $new_name) {
		if(empty($folder_id)) return false;
		$evp = $this->evp;
		$config = Registry::retrieve('config');
		$bucket = $evp->settings->s3_bucket;
		$obj = $evp->getFolder($folder_id);
		if(!$obj) return false;
		$new_name = escape(trim($new_name), true);
		query("UPDATE {$evp->options->prefix}folders SET `name` = '{$new_name}' WHERE `id` = '{$obj->id}'");
		return true;
	}

	public function folders_set_parent($folder_id, $parent_id = false) {
		if(empty($folder_id)) return false;
		$evp = $this->evp;
		$config = Registry::retrieve('config');
		$obj = $evp->getFolder($folder_id);
		if(!$obj) return false;

		if(empty($parent_id)) {
			query("UPDATE {$evp->options->prefix}folders SET `parent_id` = NULL WHERE `id` = '{$obj->id}'");
		} else {
			$parent_id = escape($parent_id, true);
			query("UPDATE {$evp->options->prefix}folders SET `parent_id` = '{$parent_id}' WHERE `id` = '{$obj->id}'");
		}

		return true;
	}

	public function split_delete($id) {
		if(empty($id)) return false;
		$evp = $this->evp;
		$config = Registry::retrieve('config');
		query("DELETE FROM {$evp->options->prefix}split_tests WHERE `id` = '{$id}'");
		return true;
	}

	public function split_embed_code($id) {
		if(empty($id)) return false;
		$evp = $this->evp;
		return $evp->splitEmbedCode($id);
	}

	public function users_delete($user_id) {
		if(empty($user_id)) return false;
		$evp = $this->evp;
		$config = Registry::retrieve('config');
		query("DELETE FROM {$evp->options->prefix}users WHERE `id` = '{$user_id}'");
		query("DELETE FROM {$evp->options->prefix}users_data WHERE `user_id` = '{$user_id}'");
		query("DELETE FROM {$evp->options->prefix}users_relationships WHERE `user_id` = '{$user_id}'");
		return true;
	}

	public function sitemaps_delete($sitemap_id) {
		if(empty($sitemap_id)) return false;
		$evp = $this->evp;
		$obj = $evp->getSitemap($sitemap_id);
		if(!$obj) return false;
		query("DELETE FROM {$evp->options->prefix}sitemaps WHERE `id` = '{$obj->id}'");
		query("DELETE FROM {$evp->options->prefix}sitemaps_data WHERE `sitemap_id` = '{$obj->id}'");
		return true;
	}

	public function sitemaps_xml($sitemap, $partial = false) {
		if(!is_object($sitemap)) $sitemap = $this->evp->getSitemap($sitemap);

		$config = Registry::retrieve('config');

	 $map = ($partial == true ? '' : "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<urlset xmlns=\"http://www.sitemaps.org/schemas/sitemap/0.9\" xmlns:video=\"http://www.google.com/schemas/sitemap-video/1.1\">\n");

		if($partial !== true && ('http://'.$sitemap->domain.'/' == currentDomain())) {
			$map = "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\n<?xml-stylesheet type=\"text/xsl\" href=\"{$config['locations']['absolute']}themes/3.0/sitemap.xsl\"?>\n".$map;
		}

  foreach($sitemap->contents as $file_ref => $file_data) {
		$file = $this->evp->getFile($file_ref);
		$settings = $this->evp->collateFileSettings($file);

		$loc = htmlentities($file->location);//str_replace(array('&'), array('&amp;'), $file->location);

		$map.= "\t<url>\n";
		$map.= "\t\t<loc>{$sitemap->domain}{$file_data->url}</loc>\n";
		$map.= "\t\t<video:video>\n";

		// If there's a splash image, let's use that instead
		if(!empty($settings->options['related_image'])) {
			$thumb = $settings->options['related_image'];
			$thumb_base = basename($thumb);
			$thumb = str_replace($thumb_base, '', $thumb);
			$thumb = $thumb.urlencode($thumb_base);
			$map.= "\t\t\t<video:thumbnail_loc>".$thumb."</video:thumbnail_loc>\n";
		} elseif(!empty($file->thumbnail_count)) {
			$img = str_replace('.'.$file->extension, '', $file->raw_location).'-thumbnail-0.jpg';
			$map.= "\t\t\t<video:thumbnail_loc>{$img}</video:thumbnail_loc>\n";
		}

		$map.= "\t\t\t<video:title>".stripslashes($file_data->title)."</video:title>\n";
		$map.= "\t\t\t<video:description>".stripslashes($file_data->description)."</video:description>\n";
		$map.= "\t\t\t<video:content_loc>{$loc}</video:content_loc>\n";
		$map.= "\t\t\t<video:player_loc>".htmlentities(stripslashes($this->evp->videoEmbedCode($file, 'sd', false, false, 'url')))."</video:player_loc>\n";
		if(!empty($file->metadata) && !empty($file->metadata['duration']) && $file->metadata['duration'] > 0) {
			$map.= "\t\t\t<video:duration>".ceil($file->metadata['duration'])."</video:duration>\n";
		}
		$map.= "\t\t</video:video>\n";
		$map.= "\t</url>\n";
  }

  $map.= ($partial == true ? '' : "</urlset>");

  return $map;
	}

	public function playlists_delete($id) {
		if(empty($id)) return false;
		$evp = $this->evp;
		$config = Registry::retrieve('config');
		query("DELETE FROM {$evp->options->prefix}playlists WHERE `id` = '{$id}'");
		query("DELETE FROM {$evp->options->prefix}files_data WHERE `id` = 'playlist::{$id}'");
		return true;
	}

	public function playlist_embed_code($id) {
		if(empty($id)) return false;
		$evp = $this->evp;
		return $evp->playlistEmbedCode($id);
	}
}
?>
